@@ -24,6 +24,7 @@
 #include "epg.h"
 #include "imagecache.h"
 #include "dvr/dvr.h"
+#include "lang_codes.h"
 
 static htsmsg_t *
 api_epg_get_list ( const char *s )
@@ -52,10 +53,10 @@ static void
 api_epg_add_channel ( htsmsg_t *m, channel_t *ch )
 {
   int64_t chnum;
-  char buf[32];
+  char buf[32], ubuf[UUID_HEX_SIZE];
   const char *s;
   htsmsg_add_str(m, "channelName", channel_get_name(ch));
-  htsmsg_add_str(m, "channelUuid", channel_get_uuid(ch));
+  htsmsg_add_str(m, "channelUuid", channel_get_uuid(ch, ubuf));
   if ((chnum = channel_get_number(ch)) >= 0) {
     uint32_t maj = chnum / CHANNEL_SPLIT;
     uint32_t min = chnum % CHANNEL_SPLIT;
@@ -80,6 +81,7 @@ api_epg_entry ( epg_broadcast_t *eb, const char *lang, access_t *perm )
   epg_episode_num_t epnum;
   epg_genre_t *eg;
   dvr_entry_t *de;
+  char ubuf[UUID_HEX_SIZE];
 
   if (!ee || !ch) return NULL;
 
@@ -115,6 +117,25 @@ api_epg_entry ( epg_broadcast_t *eb, const char *lang, access_t *perm )
   if ((s = epg_broadcast_get_description(eb, lang)))
     htsmsg_add_str(m, "description", s);
 
+  if (eb->is_new)
+    htsmsg_add_u32(m, "new", eb->is_new);
+  if (eb->is_repeat)
+    htsmsg_add_u32(m, "repeat", eb->is_repeat);
+  if (eb->is_widescreen)
+    htsmsg_add_u32(m, "widescreen", eb->is_widescreen);
+  if (eb->is_deafsigned)
+    htsmsg_add_u32(m, "deafsigned", eb->is_deafsigned);
+  if (eb->is_subtitled)
+    htsmsg_add_u32(m, "subtitled", eb->is_subtitled);
+  if (eb->is_audio_desc)
+    htsmsg_add_u32(m, "audiodesc", eb->is_audio_desc);
+  if (eb->is_hd)
+    htsmsg_add_u32(m, "hd", eb->is_hd);
+  if (eb->lines)
+    htsmsg_add_u32(m, "lines", eb->lines);
+  if (eb->aspect)
+    htsmsg_add_u32(m, "aspect", eb->aspect);
+
   /* Episode info */
   if (ee) {
     
@@ -163,12 +184,18 @@ api_epg_entry ( epg_broadcast_t *eb, const char *lang, access_t *perm )
   }
 
   /* Recording */
-  if (!access_verify2(perm, ACCESS_RECORDER) &&
-      (de = dvr_entry_find_by_event(eb)) &&
-      !access_verify_list(perm->aa_dvrcfgs,
-                          idnode_uuid_as_str(&de->de_config->dvr_id))) {
-    htsmsg_add_str(m, "dvrUuid", idnode_uuid_as_str(&de->de_id));
-    htsmsg_add_str(m, "dvrState", dvr_entry_schedstatus(de));
+  if (eb->channel && !access_verify2(perm, ACCESS_RECORDER)) {
+    /* Note: only first hit is matched */
+    LIST_FOREACH(de, &eb->channel->ch_dvrs, de_channel_link) {
+      if (de->de_bcast != eb)
+        continue;
+      if (access_verify_list(perm->aa_dvrcfgs,
+                             idnode_uuid_as_str(&de->de_config->dvr_id, ubuf)))
+        continue;
+      htsmsg_add_str(m, "dvrUuid", idnode_uuid_as_str(&de->de_id, ubuf));
+      htsmsg_add_str(m, "dvrState", dvr_entry_schedstatus(de));
+      break;
+    }
   }
 
   /* Next event */
@@ -280,17 +307,20 @@ api_epg_grid
 {
   int i;
   epg_query_t eq;
-  const char *lang, *str;
+  const char *str;
+  char *lang;
   uint32_t start, limit, end, genre;
   int64_t duration_min, duration_max;
   htsmsg_field_t *f, *f2;
   htsmsg_t *l = NULL, *e, *filter;
+  const char* mode;
 
   memset(&eq, 0, sizeof(eq));
 
-  lang = htsmsg_get_str(args, "lang");
+  lang = access_get_lang(perm, htsmsg_get_str(args, "lang"));
   if (lang)
     eq.lang = strdup(lang);
+  mode = htsmsg_get_str(args, "mode");
   str = htsmsg_get_str(args, "title");
   if (str)
     eq.stitle = strdup(str);
@@ -302,6 +332,14 @@ api_epg_grid
   if (str)
     eq.channel_tag = strdup(str);
 
+  if (mode != NULL) {
+      if (!strcmp(mode, "now")) {
+        eq.start.comp = EC_LT;
+        eq.stop.comp = EC_GT;
+        eq.start.val1 = eq.stop.val1 = gclk();
+      }
+  }
+
   duration_min = -1;
   duration_max = -1;
   htsmsg_get_s64(args, "durationMin", &duration_min);
@@ -436,6 +474,7 @@ api_epg_grid
   pthread_mutex_unlock(&global_lock);
 
   epg_query_free(&eq);
+  free(lang);
 
   /* Build response */
   *resp = htsmsg_create_map();
@@ -471,17 +510,19 @@ api_epg_alternative
   uint32_t id, entries = 0;
   htsmsg_t *l = htsmsg_create_list();
   epg_broadcast_t *e;
-  const char *lang = htsmsg_get_str(args, "lang");
+  char *lang;
 
-  if (!htsmsg_get_u32(args, "eventId", &id))
+  if (htsmsg_get_u32(args, "eventId", &id))
     return -EINVAL;
 
   /* Main Job */
+  lang = access_get_lang(perm, htsmsg_get_str(args, "lang"));
   pthread_mutex_lock(&global_lock);
   e = epg_broadcast_find_by_id(id);
   if (e && e->episode)
     api_epg_episode_broadcasts(perm, l, lang, e->episode, &entries, e);
   pthread_mutex_unlock(&global_lock);
+  free(lang);
 
   /* Build response */
   *resp = htsmsg_create_map();
@@ -499,12 +540,13 @@ api_epg_related
   htsmsg_t *l = htsmsg_create_list();
   epg_broadcast_t *e;
   epg_episode_t *ep, *ep2;
-  const char *lang = htsmsg_get_str(args, "lang");
+  char *lang;
   
-  if (!htsmsg_get_u32(args, "eventId", &id))
+  if (htsmsg_get_u32(args, "eventId", &id))
     return -EINVAL;
 
   /* Main Job */
+  lang = access_get_lang(perm, htsmsg_get_str(args, "lang"));
   pthread_mutex_lock(&global_lock);
   e = epg_broadcast_find_by_id(id);
   ep = e ? e->episode : NULL;
@@ -523,6 +565,7 @@ api_epg_related
     }
   }
   pthread_mutex_unlock(&global_lock);
+  free(lang);
 
   /* Build response */
   *resp = htsmsg_create_map();
@@ -540,7 +583,7 @@ api_epg_load
   htsmsg_t *l = htsmsg_create_list(), *ids = NULL, *m;
   htsmsg_field_t *f;
   epg_broadcast_t *e;
-  const char *lang = htsmsg_get_str(args, "lang");
+  char *lang;
 
   if (!(f = htsmsg_field_find(args, "eventId")))
     return -EINVAL;
@@ -550,6 +593,7 @@ api_epg_load
 
   /* Main Job */
   pthread_mutex_lock(&global_lock);
+  lang = access_get_lang(perm, htsmsg_get_str(args, "lang"));
   if (ids) {
     HTSMSG_FOREACH(f, ids) {
       if (htsmsg_field_get_u32(f, &id)) continue;
@@ -567,6 +611,7 @@ api_epg_load
     }
   }
   pthread_mutex_unlock(&global_lock);
+  free(lang);
 
   /* Build response */
   *resp = htsmsg_create_map();
@@ -600,7 +645,7 @@ api_epg_content_type_list(access_t *perm, void *opaque, const char *op,
   htsmsg_get_bool(args, "full", &full);
 
   *resp = htsmsg_create_map();
-  array = epg_genres_list_all(full ? 0 : 1, 0);
+  array = epg_genres_list_all(full ? 0 : 1, 0, perm->aa_lang_ui);
   htsmsg_add_msg(*resp, "entries", array);
   return 0;
 }
