@@ -36,7 +36,7 @@
 
 struct dvr_timerec_entry_queue timerec_entries;
 
-static gtimer_t dvr_timerec_timer;
+static mtimer_t dvr_timerec_timer;
 
 /**
  *
@@ -75,9 +75,9 @@ dvr_timerec_purge_spawn(dvr_timerec_entry_t *dte, int delconf)
     de->de_timerec = NULL;
     if (delconf) {
       if (de->de_sched_state == DVR_SCHEDULED)
-        dvr_entry_cancel(de);
+        dvr_entry_cancel(de, 0);
       else
-        dvr_entry_save(de);
+        idnode_changed(&de->de_id);
     }
   }
 }
@@ -88,14 +88,13 @@ dvr_timerec_purge_spawn(dvr_timerec_entry_t *dte, int delconf)
 static const char *
 dvr_timerec_title(dvr_timerec_entry_t *dte, struct tm *start)
 {
-  static char buf[256];
   size_t len;
 
   if (dte->dte_title == NULL)
-    return "Unknown";
-  len = strftime(buf, sizeof(buf) - 1, dte->dte_title, start);
-  buf[len] = '\0';
-  return buf;
+    return _("Unknown");
+  len = strftime(prop_sbuf, PROP_SBUF_LEN-1, dte->dte_title, start);
+  prop_sbuf[len] = '\0';
+  return prop_sbuf;
 }
 
 /**
@@ -109,6 +108,7 @@ dvr_timerec_check(dvr_timerec_entry_t *dte)
   struct tm tm_start, tm_stop;
   const char *title;
   char buf[200];
+  char ubuf[UUID_HEX_SIZE];
 
   if(dte->dte_enabled == 0 || dte->dte_weekdays == 0)
     goto fail;
@@ -118,7 +118,7 @@ dvr_timerec_check(dvr_timerec_entry_t *dte)
   if(dte->dte_channel == NULL)
     goto fail;
 
-  clk   = dispatch_clock;
+  clk   = gclk();
   limit = clk - 600;
   start = dvr_timerec_timecorrection(clk, dte->dte_start, &tm_start);
   stop  = dvr_timerec_timecorrection(clk, dte->dte_stop,  &tm_stop);
@@ -126,12 +126,12 @@ dvr_timerec_check(dvr_timerec_entry_t *dte)
     /* next day */
     clk += 24*60*60;
     start = dvr_timerec_timecorrection(clk, dte->dte_start, &tm_start);
-    stop  = dvr_timerec_timecorrection(clk, dte->dte_stop,  &tm_stop);
+    stop  = dvr_timerec_timecorrection(clk, dte->dte_stop, &tm_stop);
   }
   /* day boundary correction */
   while (start > stop) {
     clk += 24*60*60;
-    stop  = dvr_timerec_timecorrection(clk, dte->dte_stop, &tm_stop);
+    stop = dvr_timerec_timecorrection(clk, dte->dte_stop, &tm_stop);
   }
 
   if(dte->dte_weekdays != 0x7f) {
@@ -149,15 +149,15 @@ dvr_timerec_check(dvr_timerec_entry_t *dte)
   }
 
   title = dvr_timerec_title(dte, &tm_start);
-  snprintf(buf, sizeof(buf), "Time recording%s%s",
-           dte->dte_creator ? " by: " : "",
-           dte->dte_creator ?: "");
-  de = dvr_entry_create_(idnode_uuid_as_str(&dte->dte_config->dvr_id),
+  snprintf(buf, sizeof(buf), _("Time recording%s%s"),
+           dte->dte_comment ? ": " : "",
+           dte->dte_comment ?: "");
+  de = dvr_entry_create_(1, idnode_uuid_as_str(&dte->dte_config->dvr_id, ubuf),
                          NULL, dte->dte_channel,
                          start, stop, 0, 0, title, NULL,
-                         NULL, NULL, NULL, dte->dte_owner, buf,
+                         NULL, NULL, NULL, dte->dte_owner, dte->dte_creator,
                          NULL, dte, dte->dte_pri, dte->dte_retention,
-                         dte->dte_comment);
+                         dte->dte_removal, buf);
 
   return;
 
@@ -177,7 +177,7 @@ dvr_timerec_create(const char *uuid, htsmsg_t *conf)
 
   if (idnode_insert(&dte->dte_id, uuid, &dvr_timerec_entry_class, 0)) {
     if (uuid)
-      tvhwarn("dvr", "invalid timerec entry uuid '%s'", uuid);
+      tvhwarn(LS_DVR, "invalid timerec entry uuid '%s'", uuid);
     free(dte);
     return NULL;
   }
@@ -187,6 +187,7 @@ dvr_timerec_create(const char *uuid, htsmsg_t *conf)
   dte->dte_pri = DVR_PRIO_NORMAL;
   dte->dte_start = -1;
   dte->dte_stop = -1;
+  dte->dte_enabled = 1;
   dte->dte_config = dvr_config_find_by_name_default(NULL);
   LIST_INSERT_HEAD(&dte->dte_config->dvr_timerec_entries, dte, dte_config_link);
 
@@ -200,63 +201,42 @@ dvr_timerec_create(const char *uuid, htsmsg_t *conf)
 }
 
 dvr_timerec_entry_t*
-dvr_timerec_create_htsp(const char *dvr_config_name, const char *title,
-                            channel_t *ch, uint32_t enabled, uint32_t start, uint32_t stop,
-                            uint32_t weekdays, dvr_prio_t pri, int retention,
-                            const char *owner, const char *creator, const char *comment, 
-                            const char *name, const char *directory)
+dvr_timerec_create_htsp(htsmsg_t *conf)
 {
   dvr_timerec_entry_t *dte;
-  htsmsg_t *conf, *days;
-
-  conf = htsmsg_create_map();
-  days = htsmsg_create_list();
-
-  htsmsg_add_u32(conf, "enabled",     enabled > 0 ? 1 : 0);
-  htsmsg_add_u32(conf, "retention",   retention);
-  htsmsg_add_u32(conf, "pri",         pri);
-  htsmsg_add_str(conf, "title",       title);
-  htsmsg_add_str(conf, "config_name", dvr_config_name ?: "");
-  htsmsg_add_str(conf, "owner",       owner ?: "");
-  htsmsg_add_str(conf, "creator",     creator ?: "");
-  htsmsg_add_str(conf, "comment",     comment ?: "");
-  htsmsg_add_str(conf, "name",        name ?: "");
-  htsmsg_add_str(conf, "directory",   directory ?: "");
-  htsmsg_add_u32(conf, "start",       start);
-  htsmsg_add_u32(conf, "stop",        stop);
-
-  if (ch)
-    htsmsg_add_str(conf, "channel", idnode_uuid_as_str(&ch->ch_id));
-
-  int i;
-  for (i = 0; i < 7; i++)
-    if (weekdays & (1 << i))
-      htsmsg_add_u32(days, NULL, i + 1);
-
-  htsmsg_add_msg(conf, "weekdays", days);
 
   dte = dvr_timerec_create(NULL, conf);
   htsmsg_destroy(conf);
 
   if (dte)
-  {
-    dvr_timerec_save(dte);
-    dvr_timerec_check(dte);
-  }
+    idnode_changed(&dte->dte_id);
 
   return dte;
 }
 
+void
+dvr_timerec_update_htsp (dvr_timerec_entry_t *dte, htsmsg_t *conf)
+{
+  idnode_update(&dte->dte_id, conf);
+  idnode_changed(&dte->dte_id);
+  tvhinfo(LS_DVR, "timerec \"%s\" on \"%s\": Updated", dte->dte_title ? dte->dte_title : "",
+      (dte->dte_channel && dte->dte_channel->ch_name) ? dte->dte_channel->ch_name : "any channel");
+}
+
 /**
  *
  */
 static void
 timerec_entry_destroy(dvr_timerec_entry_t *dte, int delconf)
 {
+  char ubuf[UUID_HEX_SIZE];
+
+  idnode_save_check(&dte->dte_id, delconf);
+
   dvr_timerec_purge_spawn(dte, delconf);
 
   if (delconf)
-    hts_settings_remove("dvr/timerec/%s", idnode_uuid_as_str(&dte->dte_id));
+    hts_settings_remove("dvr/timerec/%s", idnode_uuid_as_str(&dte->dte_id, ubuf));
 
   htsp_timerec_entry_delete(dte);
 
@@ -279,34 +259,29 @@ timerec_entry_destroy(dvr_timerec_entry_t *dte, int delconf)
   free(dte);
 }
 
-/**
- *
- */
-void
-dvr_timerec_save(dvr_timerec_entry_t *dte)
-{
-  htsmsg_t *m = htsmsg_create_map();
-
-  lock_assert(&global_lock);
-
-  idnode_save(&dte->dte_id, m);
-  hts_settings_save(m, "dvr/timerec/%s", idnode_uuid_as_str(&dte->dte_id));
-  htsmsg_destroy(m);
-}
-
 /* **************************************************************************
  * DVR Autorec Entry Class definition
  * **************************************************************************/
 
 static void
-dvr_timerec_entry_class_save(idnode_t *self)
+dvr_timerec_entry_class_changed(idnode_t *self)
 {
   dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)self;
-  dvr_timerec_save(dte);
   dvr_timerec_check(dte);
   htsp_timerec_entry_update(dte);
 }
 
+static htsmsg_t *
+dvr_timerec_entry_class_save(idnode_t *self, char *filename, size_t fsize)
+{
+  dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)self;
+  htsmsg_t *m = htsmsg_create_map();
+  char ubuf[UUID_HEX_SIZE];
+  idnode_save(&dte->dte_id, m);
+  snprintf(filename, fsize, "dvr/timerec/%s", idnode_uuid_as_str(&dte->dte_id, ubuf));
+  return m;
+}
+
 static void
 dvr_timerec_entry_class_delete(idnode_t *self)
 {
@@ -328,7 +303,7 @@ dvr_timerec_entry_class_perm(idnode_t *self, access_t *a, htsmsg_t *msg_to_write
 }
 
 static const char *
-dvr_timerec_entry_class_get_title (idnode_t *self)
+dvr_timerec_entry_class_get_title (idnode_t *self, const char *lang)
 {
   dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)self;
   const char *s = "";
@@ -352,6 +327,9 @@ dvr_timerec_entry_class_channel_set(void *o, const void *v)
       return 1;
     }
   } else if (dte->dte_channel != ch) {
+    if (dte->dte_id.in_access &&
+        !channel_access(ch, dte->dte_id.in_access, 1))
+      return 0;
     if (dte->dte_channel)
       LIST_REMOVE(dte, dte_channel_link);
     dte->dte_channel = ch;
@@ -364,17 +342,16 @@ dvr_timerec_entry_class_channel_set(void *o, const void *v)
 static const void *
 dvr_timerec_entry_class_channel_get(void *o)
 {
-  static const char *ret;
   dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)o;
   if (dte->dte_channel)
-    ret = idnode_uuid_as_str(&dte->dte_channel->ch_id);
+    idnode_uuid_as_str(&dte->dte_channel->ch_id, prop_sbuf);
   else
-    ret = "";
-  return &ret;
+    prop_sbuf[0] = '\0';
+  return &prop_sbuf_ptr;
 }
 
 static char *
-dvr_timerec_entry_class_channel_rend(void *o)
+dvr_timerec_entry_class_channel_rend(void *o, const char *lang)
 {
   dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)o;
   if (dte->dte_channel)
@@ -422,14 +399,11 @@ dvr_timerec_entry_class_stop_set(void *o, const void *v)
 static const void *
 dvr_timerec_entry_class_time_get(void *o, int tm)
 {
-  static const char *ret;
-  static char buf[16];
   if (tm >= 0)
-    snprintf(buf, sizeof(buf), "%02d:%02d", tm / 60, tm % 60);
+    snprintf(prop_sbuf, PROP_SBUF_LEN, "%02d:%02d", tm / 60, tm % 60);
   else
-    strcpy(buf, "Any");
-  ret = buf;
-  return &ret;
+    strlcpy(prop_sbuf, N_("Any"), PROP_SBUF_LEN);
+  return &prop_sbuf_ptr;
 }
 
 static const void *
@@ -447,9 +421,9 @@ dvr_timerec_entry_class_stop_get(void *o)
 }
 
 static htsmsg_t *
-dvr_timerec_entry_class_time_list(void *o)
+dvr_timerec_entry_class_time_list(void *o, const char *lang)
 {
-  return dvr_autorec_entry_class_time_list(o, "Invalid");
+  return dvr_autorec_entry_class_time_list(o, tvh_gettext_lang(lang, N_("Invalid")));
 }
 
 static int
@@ -475,17 +449,16 @@ dvr_timerec_entry_class_config_name_set(void *o, const void *v)
 static const void *
 dvr_timerec_entry_class_config_name_get(void *o)
 {
-  static const char *buf;
   dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)o;
   if (dte->dte_config)
-    buf = idnode_uuid_as_str(&dte->dte_config->dvr_id);
+    idnode_uuid_as_str(&dte->dte_config->dvr_id, prop_sbuf);
   else
-    buf = "";
-  return &buf;
+    prop_sbuf[0] = '\0';
+  return &prop_sbuf_ptr;
 }
 
 static char *
-dvr_timerec_entry_class_config_name_rend(void *o)
+dvr_timerec_entry_class_config_name_rend(void *o, const char *lang)
 {
   dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)o;
   if (dte->dte_config)
@@ -525,16 +498,31 @@ dvr_timerec_entry_class_weekdays_default(void)
 }
 
 static char *
-dvr_timerec_entry_class_weekdays_rend(void *o)
+dvr_timerec_entry_class_weekdays_rend(void *o, const char *lang)
 {
   dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)o;
-  return dvr_autorec_entry_class_weekdays_rend(dte->dte_weekdays);
+  return dvr_autorec_entry_class_weekdays_rend(dte->dte_weekdays, lang);
 }
 
+static uint32_t
+dvr_timerec_entry_class_owner_opts(void *o)
+{
+  dvr_timerec_entry_t *dte = (dvr_timerec_entry_t *)o;
+  if (dte && dte->dte_id.in_access &&
+      !access_verify2(dte->dte_id.in_access, ACCESS_ADMIN))
+    return PO_ADVANCED;
+  return PO_RDONLY | PO_ADVANCED;
+}
+
+CLASS_DOC(dvrtimerec)
+PROP_DOC(dvr_timerec_title_format)
+
 const idclass_t dvr_timerec_entry_class = {
   .ic_class      = "dvrtimerec",
-  .ic_caption    = "DVR Time-Record Entry",
+  .ic_caption    = N_("DVR - Time-based Recording (Timers)"),
   .ic_event      = "dvrtimerec",
+  .ic_doc        = tvh_doc_dvrtimerec_class,
+  .ic_changed    = dvr_timerec_entry_class_changed,
   .ic_save       = dvr_timerec_entry_class_save,
   .ic_get_title  = dvr_timerec_entry_class_get_title,
   .ic_delete     = dvr_timerec_entry_class_delete,
@@ -543,32 +531,44 @@ const idclass_t dvr_timerec_entry_class = {
     {
       .type     = PT_BOOL,
       .id       = "enabled",
-      .name     = "Enabled",
+      .name     = N_("Enabled"),
+      .desc     = N_("Enable/disable the entry."),
+      .def.i    = 1,
       .off      = offsetof(dvr_timerec_entry_t, dte_enabled),
     },
     {
       .type     = PT_STR,
       .id       = "name",
-      .name     = "Name",
+      .name     = N_("Name"),
+      .desc     = N_("Name of the entry."),
       .off      = offsetof(dvr_timerec_entry_t, dte_name),
     },
     {
       .type     = PT_STR,
       .id       = "title",
-      .name     = "Title",
+      .name     = N_("Title"),
+      .desc     = N_("Title of the recording - this is used to generate the filename."),
+      .doc      = prop_doc_dvr_timerec_title_format,
       .off      = offsetof(dvr_timerec_entry_t, dte_title),
       .def.s    = "Time-%F_%R",
+      .opts     = PO_ADVANCED
     },
     {
       .type     = PT_STR,
       .id       = "directory",
-      .name     = "Directory",
+      .name     = N_("Directory"),
+      .desc     = N_("Directory override. Override the subdirectory "
+                     "rules specified by the DVR configuration and put "
+                     "all recordings done by this entry into the "
+                     "specified subdirectory"),
       .off      = offsetof(dvr_timerec_entry_t, dte_directory),
+      .opts     = PO_EXPERT
     },
     {
       .type     = PT_STR,
       .id       = "channel",
-      .name     = "Channel",
+      .name     = N_("Channel"),
+      .desc     = N_("Channel to use/used for the recording."),
       .set      = dvr_timerec_entry_class_channel_set,
       .get      = dvr_timerec_entry_class_channel_get,
       .rend     = dvr_timerec_entry_class_channel_rend,
@@ -577,76 +577,102 @@ const idclass_t dvr_timerec_entry_class = {
     {
       .type     = PT_STR,
       .id       = "start",
-      .name     = "Start",
+      .name     = N_("Start"),
+      .desc     = N_("Time to start the recording/time the recording started."),
       .set      = dvr_timerec_entry_class_start_set,
       .get      = dvr_timerec_entry_class_start_get,
       .list     = dvr_timerec_entry_class_time_list,
       .def.s    = "12:00",
-      .opts     = PO_SORTKEY,
+      .opts     = PO_SORTKEY | PO_DOC_NLIST,
     },
     {
       .type     = PT_STR,
       .id       = "stop",
-      .name     = "Stop",
+      .name     = N_("Stop"),
+      .desc     = N_("Time to stop recording/time the recording stopped."),
       .set      = dvr_timerec_entry_class_stop_set,
       .get      = dvr_timerec_entry_class_stop_get,
       .list     = dvr_timerec_entry_class_time_list,
       .def.s    = "12:00",
-      .opts     = PO_SORTKEY,
+      .opts     = PO_SORTKEY | PO_DOC_NLIST,
     },
     {
       .type     = PT_U32,
       .islist   = 1,
       .id       = "weekdays",
-      .name     = "Week Days",
+      .name     = N_("Days of Week"),
+      .desc     = N_("Record on these days only."),
       .set      = dvr_timerec_entry_class_weekdays_set,
       .get      = dvr_timerec_entry_class_weekdays_get,
       .list     = dvr_autorec_entry_class_weekdays_list,
       .rend     = dvr_timerec_entry_class_weekdays_rend,
-      .def.list = dvr_timerec_entry_class_weekdays_default
+      .def.list = dvr_timerec_entry_class_weekdays_default,
+      .opts     = PO_DOC_NLIST,
     },
     {
       .type     = PT_U32,
       .id       = "pri",
-      .name     = "Priority",
+      .name     = N_("Priority"),
+      .desc     = N_("Priority of the entry, higher-priority entries will take precedence and cancel lower-priority events."),
       .list     = dvr_entry_class_pri_list,
       .def.i    = DVR_PRIO_NORMAL,
       .off      = offsetof(dvr_timerec_entry_t, dte_pri),
-      .opts     = PO_SORTKEY,
+      .opts     = PO_SORTKEY | PO_ADVANCED | PO_DOC_NLIST,
     },
     {
-      .type     = PT_INT,
+      .type     = PT_U32,
       .id       = "retention",
-      .name     = "Retention",
+      .name     = N_("DVR log retention"),
+      .desc     = N_("Number of days to retain entry information."),
+      .def.i    = DVR_RET_REM_DVRCONFIG,
       .off      = offsetof(dvr_timerec_entry_t, dte_retention),
+      .list     = dvr_entry_class_retention_list,
+      .opts     = PO_EXPERT | PO_DOC_NLIST,
+    },
+    {
+      .type     = PT_U32,
+      .id       = "removal",
+      .name     = N_("DVR file retention period"),
+      .desc     = N_("Number of days to keep the recorded file."),
+      .def.i    = DVR_RET_REM_DVRCONFIG,
+      .off      = offsetof(dvr_timerec_entry_t, dte_removal),
+      .list     = dvr_entry_class_removal_list,
+      .opts     = PO_ADVANCED | PO_DOC_NLIST,
     },
     {
       .type     = PT_STR,
       .id       = "config_name",
-      .name     = "DVR Configuration",
+      .name     = N_("DVR configuration"),
+      .desc     = N_("DVR profile to use/used for the recording."),
       .set      = dvr_timerec_entry_class_config_name_set,
       .get      = dvr_timerec_entry_class_config_name_get,
       .rend     = dvr_timerec_entry_class_config_name_rend,
       .list     = dvr_entry_class_config_name_list,
+      .opts     = PO_ADVANCED
     },
     {
       .type     = PT_STR,
       .id       = "owner",
-      .name     = "Owner",
+      .name     = N_("Owner"),
+      .desc     = N_("Owner of the entry."),
       .off      = offsetof(dvr_timerec_entry_t, dte_owner),
-      .opts     = PO_RDONLY,
+      .get_opts = dvr_timerec_entry_class_owner_opts,
     },
     {
       .type     = PT_STR,
       .id       = "creator",
-      .name     = "Creator",
+      .name     = N_("Creator"),
+      .desc     = N_("The user who created the recording, or the "
+                     "auto-recording source and IP address if scheduled "
+                     "by a matching rule."),
       .off      = offsetof(dvr_timerec_entry_t, dte_creator),
-      .opts     = PO_RDONLY,
+      .get_opts = dvr_timerec_entry_class_owner_opts,
     },
     {
       .type     = PT_STR,
       .id       = "comment",
-      .name     = "Comment",
+      .name     = N_("Comment"),
+      .desc     = N_("Free-form text field, enter whatever you like here."),
       .off      = offsetof(dvr_timerec_entry_t, dte_comment),
     },
     {}
@@ -663,6 +689,7 @@ dvr_timerec_init(void)
   htsmsg_field_t *f;
 
   TAILQ_INIT(&timerec_entries);
+  idclass_register(&dvr_timerec_entry_class);
   if((l = hts_settings_load("dvr/timerec")) != NULL) {
     HTSMSG_FOREACH(f, l) {
       if((c = htsmsg_get_map_by_field(f)) == NULL)
@@ -689,7 +716,7 @@ dvr_timerec_timer_cb(void *aux)
 {
   dvr_timerec_entry_t *dte;
 
-  tvhtrace("dvr", "timerec update");
+  tvhtrace(LS_DVR, "timerec update");
 
   /* check all entries */
   TAILQ_FOREACH(dte, &timerec_entries, dte_link) {
@@ -697,7 +724,7 @@ dvr_timerec_timer_cb(void *aux)
   }
 
   /* load the timer */
-  gtimer_arm(&dvr_timerec_timer, dvr_timerec_timer_cb, NULL, 3550);
+  mtimer_arm_rel(&dvr_timerec_timer, dvr_timerec_timer_cb, NULL, sec2mono(3550));
 }
 
 void
@@ -749,17 +776,36 @@ timerec_destroy_by_config(dvr_config_t *kcfg, int delconf)
       LIST_INSERT_HEAD(&cfg->dvr_timerec_entries, dte, dte_config_link);
     dte->dte_config = cfg;
     if (delconf)
-      dvr_timerec_save(dte);
+      idnode_changed(&dte->dte_id);
   }
 }
 
 /**
  *
  */
-int
-dvr_timerec_get_retention( dvr_timerec_entry_t *dte )
+uint32_t
+dvr_timerec_get_retention_days( dvr_timerec_entry_t *dte )
 {
-  if (dte->dte_retention > 0)
+  if (dte->dte_retention > 0) {
+    if (dte->dte_retention > DVR_RET_REM_FOREVER)
+      return DVR_RET_REM_FOREVER;
+      
     return dte->dte_retention;
-  return dte->dte_config->dvr_retention_days;
+  }
+  return dvr_retention_cleanup(dte->dte_config->dvr_retention_days);
+}
+
+/**
+ *
+ */
+uint32_t
+dvr_timerec_get_removal_days( dvr_timerec_entry_t *dte )
+{
+  if (dte->dte_removal > 0) {
+    if (dte->dte_removal > DVR_RET_REM_FOREVER)
+      return DVR_RET_REM_FOREVER;
+
+    return dte->dte_removal;
+  }
+  return dvr_retention_cleanup(dte->dte_config->dvr_removal_days);
 }
