@@ -26,6 +26,23 @@
 #include "epggrab/private.h"
 #include "input.h"
 #include "input/mpegts/dvb_charset.h"
+#include "dvr/dvr.h"
+
+/* ************************************************************************
+ * Opaque
+ * ***********************************************************************/
+
+typedef struct eit_private
+{
+  uint16_t pid;
+  uint8_t  conv;
+  uint8_t  spec;
+} eit_private_t;
+
+#define EIT_CONV_HUFFMAN     1
+
+#define EIT_SPEC_UK_FREESAT  1
+#define EIT_SPEC_NZ_FREEVIEW 2
 
 /* ************************************************************************
  * Status handling
@@ -33,8 +50,8 @@
 
 typedef struct eit_event
 {
-  char              uri[257];
-  char              suri[257];
+  char              uri[529];
+  char              suri[529];
   
   lang_str_t       *title;
   lang_str_t       *summary;
@@ -66,12 +83,12 @@ _eit_dtag_dump
 #if APS_DEBUG
   int i = 0, j = 0;
   char tmp[100];
-  tvhlog(LOG_DEBUG, mod->id, "  dtag 0x%02X len %d", dtag, dlen);
+  tvhdebug(mod->subsys, "%s:  dtag 0x%02X len %d", mt->mt_name, dtag, dlen);
   while (i < dlen) {
     j += sprintf(tmp+j, "%02X ", buf[i]);
     i++;
     if ((i % 8) == 0 || (i == dlen)) {
-      tvhlog(LOG_DEBUG, mod->id, "    %s", tmp);
+      tvhdebug(mod->subsys, "%s:    %s", mt->mt_name, tmp);
       j = 0;
     }
   }
@@ -91,20 +108,15 @@ static dvb_string_conv_t _eit_freesat_conv[2] = {
  * Get string
  */
 static int _eit_get_string_with_len
-  ( epggrab_module_t *m,
+  ( epggrab_module_t *mod,
     char *dst, size_t dstlen, 
-		const uint8_t *src, size_t srclen, const char *charset )
+    const uint8_t *src, size_t srclen, const char *charset )
 {
+  epggrab_module_ota_t *m = (epggrab_module_ota_t *)mod;
   dvb_string_conv_t *cptr = NULL;
 
-  /* Enable huffman decode (for freeview and/or freesat) */
-  m = epggrab_module_find_by_id("uk_freesat");
-  if (m && m->enabled) {
+  if (((eit_private_t *)m->opaque)->conv == EIT_CONV_HUFFMAN)
     cptr = _eit_freesat_conv;
-  } else {
-    m = epggrab_module_find_by_id("uk_freeview");
-    if (m && m->enabled) cptr = _eit_freesat_conv;
-  }
 
   /* Convert */
   return dvb_get_string_with_len(dst, dstlen, src, srclen, charset, cptr);
@@ -281,6 +293,14 @@ static int _eit_desc_component
     /* Described */
     if (t == 0x40 || t == 0x44)
       ev->ad = 1;
+
+  /* HEVC */
+  } else if ( c == 0x9 ) {
+
+    ev->ws = 1;
+    if (t > 3)
+      ev->hd = 2;
+
   }
 
   return 0;
@@ -366,9 +386,8 @@ static int _eit_desc_crid
     
       if (crid) {
         if (strstr(buf, "crid://") == buf) {
-          strncpy(crid, buf, clen);
-          crid[clen-1] = '\0';
-        } else if ( *buf != '/' ) {
+          strlcpy(crid, buf, clen);
+        } else if (*buf != '/') {
           snprintf(crid, clen, "crid://%s", buf);
         } else {
           const char *defauth = svc->s_dvb_cridauth;
@@ -396,22 +415,22 @@ static int _eit_desc_crid
  * ***********************************************************************/
 
 static int _eit_process_event_one
-  ( epggrab_module_t *mod, int tableid,
+  ( epggrab_module_t *mod, int tableid, int sect,
     mpegts_service_t *svc, channel_t *ch,
     const uint8_t *ptr, int len,
     int local, int *resched, int *save )
 {
-  int save2 = 0;
-  int ret, dllen;
+  int dllen, save2 = 0;
   time_t start, stop;
   uint16_t eid;
-  uint8_t dtag, dlen;
+  uint8_t dtag, dlen, running;
   epg_broadcast_t *ebc;
-  epg_episode_t *ee;
+  epg_episode_t *ee = NULL;
   epg_serieslink_t *es;
+  epg_running_t run;
   eit_event_t ev;
-
-  if ( len < 12 ) return -1;
+  uint32_t changes2 = 0, changes3 = 0, changes4 = 0;
+  char tm1[32], tm2[32];
 
   /* Core fields */
   eid   = ptr[0] << 8 | ptr[1];
@@ -419,20 +438,22 @@ static int _eit_process_event_one
   stop  = start + bcdtoint(ptr[7] & 0xff) * 3600 +
                   bcdtoint(ptr[8] & 0xff) * 60 +
                   bcdtoint(ptr[9] & 0xff);
+  running = (ptr[10] >> 5) & 0x07;
   dllen = ((ptr[10] & 0x0f) << 8) | ptr[11];
 
   len -= 12;
   ptr += 12;
   if ( len < dllen ) return -1;
-  ret  = 12 + dllen;
 
   /* Find broadcast */
-  ebc  = epg_broadcast_find_by_time(ch, start, stop, eid, 1, &save2);
-  tvhtrace("eit", "svc='%s', ch='%s', eid=%5d, start=%"PRItime_t","
-                  " stop=%"PRItime_t", ebc=%p",
+  ebc  = epg_broadcast_find_by_time(ch, mod, start, stop, 1, &save2, &changes2);
+  tvhtrace(LS_TBL_EIT, "svc='%s', ch='%s', eid=%5d, tbl=%02x, running=%d, start=%s,"
+                       " stop=%s, ebc=%p",
            svc->s_dvb_svcname ?: "(null)", ch ? channel_get_name(ch) : "(null)",
-           eid, start, stop, ebc);
-  if (!ebc) return dllen + 12;
+           eid, tableid, running,
+           gmtime2local(start, tm1, sizeof(tm1)),
+           gmtime2local(stop, tm2, sizeof(tm2)), ebc);
+  if (!ebc) return 0;
 
   /* Mark re-schedule detect (only now/next) */
   if (save2 && tableid < 0x50) *resched = 1;
@@ -451,8 +472,8 @@ static int _eit_process_event_one
     ptr   += 2;
     if (dllen < dlen) break;
 
-    tvhtrace(mod->id, "  dtag %02X dlen %d", dtag, dlen);
-    tvhlog_hexdump(mod->id, ptr, dlen);
+    tvhtrace(mod->subsys, "%s:  dtag %02X dlen %d", mod->id, dtag, dlen);
+    tvhlog_hexdump(mod->subsys, ptr, dlen);
 
     switch (dtag) {
       case DVB_DESC_SHORT_EVENT:
@@ -488,26 +509,30 @@ static int _eit_process_event_one
    * Broadcast
    */
 
+  *save |= epg_broadcast_set_dvb_eid(ebc, eid, &changes2);
+
   /* Summary/Description */
-  if ( ev.summary )
-    *save |= epg_broadcast_set_summary2(ebc, ev.summary, mod);
-  if ( ev.desc )
-    *save |= epg_broadcast_set_description2(ebc, ev.desc, mod);
+  if (ev.summary)
+    *save |= epg_broadcast_set_summary(ebc, ev.summary, &changes2);
+  if (ev.desc)
+    *save |= epg_broadcast_set_description(ebc, ev.desc, &changes2);
 
   /* Broadcast Metadata */
-  *save |= epg_broadcast_set_is_hd(ebc, ev.hd, mod);
-  *save |= epg_broadcast_set_is_widescreen(ebc, ev.ws, mod);
-  *save |= epg_broadcast_set_is_audio_desc(ebc, ev.ad, mod);
-  *save |= epg_broadcast_set_is_subtitled(ebc, ev.st, mod);
-  *save |= epg_broadcast_set_is_deafsigned(ebc, ev.ds, mod);
+  *save |= epg_broadcast_set_is_hd(ebc, ev.hd, &changes2);
+  *save |= epg_broadcast_set_is_widescreen(ebc, ev.ws, &changes2);
+  *save |= epg_broadcast_set_is_audio_desc(ebc, ev.ad, &changes2);
+  *save |= epg_broadcast_set_is_subtitled(ebc, ev.st, &changes2);
+  *save |= epg_broadcast_set_is_deafsigned(ebc, ev.ds, &changes2);
 
   /*
    * Series link
    */
 
   if (*ev.suri) {
-    if ((es = epg_serieslink_find_by_uri(ev.suri, 1, save)))
-      *save |= epg_broadcast_set_serieslink(ebc, es, mod);
+    if ((es = epg_serieslink_find_by_uri(ev.suri, mod, 1, save, &changes3))) {
+      *save |= epg_broadcast_set_serieslink(ebc, es, &changes2);
+      *save |= epg_serieslink_change_finish(es, changes3, 0);
+    }
   }
 
   /*
@@ -516,30 +541,32 @@ static int _eit_process_event_one
 
   /* Find episode */
   if (*ev.uri) {
-    if ((ee = epg_episode_find_by_uri(ev.uri, 1, save)))
-      *save |= epg_broadcast_set_episode(ebc, ee, mod);
-
-  /* Existing/Artificial */
-  } else
-    ee = epg_broadcast_get_episode(ebc, 1, save);
+    ee = epg_episode_find_by_uri(ev.uri, mod, 1, save, &changes4);
+  } else {
+    ee = epg_episode_find_by_broadcast(ebc, mod, 1, save, &changes4);
+  }
 
   /* Update Episode */
   if (ee) {
-    *save |= epg_episode_set_is_bw(ee, ev.bw, mod);
-    if ( ev.title )
-      *save |= epg_episode_set_title2(ee, ev.title, mod);
-    if ( ev.genre )
-      *save |= epg_episode_set_genre(ee, ev.genre, mod);
-    if ( ev.parental )
-      *save |= epg_episode_set_age_rating(ee, ev.parental, mod);
-    if ( ev.summary )
-      *save |= epg_episode_set_subtitle2(ee, ev.summary, mod);
+    *save |= epg_broadcast_set_episode(ebc, ee, &changes2);
+    *save |= epg_episode_set_is_bw(ee, ev.bw, &changes4);
+    if (ev.title)
+      *save |= epg_episode_set_title(ee, ev.title, &changes4);
+    if (ev.genre)
+      *save |= epg_episode_set_genre(ee, ev.genre, &changes4);
+    if (ev.parental)
+      *save |= epg_episode_set_age_rating(ee, ev.parental, &changes4);
+    if (ev.summary)
+      *save |= epg_episode_set_subtitle(ee, ev.summary, &changes4);
 #if TODO_ADD_EXTRA
-    if ( ev.extra )
-      *save |= epg_episode_set_extra(ee, extra, mod);
+    if (ev.extra)
+      *save |= epg_episode_set_extra(ee, extra, &changes4);
 #endif
+    *save |= epg_episode_change_finish(ee, changes4, 0);
   }
 
+  *save |= epg_broadcast_change_finish(ebc, changes2, 0);
+
   /* Tidy up */
 #if TODO_ADD_EXTRA
   if (ev.extra)   htsmsg_destroy(ev.extra);
@@ -549,23 +576,42 @@ static int _eit_process_event_one
   if (ev.summary) lang_str_destroy(ev.summary);
   if (ev.desc)    lang_str_destroy(ev.desc);
 
-  return ret;
+  /* use running flag only for current broadcast */
+  if (running && tableid == 0x4e) {
+    if (sect == 0) {
+      switch (running) {
+      case 2:  run = EPG_RUNNING_WARM;  break;
+      case 3:  run = EPG_RUNNING_PAUSE; break;
+      case 4:  run = EPG_RUNNING_NOW;   break;
+      default: run = EPG_RUNNING_STOP;  break;
+      }
+      epg_broadcast_notify_running(ebc, EPG_SOURCE_EIT, run);
+    } else if (sect == 1 && running != 2 && running != 3 && running != 4) {
+      epg_broadcast_notify_running(ebc, EPG_SOURCE_EIT, EPG_RUNNING_STOP);
+    }
+  }
+
+  return 0;
 }
 
 static int _eit_process_event
-  ( epggrab_module_t *mod, int tableid,
+  ( epggrab_module_t *mod, int tableid, int sect,
     mpegts_service_t *svc, const uint8_t *ptr, int len,
     int local, int *resched, int *save )
 {
-  channel_service_mapping_t *csm;
-  int ret = 0;
+  idnode_list_mapping_t *ilm;
+  channel_t *ch;
 
   if ( len < 12 ) return -1;
 
-  LIST_FOREACH(csm, &svc->s_channels, csm_svc_link)
-    ret = _eit_process_event_one(mod, tableid, svc, csm->csm_chn,
-                                 ptr, len, local, resched, save);
-  return ret;
+  LIST_FOREACH(ilm, &svc->s_channels, ilm_in1_link) {
+    ch = (channel_t *)ilm->ilm_in2;
+    if (!ch->ch_enabled || ch->ch_epg_parent) continue;
+    if (_eit_process_event_one(mod, tableid, sect, svc, ch,
+                               ptr, len, local, resched, save) < 0)
+      return -1;
+  }
+  return 12 + (((ptr[10] & 0x0f) << 8) | ptr[11]);
 }
 
 
@@ -573,8 +619,7 @@ static int
 _eit_callback
   (mpegts_table_t *mt, const uint8_t *ptr, int len, int tableid)
 {
-  int r;
-  int sect, last, ver, save, resched;
+  int r, sect, last, ver, save, resched, spec;
   uint8_t  seg;
   uint16_t onid, tsid, sid;
   uint32_t extraid;
@@ -584,6 +629,8 @@ _eit_callback
   epggrab_module_t     *mod;
   epggrab_ota_mux_t    *ota = NULL;
   mpegts_psi_table_state_t *st;
+  th_subscription_t    *ths;
+  char ubuf[UUID_HEX_SIZE];
 
   if (!epggrab_ota_running)
     return -1;
@@ -591,10 +638,21 @@ _eit_callback
   mm  = mt->mt_mux;
   map = mt->mt_opaque;
   mod = (epggrab_module_t *)map->om_module;
+  spec = ((eit_private_t *)((epggrab_module_ota_t *)mod)->opaque)->spec;
+
+  /* Statistics */
+  ths = mpegts_mux_find_subscription_by_name(mm, "epggrab");
+  if (ths) {
+    subscription_add_bytes_in(ths, len);
+    subscription_add_bytes_out(ths, len);
+  }
 
   /* Validate */
-  if(tableid < 0x4e || tableid > 0x6f || len < 11)
+  if(tableid < 0x4e || tableid > 0x6f || len < 11) {
+    if (ths)
+      atomic_add(&ths->ths_total_err, 1);
     return -1;
+  }
 
   /* Basic info */
   sid     = ptr[0] << 8 | ptr[1];
@@ -604,15 +662,27 @@ _eit_callback
   extraid = ((uint32_t)tsid << 16) | sid;
   // TODO: extra ID should probably include onid
 
+  tvhtrace(LS_TBL_EIT, "%s: sid %i tsid %04x onid %04x seg %02x",
+           mt->mt_name, sid, tsid, onid, seg);
+
+  /* Local EIT contents - give them another priority to override main events */
+  if (spec == EIT_SPEC_NZ_FREEVIEW &&
+      ((tsid > 0x19 && tsid < 0x1d) ||
+       (tsid > 0x1e && tsid < 0x21)))
+    mod = epggrab_module_find_by_id("nz_freeview");
+
   /* Register interest */
-  if (tableid >= 0x50)
+  if (tableid == 0x4e || (tableid >= 0x50 && tableid < 0x60) ||
+      spec == EIT_SPEC_UK_FREESAT /* uk_freesat hack */)
     ota = epggrab_ota_register((epggrab_module_ota_t*)mod, NULL, mm);
 
   /* Begin */
   r = dvb_table_begin((mpegts_psi_table_t *)mt, ptr, len,
                       tableid, extraid, 11, &st, &sect, &last, &ver);
-  if (r != 1) return r;
-  if (st) {
+  if (r == 0) goto complete;
+  if (r < 0) return r;
+  if (tableid != 0x4e && r != 1) return r;
+  if (st && r > 0) {
     uint32_t mask;
     int sa = seg & 0xF8;
     int sb = 7 - (seg & 0x07);
@@ -624,17 +694,16 @@ _eit_callback
   /* Get transport stream */
   // Note: tableid=0x4f,0x60-0x6f is other TS
   //       so must find the tdmi
-  if(tableid == 0x4f || tableid >= 0x60) {
-    mm = mpegts_network_find_mux(mm->mm_network, onid, tsid);
-
+  if (tableid == 0x4f || tableid >= 0x60) {
+    mm = mpegts_network_find_mux(mm->mm_network, onid, tsid, 1);
   } else {
-    if (mm->mm_tsid != tsid ||
-        mm->mm_onid != onid) {
+    if ((mm->mm_tsid != tsid || mm->mm_onid != onid) &&
+        !mm->mm_eit_tsid_nocheck) {
       if (mm->mm_onid != MPEGTS_ONID_NONE &&
           mm->mm_tsid != MPEGTS_TSID_NONE)
-        tvhtrace("eit",
-                "invalid tsid found tid 0x%02X, onid:tsid %d:%d != %d:%d",
-                tableid, mm->mm_onid, mm->mm_tsid, onid, tsid);
+        tvhtrace(LS_TBL_EIT,
+                "%s: invalid tsid found tid 0x%02X, onid:tsid %d:%d != %d:%d",
+                mt->mt_name, tableid, mm->mm_onid, mm->mm_tsid, onid, tsid);
       mm = NULL;
     }
   }
@@ -644,10 +713,19 @@ _eit_callback
   /* Get service */
   svc = mpegts_mux_find_service(mm, sid);
   if (!svc) {
-    tvhtrace("eit", "sid %i not found", sid);
+    /* NZ Freesat: use main data */
+    if (spec == EIT_SPEC_NZ_FREEVIEW && onid == 0x222a &&
+        (tsid == 0x19 || tsid == 0x1d)) {
+      svc = mpegts_network_find_active_service(mm->mm_network, sid, &mm);
+      if (svc)
+        goto svc_ok;
+    }
+
+    tvhtrace(LS_TBL_EIT, "sid %i not found", sid);
     goto done;
   }
 
+svc_ok:
   if (map->om_first) {
     map->om_tune_count++;
     map->om_first = 0;
@@ -655,7 +733,7 @@ _eit_callback
 
   /* Register this */
   if (ota)
-    epggrab_ota_service_add(map, ota, idnode_uuid_as_str(&svc->s_id), 1);
+    epggrab_ota_service_add(map, ota, idnode_uuid_as_str(&svc->s_id, ubuf), 1);
 
   /* No point processing */
   if (!LIST_FIRST(&svc->s_channels))
@@ -670,10 +748,11 @@ _eit_callback
   ptr += 11;
   while (len) {
     int r;
-    if ((r = _eit_process_event(mod, tableid, svc, ptr, len,
+    if ((r = _eit_process_event(mod, tableid, sect, svc, ptr, len,
                                 mm->mm_network->mn_localtime,
                                 &resched, &save)) < 0)
       break;
+    assert(r > 0);
     len -= r;
     ptr += r;
   }
@@ -684,7 +763,8 @@ _eit_callback
   
 done:
   r = dvb_table_end((mpegts_psi_table_t *)mt, st, sect);
-  if (ota && !r)
+complete:
+  if (ota && !r && (tableid >= 0x50 && tableid < 0x60))
     epggrab_ota_complete((epggrab_module_ota_t*)mod, ota);
   
   return r;
@@ -697,36 +777,54 @@ done:
 static int _eit_start
   ( epggrab_ota_map_t *map, mpegts_mux_t *dm )
 {
-  epggrab_module_ota_t *m = map->om_module;
-  int pid, opts = 0;
+  epggrab_module_ota_t *m = map->om_module, *eit = NULL;
+  eit_private_t *priv = (eit_private_t *)m->opaque;
+  int pid, opts = 0, spec;
 
   /* Disabled */
   if (!m->enabled && !map->om_forced) return -1;
 
-  /* Freeview (switch to EIT, ignore if explicitly enabled) */
-  // Note: do this as PID is the same
-  if (!strcmp(m->id, "uk_freeview")) {
-    m = (epggrab_module_ota_t*)epggrab_module_find_by_id("eit");
-    if (m->enabled) return -1;
+  spec = priv->spec;
+
+  /* Do string conversions also for the EIT table */
+  /* FIXME: It should be done only for selected muxes or networks */
+  if (((eit_private_t *)m->opaque)->conv) {
+    eit = (epggrab_module_ota_t*)epggrab_module_find_by_id("eit");
+    ((eit_private_t *)eit->opaque)->spec = priv->spec;
+    ((eit_private_t *)eit->opaque)->conv = priv->conv;
+  }
+
+  if (spec == EIT_SPEC_NZ_FREEVIEW) {
+    if (eit == NULL)
+      eit = (epggrab_module_ota_t*)epggrab_module_find_by_id("eit");
+    if (eit->enabled)
+      map->om_complete = 1;
+  }
+
+  pid = priv->pid;
+
+  /* Freeview UK/NZ (switch to EIT, ignore if explicitly enabled) */
+  /* Note: do this as PID is the same */
+  if (pid == 0 && strcmp(m->id, "eit")) {
+    if (eit == NULL)
+      eit = (epggrab_module_ota_t*)epggrab_module_find_by_id("eit");
+    if (eit->enabled) return -1;
   }
 
   /* Freesat (3002/3003) */
-  if (!strcmp("uk_freesat", m->id)) {
-    mpegts_table_add(dm, 0, 0, dvb_bat_callback, NULL, "bat", MT_CRC, 3002, MPS_WEIGHT_EIT);
-    pid = 3003;
-
-  /* Viasat Baltic (0x39) */
-  } else if (!strcmp("viasat_baltic", m->id)) {
-    pid = 0x39;
+  if (pid == 3003 && !strcmp("uk_freesat", m->id))
+    mpegts_table_add(dm, DVB_BAT_BASE, DVB_BAT_MASK, dvb_bat_callback, NULL,
+                     "bat", LS_TBL_BASE, MT_CRC, 3002, MPS_WEIGHT_EIT);
 
   /* Standard (0x12) */
-  } else {
+  if (pid == 0) {
     pid  = DVB_EIT_PID;
     opts = MT_RECORD;
   }
-  mpegts_table_add(dm, 0, 0, _eit_callback, map, m->id, MT_CRC | opts, pid, MPS_WEIGHT_EIT);
+
+  mpegts_table_add(dm, 0, 0, _eit_callback, map, m->id, LS_TBL_EIT, MT_CRC | opts, pid, MPS_WEIGHT_EIT);
   // TODO: might want to limit recording to EITpf only
-  tvhlog(LOG_DEBUG, m->id, "installed table handlers");
+  tvhdebug(m->subsys, "%s: installed table handlers", m->id);
   return 0;
 }
 
@@ -765,19 +863,42 @@ static int _eit_tune
   return r;
 }
 
+#define EIT_OPS(name, _pid, _conv, _spec) \
+  static eit_private_t opaque_##name = { \
+    .pid = (_pid), \
+    .conv = (_conv), \
+    .spec = (_spec), \
+  }; \
+  static epggrab_ota_module_ops_t name = { \
+    .start  = _eit_start, \
+    .tune   = _eit_tune, \
+    .opaque = &opaque_##name, \
+  }
+
+#define EIT_CREATE(id, name, prio, ops) \
+  epggrab_module_ota_create(NULL, id, LS_TBL_EIT, NULL, name, prio, ops)
+
 void eit_init ( void )
 {
-  static epggrab_ota_module_ops_t ops = {
-    .start = _eit_start,
-    .tune  = _eit_tune,
-  };
+  EIT_OPS(ops, 0, 0, 0);
+  EIT_OPS(ops_uk_freesat, 3003, EIT_CONV_HUFFMAN, EIT_SPEC_UK_FREESAT);
+  EIT_OPS(ops_uk_freeview, 0, EIT_CONV_HUFFMAN, 0);
+  EIT_OPS(ops_nz_freeview, 0, EIT_CONV_HUFFMAN, EIT_SPEC_NZ_FREEVIEW);
+  EIT_OPS(ops_baltic, 0x39, 0, 0);
+  EIT_OPS(ops_bulsat, 0x12b, 0, 0);
 
-  epggrab_module_ota_create(NULL, "eit", "EIT: DVB Grabber", 1, &ops, NULL);
-  epggrab_module_ota_create(NULL, "uk_freesat", "UK: Freesat", 5, &ops, NULL);
-  epggrab_module_ota_create(NULL, "uk_freeview", "UK: Freeview", 5, &ops, NULL);
-  epggrab_module_ota_create(NULL, "viasat_baltic", "VIASAT: Baltic", 5, &ops, NULL);
+  EIT_CREATE("eit", "EIT: DVB Grabber", 1, &ops);
+  EIT_CREATE("uk_freesat", "UK: Freesat", 5, &ops_uk_freesat);
+  EIT_CREATE("uk_freeview", "UK: Freeview", 5, &ops_uk_freeview);
+  EIT_CREATE("nz_freeview", "New Zealand: Freeview", 5, &ops_nz_freeview);
+  EIT_CREATE("viasat_baltic", "VIASAT: Baltic", 5, &ops_baltic);
+  EIT_CREATE("Bulsatcom_39E", "Bulsatcom: Bula 39E", 5, &ops_bulsat);
 }
 
 void eit_done ( void )
 {
 }
+
+void eit_load ( void )
+{
+}
