@@ -34,8 +34,6 @@
 #include "settings.h"
 #include "input.h"
 
-static epggrab_channel_tree_t _opentv_channels;
-
 #define OPENTV_TITLE_BASE   0xA0
 #define OPENTV_SUMMARY_BASE 0xA8
 #define OPENTV_TABLE_MASK   0xFC
@@ -78,15 +76,18 @@ typedef struct opentv_module_t
   int                   sid;
   int                   bouquetid;
   int                   bouquet_auto;
-  int                   *channel;
-  int                   *title;
-  int                   *summary;
-  opentv_dict_t         *dict;
-  opentv_genre_t        *genre;
-  opentv_pattern_list_t  p_snum;
-  opentv_pattern_list_t  p_enum;
-  opentv_pattern_list_t  p_pnum;
-  opentv_pattern_list_t  p_subt;
+  int                  *channel;
+  int                  *title;
+  int                  *summary;
+  opentv_dict_t        *dict;
+  opentv_genre_t       *genre;
+  int                   titles_time;
+  int                   summaries_time;
+  opentv_pattern_list_t p_snum;
+  opentv_pattern_list_t p_enum;
+  opentv_pattern_list_t p_pnum;
+  opentv_pattern_list_t p_subt;
+  opentv_pattern_list_t p_cleanup_title;
 } opentv_module_t;
 
 /*
@@ -147,6 +148,8 @@ typedef struct opentv_status
   epggrab_ota_map_t *os_map;
   int                os_refcount;
   epggrab_ota_mux_t *os_ota;
+  int64_t            os_titles_start;
+  int64_t            os_summaries_start;
 } opentv_status_t;
 
 static void
@@ -169,8 +172,7 @@ static epggrab_channel_t *_opentv_find_epggrab_channel
 {
   char chid[256];
   snprintf(chid, sizeof(chid), "%s-%d", mod->id, cid);
-  return epggrab_channel_find(&_opentv_channels, chid, create, save,
-                              (epggrab_module_t*)mod);
+  return epggrab_channel_find((epggrab_module_t*)mod, chid, create, save);
 }
 
 /* ************************************************************************
@@ -266,7 +268,7 @@ static int _opentv_parse_event
   int      i    = 4, r;
 
   if (slen+4 > len) {
-    tvhtrace("opentv", "event len (%d) > table len (%d)", slen+4, len);
+    tvhtrace(LS_OPENTV, "event len (%d) > table len (%d)", slen+4, len);
     return -1;
   }
 
@@ -298,7 +300,7 @@ static void *_opentv_apply_pattern_list(char *buf, size_t size_buf, const char *
       memcpy(buf, text + match[1].rm_so, size);
       buf[size] = '\0';
       if (size) {
-         tvhtrace("opentv","  pattern \"%s\" matches with '%s'", p->text, buf);
+         tvhtrace(LS_OPENTV,"  pattern \"%s\" matches with '%s'", p->text, buf);
          return buf;
       }
     }
@@ -312,13 +314,16 @@ opentv_parse_event_section_one
     channel_t *ch, const char *lang,
     const uint8_t *buf, int len )
 {
-  int i, r, save = 0;
+  int i, r, save = 0, merge;
   opentv_module_t  *mod = sta->os_mod;
   epggrab_module_t *src = (epggrab_module_t*)mod;
   epg_broadcast_t *ebc;
   epg_episode_t *ee;
   epg_serieslink_t *es;
   opentv_event_t ev;
+  char buffer[2048], *s;
+  lang_str_t *ls;
+  uint32_t changes, changes2, changes3;
 
   /* Loop around event entries */
   i = 7;
@@ -332,28 +337,38 @@ opentv_parse_event_section_one
      * Broadcast
      */
 
+    merge = changes = changes2 = changes3 = 0;
+
     /* Find broadcast */
     if (ev.start && ev.stop) {
-      ebc = epg_broadcast_find_by_time(ch, ev.start, ev.stop,
-                                       ev.eid, 1, &save);
-      tvhdebug("opentv", "find by time start %"PRItime_t " stop "
+      ebc = epg_broadcast_find_by_time(ch, src, ev.start, ev.stop,
+                                       1, &save, &changes);
+      tvhdebug(LS_OPENTV, "find by time start %"PRItime_t " stop "
                "%"PRItime_t " eid %d = %p",
                ev.start, ev.stop, ev.eid, ebc);
+      save |= epg_broadcast_set_dvb_eid(ebc, ev.eid, &changes);
     } else {
       ebc = epg_broadcast_find_by_eid(ch, ev.eid);
-      tvhdebug("opentv", "find by eid %d = %p", ev.eid, ebc);
+      tvhdebug(LS_OPENTV, "find by eid %d = %p", ev.eid, ebc);
+      if (ebc && ebc->grabber != src)
+        goto done;
+      merge = 1;
     }
     if (!ebc)
       goto done;
 
     /* Summary / Description */
     if (ev.summary) {
-      tvhdebug("opentv", "  summary '%s'", ev.summary);
-      save |= epg_broadcast_set_summary(ebc, ev.summary, lang, src);
+      tvhdebug(LS_OPENTV, "  summary '%s'", ev.summary);
+      ls = lang_str_create2(ev.summary, lang);
+      save |= epg_broadcast_set_summary(ebc, ls, &changes);
+      lang_str_destroy(ls);
     }
     if (ev.desc) {
-      tvhdebug("opentv", "  desc '%s'", ev.desc);
-      save |= epg_broadcast_set_description(ebc, ev.desc, lang, src);
+      tvhdebug(LS_OPENTV, "  desc '%s'", ev.desc);
+      ls = lang_str_create2(ev.desc, lang);
+      save |= epg_broadcast_set_description(ebc, ls, &changes);
+      lang_str_destroy(ls);
     }
 
     /*
@@ -361,70 +376,81 @@ opentv_parse_event_section_one
      */
 
     if (ev.serieslink) {
-      char suri[257];
+      char suri[257], ubuf[UUID_HEX_SIZE];
       snprintf(suri, 256, "opentv://channel-%s/series-%d",
-               channel_get_uuid(ch), ev.serieslink);
-      if ((es = epg_serieslink_find_by_uri(suri, 1, &save)))
-        save |= epg_broadcast_set_serieslink(ebc, es, src);
+               channel_get_uuid(ch, ubuf), ev.serieslink);
+      if ((es = epg_serieslink_find_by_uri(suri, src, 1, &save, &changes2))) {
+        save |= epg_broadcast_set_serieslink(ebc, es, &changes);
+        save |= epg_serieslink_change_finish(es, changes2, merge);
+      }
     }
 
     /*
      * Episode
      */
 
-    if ((ee = epg_broadcast_get_episode(ebc, 1, &save))) {
-      tvhdebug("opentv", "  find episode %p", ee);
+    if ((ee = epg_episode_find_by_broadcast(ebc, src, 1, &save, &changes3))) {
+      save |= epg_broadcast_set_episode(ebc, ee, &changes);
+      tvhdebug(LS_OPENTV, "  find episode %p", ee);
       if (ev.title) {
-        int size;
+        tvhdebug(LS_OPENTV, "    title '%s'", ev.title);
 
-        size = strlen(ev.title);
-        while (size > 0 && isspace(ev.title[size - 1]))
-          ev.title[--size] = '\0';
-
-        tvhdebug("opentv", "    title '%s'", ev.title);
-        save |= epg_episode_set_title(ee, ev.title, lang, src);
+        /* try to cleanup the title */
+        if (_opentv_apply_pattern_list(buffer, sizeof(buffer), ev.title, &mod->p_cleanup_title)) {
+          tvhtrace(LS_OPENTV, "  clean title '%s'", buffer);
+          s = buffer;
+        } else {
+          s = ev.title;
+        }
+        ls = lang_str_create2(s, lang);
+        save |= epg_episode_set_title(ee, ls, &changes3);
+        lang_str_destroy(ls);
       }
       if (ev.cat) {
         epg_genre_list_t *egl = calloc(1, sizeof(epg_genre_list_t));
         epg_genre_list_add_by_eit(egl, ev.cat);
-        save |= epg_episode_set_genre(ee, egl, src);
+        save |= epg_episode_set_genre(ee, egl, &changes3);
         epg_genre_list_destroy(egl);
       }
       if (ev.summary) {
-        char buf[1024];
         epg_episode_num_t en;
 
         memset(&en, 0, sizeof(en));
         /* search for season number */
-        if (_opentv_apply_pattern_list(buf, sizeof(buf), ev.summary, &mod->p_snum))
-          if ((en.s_num = atoi(buf)))
-            tvhtrace("opentv","  extract season number %d", en.s_num);
+        if (_opentv_apply_pattern_list(buffer, sizeof(buffer), ev.summary, &mod->p_snum))
+          if ((en.s_num = atoi(buffer)))
+            tvhtrace(LS_OPENTV,"  extract season number %d", en.s_num);
         /* ...for episode number */
-        if (_opentv_apply_pattern_list(buf, sizeof(buf), ev.summary, &mod->p_enum))
-          if ((en.e_num = atoi(buf)))
-            tvhtrace("opentv","  extract episode number %d", en.e_num);
+        if (_opentv_apply_pattern_list(buffer, sizeof(buffer), ev.summary, &mod->p_enum))
+          if ((en.e_num = atoi(buffer)))
+            tvhtrace(LS_OPENTV,"  extract episode number %d", en.e_num);
         /* ...for part number */
-        if (_opentv_apply_pattern_list(buf, sizeof(buf), ev.summary, &mod->p_pnum)) {
-          if (buf[0] >= 'a' && buf[0] <= 'z')
-            en.p_num = buf[0] - 'a' + 1;
+        if (_opentv_apply_pattern_list(buffer, sizeof(buffer), ev.summary, &mod->p_pnum)) {
+          if (buffer[0] >= 'a' && buffer[0] <= 'z')
+            en.p_num = buffer[0] - 'a' + 1;
           else
-            if (buf[0] >= 'A' && buf[0] <= 'Z')
-              en.p_num = buf[0] - 'A' + 1;
+            if (buffer[0] >= 'A' && buffer[0] <= 'Z')
+              en.p_num = buffer[0] - 'A' + 1;
           if (en.p_num)
-            tvhtrace("opentv","  extract part number %d", en.p_num);
+            tvhtrace(LS_OPENTV,"  extract part number %d", en.p_num);
         }
         /* save any found number */
         if (en.s_num || en.e_num || en.p_num)
-          save |= epg_episode_set_epnum(ee, &en, src);
+          save |= epg_episode_set_epnum(ee, &en, &changes3);
 
         /* ...for subtitle */
-        if (_opentv_apply_pattern_list(buf, sizeof(buf), ev.summary, &mod->p_subt)) {
-          tvhtrace("opentv", "  extract subtitle '%s'", buf);
-          save |= epg_episode_set_subtitle(ee, buf, lang, src);
+        if (_opentv_apply_pattern_list(buffer, sizeof(buffer), ev.summary, &mod->p_subt)) {
+          tvhtrace(LS_OPENTV, "  extract subtitle '%s'", buffer);
+          ls = lang_str_create2(buffer, lang);
+          save |= epg_episode_set_subtitle(ee, ls, &changes3);
+          lang_str_destroy(ls);
         }
       }
+      save |= epg_episode_change_finish(ee, changes3, merge);
     }
 
+    save |= epg_broadcast_change_finish(ebc, changes, merge);
+
     /* Cleanup */
 done:
     if (ev.title)   free(ev.title);
@@ -441,8 +467,9 @@ opentv_parse_event_section
     const uint8_t *buf, int len )
 {
   opentv_module_t *mod = sta->os_mod;
+  channel_t *ch;
   epggrab_channel_t *ec;
-  epggrab_channel_link_t *ecl;
+  idnode_list_mapping_t *ilm;
   const char *lang = NULL;
   int save = 0;
 
@@ -455,8 +482,12 @@ opentv_parse_event_section
   if (!(ec = _opentv_find_epggrab_channel(mod, cid, 0, NULL))) return 0;
 
   /* Iterate all channels */
-  LIST_FOREACH(ecl, &ec->channels, ecl_epg_link)
-    save |= opentv_parse_event_section_one(sta, cid, mjd, ecl->ecl_channel, lang, buf, len);
+  LIST_FOREACH(ilm, &ec->channels, ilm_in2_link) {
+    ch = (channel_t *)ilm->ilm_in2;
+    if (!ch->ch_enabled || ch->ch_epg_parent) continue;
+    save |= opentv_parse_event_section_one(sta, cid, mjd, ch,
+                                           lang, buf, len);
+  }
 
   /* Update EPG */
   if (save) epg_updated();
@@ -475,43 +506,39 @@ opentv_desc_channels
   opentv_status_t *sta = mt->mt_opaque;
   opentv_module_t *mod = sta->os_mod;
   epggrab_channel_t *ec;
-  epggrab_channel_link_t *ecl;
+  idnode_list_mapping_t *ilm;
   mpegts_service_t *svc;
   channel_t *ch;
   int sid, cid, cnum, unk;
-#if ENABLE_TRACE
   int type;
-#endif
   int save = 0;
   int i = 2;
 
   while (i < len) {
     sid  = ((int)buf[i] << 8) | buf[i+1];
-#if ENABLE_TRACE
     type = buf[2];
-#endif
     cid  = ((int)buf[i+3] << 8) | buf[i+4];
     cnum = ((int)buf[i+5] << 8) | buf[i+6];
     unk  = ((int)buf[i+7] << 8) | buf[i+8];
-    tvhtrace(mt->mt_name, "     sid %04X type %02X cid %04X cnum %d unk %04X", sid, type, cid, cnum, unk);
+    tvhtrace(LS_OPENTV, "%s:     sid %04X type %02X cid %04X cnum %d unk %04X", mt->mt_name, sid, type, cid, cnum, unk);
     cnum = cnum < 65535 ? cnum : 0;
 
     /* Find the service */
     svc = mpegts_service_find(mm, sid, 0, 0, NULL);
-    tvhtrace(mt->mt_name, "     svc %p [%s]", svc, svc ? svc->s_nicename : NULL);
+    tvhtrace(LS_OPENTV, "%s:     svc %p [%s]", mt->mt_name, svc, svc ? svc->s_nicename : NULL);
     if (svc && svc->s_dvb_opentv_chnum != cnum &&
         (!svc->s_dvb_opentv_id || svc->s_dvb_opentv_id == unk)) {
       if (mod->bouquetid != nbid) {
         if (mod->bouquet_auto) {
           if (nbid < mod->bouquetid) {
-            tvhwarn(mt->mt_name, "bouquet id set to %d, report this!", nbid);
+            tvhwarn(LS_OPENTV, "%s: bouquet id set to %d, report this!", mt->mt_name, nbid);
             mod->bouquetid = nbid;
           } else
             goto skip_chnum;
         } else
           goto skip_chnum;
       }
-      tvhtrace(mt->mt_name, "      cnum changed (%i != %i)", cnum, (int)svc->s_dvb_opentv_chnum);
+      tvhtrace(LS_OPENTV, "%s:      cnum changed (%i != %i)", mt->mt_name, cnum, (int)svc->s_dvb_opentv_chnum);
       svc->s_dvb_opentv_chnum = cnum;
       svc->s_dvb_opentv_id = unk;
       service_request_save((service_t *)svc, 0);
@@ -519,17 +546,17 @@ opentv_desc_channels
 skip_chnum:
     if (svc && LIST_FIRST(&svc->s_channels)) {
       ec  =_opentv_find_epggrab_channel(mod, cid, 1, &save);
-      ecl = LIST_FIRST(&ec->channels);
-      ch  = LIST_FIRST(&svc->s_channels)->csm_chn;
-      tvhtrace(mt->mt_name, "       ec = %p, ecl = %p", ec, ecl);
+      ilm = LIST_FIRST(&ec->channels);
+      ch  = (channel_t *)LIST_FIRST(&svc->s_channels)->ilm_in2;
+      tvhtrace(mt->mt_subsys, "%s:       ec = %p, ilm = %p", mt->mt_name, ec, ilm);
 
-      if (ecl && ecl->ecl_channel != ch) {
-        epggrab_channel_link_delete(ecl, 1);
-        ecl = NULL;
+      if (ilm && ilm->ilm_in2 != &ch->ch_id) {
+        epggrab_channel_link_delete(ec, ch, 1);
+        ilm = NULL;
       }
       
-      if (!ecl)
-        epggrab_channel_link(ec, ch);
+      if (!ilm && ec->enabled)
+        epggrab_channel_link(ec, ch, NULL);
       save |= epggrab_channel_set_number(ec, cnum, 0);
     }
     i += 9;
@@ -548,6 +575,7 @@ opentv_table_callback
   opentv_status_t *sta;
   opentv_module_t *mod;
   epggrab_ota_mux_t *ota;
+  th_subscription_t *ths;
 
   if (!epggrab_ota_running) return -1;
 
@@ -563,6 +591,14 @@ opentv_table_callback
   mjd = ((int)buf[5] << 8) | buf[6];
   mjd = (mjd - 40587) * 86400;
 
+  /* Statistics */
+  ths = mpegts_mux_find_subscription_by_name(mt->mt_mux, "epggrab");
+  if (ths) {
+    subscription_add_bytes_in(ths, len);
+    subscription_add_bytes_out(ths, len);
+  }
+
+
   /* Begin */
   r = dvb_table_begin((mpegts_psi_table_t *)mt, buf, len,
                       tableid, (uint64_t)cid << 32 | mjd, 7,
@@ -579,36 +615,41 @@ opentv_table_callback
 done:
   if (!r) {
     sta->os_map->om_first = 0; /* valid data mark */
-    tvhtrace(mt->mt_name, "pid %d complete remain %d",
-             mt->mt_pid, sta->os_refcount-1);
+    tvhtrace(mt->mt_subsys, "%s: pid %d complete remain %d",
+             mt->mt_name, mt->mt_pid, sta->os_refcount-1);
   
     /* Last PID */
     if (sta->os_refcount == 1) {
   
       if (mt->mt_table == OPENTV_TITLE_BASE) {
-        int *t;
-        tvhinfo(mt->mt_name, "titles complete");
+        if (sta->os_titles_start + sec2mono(mod->titles_time) < mclk()) {
+          int *t;
+          tvhinfo(mt->mt_subsys, "%s: titles complete", mt->mt_name);
 
-        /* Summaries */
-        t = mod->summary;
-        while (*t) {
-          mpegts_table_t *mt2;
-          mt2 = mpegts_table_add(mt->mt_mux,
-                                 OPENTV_SUMMARY_BASE, OPENTV_TABLE_MASK,
-                                 opentv_table_callback, sta,
-                                 mod->id, MT_CRC, *t++,
-                                 MPS_WEIGHT_EIT);
-          if (mt2) {
-            sta->os_refcount++;
-            mt2->mt_destroy    = opentv_status_destroy;
+          /* Summaries */
+          t = mod->summary;
+          while (*t) {
+            mpegts_table_t *mt2;
+            mt2 = mpegts_table_add(mt->mt_mux,
+                                   OPENTV_SUMMARY_BASE, OPENTV_TABLE_MASK,
+                                   opentv_table_callback, sta,
+                                   mod->id, LS_OPENTV, MT_CRC, *t++,
+                                   MPS_WEIGHT_EIT);
+            if (mt2) {
+              sta->os_refcount++;
+              mt2->mt_destroy    = opentv_status_destroy;
+            }
           }
+          mpegts_table_destroy(mt);
+          sta->os_summaries_start = mclk();
         }
-        mpegts_table_destroy(mt);
       } else {
-        tvhinfo(mt->mt_name, "summaries complete");
-        mpegts_table_destroy(mt);
-        if (ota)
-          epggrab_ota_complete((epggrab_module_ota_t*)mod, ota);
+        if (sta->os_summaries_start + sec2mono(mod->summaries_time) < mclk()) {
+          tvhinfo(mt->mt_subsys, "%s: summaries complete", mt->mt_name);
+          mpegts_table_destroy(mt);
+          if (ota)
+            epggrab_ota_complete((epggrab_module_ota_t*)mod, ota);
+        }
       }
     } else {
       mpegts_table_destroy(mt);
@@ -626,6 +667,7 @@ opentv_bat_callback
   opentv_status_t *sta;
   opentv_module_t *mod;
   epggrab_ota_mux_t *ota;
+  th_subscription_t *ths;
 
   if (!epggrab_ota_running) return -1;
 
@@ -633,6 +675,13 @@ opentv_bat_callback
   mod = sta->os_mod;
   ota = sta->os_ota;
 
+  /* Statistics */
+  ths = mpegts_mux_find_subscription_by_name(mt->mt_mux, "epggrab");
+  if (ths) {
+    subscription_add_bytes_in(ths, len);
+    subscription_add_bytes_out(ths, len);
+  }
+
   r = dvb_bat_callback(mt, buf, len, tableid);
 
   /* Register */
@@ -643,7 +692,7 @@ opentv_bat_callback
 
   /* Complete */
   if (!r) {
-    tvhinfo(mt->mt_name, "channels complete");
+    tvhinfo(mt->mt_subsys, "%s: channels complete", mt->mt_name);
 
     /* Install event handlers */
     t = mod->title;
@@ -652,7 +701,7 @@ opentv_bat_callback
       mt2 = mpegts_table_add(mt->mt_mux,
                              OPENTV_TITLE_BASE, OPENTV_TABLE_MASK,
                              opentv_table_callback, mt->mt_opaque,
-                             mod->id, MT_CRC, *t++,
+                             mod->id, LS_OPENTV, MT_CRC, *t++,
                              MPS_WEIGHT_EIT);
       if (mt2) {
         if (!mt2->mt_destroy) {
@@ -661,6 +710,7 @@ opentv_bat_callback
         }
       }
     }
+    sta->os_titles_start = mclk();
 
     /* Remove BAT handler */
     mpegts_table_destroy(mt);
@@ -691,7 +741,7 @@ static int _opentv_start
   if (mod->tsid != mm->mm_tsid) return -1;
 
   /* Install tables */
-  tvhdebug(mod->id, "install table handlers");
+  tvhdebug(mod->subsys, "%s: install table handlers", mod->id);
 
   /* Channels */
   t   = mod->channel;
@@ -703,7 +753,7 @@ static int _opentv_start
     }
     mt = mpegts_table_add(mm, DVB_BAT_BASE, DVB_BAT_MASK,
                           opentv_bat_callback, sta,
-                          m->id, MT_CRC, *t++,
+                          m->id, LS_OPENTV, MT_CRC, *t++,
                           MPS_WEIGHT_EIT);
     if (mt) {
       mt->mt_mux_cb  = bat_desc;
@@ -760,11 +810,11 @@ static void _opentv_compile_pattern_list ( opentv_pattern_list_t *list, htsmsg_t
     pattern = calloc(1, sizeof(opentv_pattern_t));
     pattern->text = strdup(s);
     if (regcomp(&pattern->compiled, pattern->text, REG_EXTENDED)) {
-      tvhlog(LOG_WARNING, "opentv", "error compiling pattern \"%s\"", pattern->text);
+      tvhwarn(LS_OPENTV, "error compiling pattern \"%s\"", pattern->text);
       free(pattern->text);
       free(pattern);
     } else {
-      tvhtrace("opentv", "compiled pattern \"%s\"", pattern->text);
+      tvhtrace(LS_OPENTV, "compiled pattern \"%s\"", pattern->text);
       TAILQ_INSERT_TAIL(list, pattern, p_links);
     }
   }
@@ -776,7 +826,7 @@ static int _opentv_genre_load_one ( const char *id, htsmsg_t *m )
   opentv_genre_t *genre = calloc(1, sizeof(opentv_genre_t));
   genre->id = (char*)id;
   if (RB_INSERT_SORTED(&_opentv_genres, genre, h_link, _genre_cmp)) {
-    tvhlog(LOG_DEBUG, "opentv", "ignore duplicate genre map %s", id);
+    tvhdebug(LS_OPENTV, "ignore duplicate genre map %s", id);
     free(genre);
     return 0;
   } else {
@@ -799,9 +849,9 @@ static void _opentv_genre_load ( htsmsg_t *m )
     if ((e = htsmsg_get_list(m, f->hmf_name))) {
       if ((r = _opentv_genre_load_one(f->hmf_name, e))) {
         if (r > 0) 
-          tvhlog(LOG_DEBUG, "opentv", "genre map %s loaded", f->hmf_name);
+          tvhdebug(LS_OPENTV, "genre map %s loaded", f->hmf_name);
         else
-          tvhlog(LOG_WARNING, "opentv", "genre map %s failed", f->hmf_name);
+          tvhwarn(LS_OPENTV, "genre map %s failed", f->hmf_name);
       }
     }
   }
@@ -813,7 +863,7 @@ static int _opentv_dict_load_one ( const char *id, htsmsg_t *m )
   opentv_dict_t *dict = calloc(1, sizeof(opentv_dict_t));
   dict->id = (char*)id;
   if (RB_INSERT_SORTED(&_opentv_dicts, dict, h_link, _dict_cmp)) {
-    tvhlog(LOG_DEBUG, "opentv", "ignore duplicate dictionary %s", id);
+    tvhdebug(LS_OPENTV, "ignore duplicate dictionary %s", id);
     free(dict);
     return 0;
   } else {
@@ -838,9 +888,9 @@ static void _opentv_dict_load ( htsmsg_t *m )
     if ((e = htsmsg_get_list(m, f->hmf_name))) {
       if ((r = _opentv_dict_load_one(f->hmf_name, e))) {
         if (r > 0) 
-          tvhlog(LOG_DEBUG, "opentv", "dictionary %s loaded", f->hmf_name);
+          tvhdebug(LS_OPENTV, "dictionary %s loaded", f->hmf_name);
         else
-          tvhlog(LOG_WARNING, "opentv", "dictionary %s failed", f->hmf_name);
+          tvhwarn(LS_OPENTV, "dictionary %s failed", f->hmf_name);
       }
     }
   } 
@@ -872,6 +922,7 @@ static void _opentv_done( void *m )
   _opentv_free_pattern_list(&mod->p_enum);
   _opentv_free_pattern_list(&mod->p_pnum);
   _opentv_free_pattern_list(&mod->p_subt);
+  _opentv_free_pattern_list(&mod->p_cleanup_title);
 }
 
 static int _opentv_tune
@@ -892,6 +943,7 @@ static int _opentv_prov_load_one ( const char *id, htsmsg_t *m )
   char ibuf[100], nbuf[1000];
   htsmsg_t *cl, *tl, *sl;
   uint32_t tsid, sid, onid, bouquetid;
+  uint32_t titles_time, summaries_time;
   const char *str, *name;
   opentv_dict_t *dict;
   opentv_genre_t *genre;
@@ -914,6 +966,8 @@ static int _opentv_prov_load_one ( const char *id, htsmsg_t *m )
   if (htsmsg_get_u32(m, "tsid", &tsid)) return -1;
   if (htsmsg_get_u32(m, "sid", &sid)) return -1;
   if (htsmsg_get_u32(m, "bouquetid", &bouquetid)) return -1;
+  titles_time = htsmsg_get_u32_or_default(m, "titles_time", 30);
+  summaries_time = htsmsg_get_u32_or_default(m, "summaries_time", 240);
 
   /* Genre map (optional) */
   str = htsmsg_get_str(m, "genre");
@@ -928,9 +982,9 @@ static int _opentv_prov_load_one ( const char *id, htsmsg_t *m )
 
   /* Create */
   sprintf(nbuf, "OpenTV: %s", name);
-  mod = (opentv_module_t*)
+  mod = (opentv_module_t *)
     epggrab_module_ota_create(calloc(1, sizeof(opentv_module_t)),
-                              ibuf, nbuf, 2, &ops, NULL);
+                              ibuf, LS_OPENTV, NULL, nbuf, 2, &ops);
 
   /* Add provider details */
   mod->dict     = dict;
@@ -940,15 +994,16 @@ static int _opentv_prov_load_one ( const char *id, htsmsg_t *m )
   mod->sid      = sid;
   mod->bouquetid = bouquetid;
   mod->bouquet_auto = bouquetid == 0;
+  mod->titles_time = MAX(titles_time, 120);
+  mod->summaries_time = MAX(summaries_time, 600);
   mod->channel  = _pid_list_to_array(cl);
   mod->title    = _pid_list_to_array(tl);
   mod->summary  = _pid_list_to_array(sl);
-  mod->channels = &_opentv_channels;
-  mod->ch_rem   = epggrab_module_ch_rem;
   _opentv_compile_pattern_list(&mod->p_snum, htsmsg_get_list(m, "season_num"));
   _opentv_compile_pattern_list(&mod->p_enum, htsmsg_get_list(m, "episode_num"));
   _opentv_compile_pattern_list(&mod->p_pnum, htsmsg_get_list(m, "part_num"));
   _opentv_compile_pattern_list(&mod->p_subt, htsmsg_get_list(m, "subtitle"));
+  _opentv_compile_pattern_list(&mod->p_cleanup_title, htsmsg_get_list(m, "cleanup_title"));
 
   return 1;
 }
@@ -962,9 +1017,9 @@ static void _opentv_prov_load ( htsmsg_t *m )
     if ((e = htsmsg_get_map_by_field(f))) {
       if ((r = _opentv_prov_load_one(f->hmf_name, e))) {
         if (r > 0)
-          tvhlog(LOG_DEBUG, "opentv", "provider %s loaded", f->hmf_name);
+          tvhdebug(LS_OPENTV, "provider %s loaded", f->hmf_name);
         else
-          tvhlog(LOG_WARNING, "opentv", "provider %s failed", f->hmf_name);
+          tvhwarn(LS_OPENTV, "provider %s failed", f->hmf_name);
       }
     }
   }
@@ -979,22 +1034,20 @@ void opentv_init ( void )
 {
   htsmsg_t *m;
 
-  RB_INIT(&_opentv_channels);
-
   /* Load dictionaries */
   if ((m = hts_settings_load("epggrab/opentv/dict")))
     _opentv_dict_load(m);
-  tvhlog(LOG_DEBUG, "opentv", "dictonaries loaded");
+  tvhdebug(LS_OPENTV, "dictonaries loaded");
 
   /* Load genres */
   if ((m = hts_settings_load("epggrab/opentv/genre")))
     _opentv_genre_load(m);
-  tvhlog(LOG_DEBUG, "opentv", "genre maps loaded");
+  tvhdebug(LS_OPENTV, "genre maps loaded");
 
   /* Load providers */
   if ((m = hts_settings_load("epggrab/opentv/prov")))
     _opentv_prov_load(m);
-  tvhlog(LOG_DEBUG, "opentv", "providers loaded");
+  tvhdebug(LS_OPENTV, "providers loaded");
 }
 
 void opentv_done ( void )
@@ -1002,7 +1055,6 @@ void opentv_done ( void )
   opentv_dict_t *dict;
   opentv_genre_t *genre;
 
-  epggrab_channel_flush(&_opentv_channels, 0);
   while ((dict = RB_FIRST(&_opentv_dicts)) != NULL) {
     RB_REMOVE(&_opentv_dicts, dict, h_link);
     huffman_tree_destroy(dict->codes);
@@ -1018,5 +1070,9 @@ void opentv_done ( void )
 
 void opentv_load ( void )
 {
-  // TODO: do we want to keep a list of channels stored?
+  epggrab_module_t *m;
+
+  LIST_FOREACH(m, &epggrab_modules, link)
+    if (strncmp(m->id, "opentv-", 7) == 0)
+      epggrab_module_channels_load(m->id);
 }
