@@ -21,12 +21,11 @@
 #include "lang_codes.h"
 #include "service.h"
 #include "access.h"
+#include "descrambler/caid.h"
 #include "esfilter.h"
 
 struct esfilter_entry_queue esfilters[ESF_CLASS_LAST + 1];
 
-static void esfilter_class_save(idnode_t *self);
-
 /*
  * Class masks
  */
@@ -127,7 +126,7 @@ esfilter_reindex(esfilter_class_t cls)
   TAILQ_FOREACH(esf, &esfilters[cls], esf_link)
     if (esf->esf_save) {
       esf->esf_save = 0;
-      esfilter_class_save((idnode_t *)esf);
+      idnode_changed(&esf->esf_id);
     }
 }
 
@@ -159,12 +158,12 @@ esfilter_create
     }
   }
   if (!c) {
-    tvherror("esfilter", "wrong class %d!", cls);
+    tvherror(LS_ESFILTER, "wrong class %d!", cls);
     abort();
   }
   if (idnode_insert(&esf->esf_id, uuid, c, 0)) {
     if (uuid)
-      tvherror("esfilter", "invalid uuid '%s'", uuid);
+      tvherror(LS_ESFILTER, "invalid uuid '%s'", uuid);
     free(esf);
     return NULL;
   }
@@ -173,7 +172,7 @@ esfilter_create
   if (ESF_CLASS_IS_VALID(cls))
     esf->esf_class = cls;
   else if (!ESF_CLASS_IS_VALID(esf->esf_class)) {
-    tvherror("esfilter", "wrong class %d!", esf->esf_class);
+    tvherror(LS_ESFILTER, "wrong class %d!", esf->esf_class);
     abort();
   }
   if (esf->esf_index) {
@@ -184,16 +183,18 @@ esfilter_create
   if (!conf)
     esfilter_reindex(esf->esf_class);
   if (save)
-    esfilter_class_save((idnode_t *)esf);
+    idnode_changed(&esf->esf_id);
   return esf;
 }
 
 static void
 esfilter_delete(esfilter_t *esf, int delconf)
 {
+  char ubuf[UUID_HEX_SIZE];
   if (delconf)
-    hts_settings_remove("esfilter/%s", idnode_uuid_as_str(&esf->esf_id));
+    hts_settings_remove("esfilter/%s", idnode_uuid_as_str(&esf->esf_id, ubuf));
   TAILQ_REMOVE(&esfilters[esf->esf_class], esf, esf_link);
+  idnode_save_check(&esf->esf_id, delconf);
   idnode_unlink(&esf->esf_id);
   free(esf->esf_comment);
   free(esf);
@@ -203,20 +204,21 @@ esfilter_delete(esfilter_t *esf, int delconf)
  * Class functions
  */
 
-static void
-esfilter_class_save(idnode_t *self)
+static htsmsg_t *
+esfilter_class_save(idnode_t *self, char *filename, size_t fsize)
 {
   htsmsg_t *c = htsmsg_create_map();
+  char ubuf[UUID_HEX_SIZE];
   idnode_save(self, c);
-  hts_settings_save(c, "esfilter/%s", idnode_uuid_as_str(self));
-  htsmsg_destroy(c);
+  snprintf(filename, fsize, "esfilter/%s", idnode_uuid_as_str(self, ubuf));
+  return c;
 }
 
 static const char *
-esfilter_class_get_title(idnode_t *self)
+esfilter_class_get_title(idnode_t *self, const char *lang)
 {
   esfilter_t *esf = (esfilter_t *)self;
-  return idnode_uuid_as_str(&esf->esf_id);
+  return idnode_uuid_as_str(&esf->esf_id, prop_sbuf);
 }
 
 static void
@@ -264,7 +266,7 @@ esfilter_class_type_get(void *o)
 }
 
 static char *
-esfilter_class_type_rend (void *o)
+esfilter_class_type_rend (void *o, const char *lang)
 {
   char *str;
   htsmsg_t *l = htsmsg_create_list();
@@ -276,7 +278,7 @@ esfilter_class_type_rend (void *o)
       htsmsg_add_str(l, NULL, streaming_component_type2txt(i));
   }
 
-  str = htsmsg_list_2_csv(l);
+  str = htsmsg_list_2_csv(l, ',', 1);
   htsmsg_destroy(l);
   return str;
 }
@@ -305,10 +307,11 @@ esfilter_class_type_set_(void *o, const void *v, esfilter_class_t cls)
 }
 
 static htsmsg_t *
-esfilter_class_type_enum_(void *o, esfilter_class_t cls)
+esfilter_class_type_enum_(void *o, const char *lang, esfilter_class_t cls)
 {
   uint32_t mask = esfilterclsmask[cls];
   htsmsg_t *l = htsmsg_create_list();
+  const char *any = N_("ANY");
   int i;
 
   for (i = SCT_UNKNOWN; i <= SCT_LAST; i++) {
@@ -316,7 +319,8 @@ esfilter_class_type_enum_(void *o, esfilter_class_t cls)
       htsmsg_t *e = htsmsg_create_map();
       htsmsg_add_u32(e, "key", i);
       htsmsg_add_str(e, "val",
-          i == SCT_UNKNOWN ? "ANY" : streaming_component_type2txt(i));
+          i == SCT_UNKNOWN ? tvh_gettext_lang(lang, any) :
+                             streaming_component_type2txt(i));
       htsmsg_add_msg(l, NULL, e);
     }
   }
@@ -326,8 +330,8 @@ esfilter_class_type_enum_(void *o, esfilter_class_t cls)
 #define ESFILTER_CLS(func, type) \
 static int esfilter_class_type_set_##func(void *o, const void *v) \
   { return esfilter_class_type_set_(o, v, type); } \
-static htsmsg_t * esfilter_class_type_enum_##func(void *o) \
-  { return esfilter_class_type_enum_(o, type); }
+static htsmsg_t * esfilter_class_type_enum_##func(void *o, const char *lang) \
+  { return esfilter_class_type_enum_(o, lang, type); }
 
 ESFILTER_CLS(video, ESF_CLASS_VIDEO);
 ESFILTER_CLS(audio, ESF_CLASS_AUDIO);
@@ -352,30 +356,28 @@ esfilter_class_language_set(void *o, const void *v)
   const char *s = v;
   char n[4];
   int save;
-  strncpy(n, s && s[0] ? lang_code_get(s) : "", 4);
-  n[3] = 0;
+  strlcpy(n, s && s[0] ? lang_code_get(s) : "", 4);
   save = strcmp(esf->esf_language, n);
   strcpy(esf->esf_language, n);
   return save;
 }
 
 static htsmsg_t *
-esfilter_class_language_enum(void *o)
+esfilter_class_language_enum(void *o, const char *lang)
 {
   htsmsg_t *l = htsmsg_create_list();
   const lang_code_t *lc = lang_codes;
+  const char *any = N_("ANY");
   char buf[128];
 
   while (lc->code2b) {
-    htsmsg_t *e = htsmsg_create_map();
+    htsmsg_t *e;
     if (!strcmp(lc->code2b, "und")) {
-      htsmsg_add_str(e, "key", "");
-      htsmsg_add_str(e, "val", "ANY");
+      e = htsmsg_create_key_val("", tvh_gettext_lang(lang, any));
     } else {
-      htsmsg_add_str(e, "key", lc->code2b);
       snprintf(buf, sizeof(buf), "%s (%s)", lc->desc, lc->code2b);
       buf[sizeof(buf)-1] = '\0';
-      htsmsg_add_str(e, "val", buf);
+      e = htsmsg_create_key_val(lc->code2b, buf);
     }
     htsmsg_add_msg(l, NULL, e);
     lc++;
@@ -399,15 +401,14 @@ esfilter_class_service_set(void *o, const void *v)
   const char *s = v;
   int save = 0;
   if (strncmp(esf->esf_service, s, UUID_HEX_SIZE)) {
-    strncpy(esf->esf_service, s, UUID_HEX_SIZE);
-    esf->esf_service[UUID_HEX_SIZE-1] = '\0';
+    strlcpy(esf->esf_service, s, UUID_HEX_SIZE);
     save = 1;
   }
   return save;
 }
 
 static htsmsg_t *
-esfilter_class_service_enum(void *o)
+esfilter_class_service_enum(void *o, const char *lang)
 {
   htsmsg_t *e, *m = htsmsg_create_map();
   htsmsg_add_str(m, "type",  "api");
@@ -436,7 +437,7 @@ esfilter_build_ca_cmp(const void *_a, const void *_b)
 static htsmsg_t *
 esfilter_build_ca_enum(int provider)
 {
-  htsmsg_t *e, *l;
+  htsmsg_t *l;
   uint32_t *a = alloca(sizeof(uint32_t) * MAX_ITEMS);
   char buf[16], buf2[128];
   service_t *s;
@@ -464,20 +465,14 @@ esfilter_build_ca_enum(int provider)
 
   l = htsmsg_create_list();
 
-  e = htsmsg_create_map();
-  htsmsg_add_str(e, "key", provider ? "ffffff" : "ffff");
-  htsmsg_add_str(e, "val", "ANY");
-  htsmsg_add_msg(l, NULL, e);
+  htsmsg_add_msg(l, NULL, htsmsg_create_key_val(provider ? "ffffff" : "ffff", "ANY"));
 
   for (i = 0; i < count; i++) {
-    e = htsmsg_create_map();
     snprintf(buf, sizeof(buf), provider ? "%06x" : "%04x", a[i]);
     if (!provider)
       snprintf(buf2, sizeof(buf2), "%04x - %s",
-               a[i], descrambler_caid2name(a[i]));
-    htsmsg_add_str(e, "key", buf);
-    htsmsg_add_str(e, "val", provider ? buf : buf2);
-    htsmsg_add_msg(l, NULL, e);
+               a[i], caid2name(a[i]));
+    htsmsg_add_msg(l, NULL, htsmsg_create_key_val(buf, provider ? buf : buf2));
   }
   return l;
 
@@ -509,7 +504,7 @@ esfilter_class_caid_set(void *o, const void *v)
 }
 
 static htsmsg_t *
-esfilter_class_caid_enum(void *o)
+esfilter_class_caid_enum(void *o, const char *lang)
 {
   return esfilter_build_ca_enum(0);
 }
@@ -546,7 +541,7 @@ esfilter_class_caprovider_set(void *o, const void *v)
 }
 
 static htsmsg_t *
-esfilter_class_caprovider_enum(void *o)
+esfilter_class_caprovider_enum(void *o, const char *lang)
 {
   return esfilter_build_ca_enum(1);
 }
@@ -572,7 +567,7 @@ esfilter_class_action_set(void *o, const void *v)
 }
 
 static htsmsg_t *
-esfilter_class_action_enum(void *o)
+esfilter_class_action_enum(void *o, const char *lang)
 {
   htsmsg_t *l = htsmsg_create_list();
   int i;
@@ -586,9 +581,13 @@ esfilter_class_action_enum(void *o)
   return l;
 }
 
+CLASS_DOC(filters)
+PROP_DOC(action)
+
 const idclass_t esfilter_class = {
   .ic_class      = "esfilter",
-  .ic_caption    = "Elementary Stream Filter",
+  .ic_caption    = N_("Elementary stream filter"),
+  .ic_doc        = tvh_doc_filters_class,
   .ic_event      = "esfilter",
   .ic_perm_def   = ACCESS_ADMIN,
   .ic_save       = esfilter_class_save,
@@ -600,21 +599,22 @@ const idclass_t esfilter_class = {
     {
       .type     = PT_INT,
       .id       = "class",
-      .name     = "Class",
-      .opts     = PO_RDONLY | PO_HIDDEN,
+      .name     = N_("Class"),
+      .opts     = PO_RDONLY | PO_HIDDEN | PO_DOC_NLIST,
       .off      = offsetof(esfilter_t, esf_class),
     },
     {
       .type     = PT_INT,
       .id       = "index",
-      .name     = "Index",
-      .opts     = PO_RDONLY | PO_HIDDEN,
+      .name     = N_("Index"),
+      .opts     = PO_RDONLY | PO_HIDDEN | PO_DOC_NLIST,
       .off      = offsetof(esfilter_t, esf_index),
     },
     {
       .type     = PT_BOOL,
       .id       = "enabled",
-      .name     = "Enabled",
+      .name     = N_("Enabled"),
+      .desc     = N_("Enable this filter."),
       .off      = offsetof(esfilter_t, esf_enabled),
     },
     {}
@@ -624,13 +624,15 @@ const idclass_t esfilter_class = {
 const idclass_t esfilter_class_video = {
   .ic_super      = &esfilter_class,
   .ic_class      = "esfilter_video",
-  .ic_caption    = "Video Stream Filter",
+  .ic_caption    = N_("Video Stream Filter"),
   .ic_properties = (const property_t[]){
     {
       .type     = PT_STR,
       .islist   = 1,
       .id       = "type",
-      .name     = "Stream Type",
+      .name     = N_("Stream type"),
+      .desc     = N_("The video stream types the filter should apply "
+                     "to."),
       .get      = esfilter_class_type_get,
       .set      = esfilter_class_type_set_video,
       .list     = esfilter_class_type_enum_video,
@@ -639,15 +641,19 @@ const idclass_t esfilter_class_video = {
     {
       .type     = PT_STR,
       .id       = "language",
-      .name     = "Language",
+      .name     = N_("Language"),
+      .desc     = N_("The language to which the filter should apply."),
       .get      = esfilter_class_language_get,
       .set      = esfilter_class_language_set,
       .list     = esfilter_class_language_enum,
+      .opts     = PO_DOC_NLIST,
     },
     {
       .type     = PT_STR,
       .id       = "service",
-      .name     = "Service",
+      .name     = N_("Service"),
+      .desc     = N_("The service to which the filter should apply. "
+                     "Leave blank to apply the filter to all services."),
       .get      = esfilter_class_service_get,
       .set      = esfilter_class_service_set,
       .list     = esfilter_class_service_enum,
@@ -655,33 +661,54 @@ const idclass_t esfilter_class_video = {
     {
       .type     = PT_INT,
       .id       = "sindex",
-      .name     = "Stream Index",
+      .name     = N_("Stream index"),
+      .desc     = N_("The logical stream index to compare. Note that "
+                     "this index is computed using all filters."
+                     "Example: If filter is set to AC3 audio type and "
+                     "the language to 'eng' and there are two AC3 "
+                     "'eng' streams in the service, the first stream "
+                     "could be identified using number 1 and the "
+                     "second using number 2."),
       .off      = offsetof(esfilter_t, esf_sindex),
     },
     {
       .type     = PT_INT,
       .id       = "pid",
-      .name     = "PID",
+      .name     = N_("PID"),
+      .desc     = N_("Program identification (PID) number to compare. "
+                     "Zero means any. This comparison is processed "
+                     "only when service comparison is active and for "
+                     "the Conditional Access filter."),
       .off      = offsetof(esfilter_t, esf_pid),
     },
     {
       .type     = PT_INT,
       .id       = "action",
-      .name     = "Action",
+      .name     = N_("Action"),
+      .desc     = N_("The rule action defines the operation when all "
+                     "comparisons succeed. See Help for more "
+                     "information on what the various rules do."),
       .get      = esfilter_class_action_get,
       .set      = esfilter_class_action_set,
       .list     = esfilter_class_action_enum,
+      .opts     = PO_DOC_NLIST,
+      .doc      = prop_doc_action,
     },
     {
       .type     = PT_BOOL,
       .id       = "log",
-      .name     = "Log",
+      .name     = N_("Log"),
+      .desc     = N_("Write a short message to log identifying the "
+                     "matched parameters. It is useful for debugging "
+                     "your setup or structure of incoming streams."),
       .off      = offsetof(esfilter_t, esf_log),
     },
     {
       .type     = PT_STR,
       .id       = "comment",
-      .name     = "Comment",
+      .name     = N_("Comment"),
+      .desc     = N_("Free-format text field. Enter whatever you "
+                     "like here."),
       .off      = offsetof(esfilter_t, esf_comment),
     },
     {}
@@ -691,13 +718,15 @@ const idclass_t esfilter_class_video = {
 const idclass_t esfilter_class_audio = {
   .ic_super      = &esfilter_class,
   .ic_class      = "esfilter_audio",
-  .ic_caption    = "Audio Stream Filter",
+  .ic_caption    = N_("Audio Stream Filter"),
   .ic_properties = (const property_t[]){
     {
       .type     = PT_STR,
       .islist   = 1,
       .id       = "type",
-      .name     = "Stream Type",
+      .name     = N_("Stream type"),
+      .desc     = N_("The audio stream types the filter should apply "
+                     "to."),
       .get      = esfilter_class_type_get,
       .set      = esfilter_class_type_set_audio,
       .list     = esfilter_class_type_enum_audio,
@@ -706,15 +735,20 @@ const idclass_t esfilter_class_audio = {
     {
       .type     = PT_STR,
       .id       = "language",
-      .name     = "Language",
+      .name     = N_("Language"),
+      .desc     = N_("The language to which the filter should apply."),
       .get      = esfilter_class_language_get,
       .set      = esfilter_class_language_set,
       .list     = esfilter_class_language_enum,
+      .opts     = PO_DOC_NLIST,
     },
     {
       .type     = PT_STR,
       .id       = "service",
-      .name     = "Service",
+      .name     = N_("Service"),
+      .desc     = N_("The service the filter should apply to. "
+                     "Leave blank to apply the filter to all "
+                     "services."),
       .get      = esfilter_class_service_get,
       .set      = esfilter_class_service_set,
       .list     = esfilter_class_service_enum,
@@ -722,33 +756,54 @@ const idclass_t esfilter_class_audio = {
     {
       .type     = PT_INT,
       .id       = "sindex",
-      .name     = "Stream Index",
+      .name     = N_("Stream index"),
+      .desc     = N_("The logical stream index to compare. Note that "
+                     "this index is computed using all filters."
+                     "Example: If filter is set to AC3 audio type and "
+                     "the language to 'eng' and there are two AC3 "
+                     "'eng' streams in the service, the first stream "
+                     "could be identified using number 1 and the "
+                     "second using number 2."),
       .off      = offsetof(esfilter_t, esf_sindex),
     },
     {
       .type     = PT_INT,
       .id       = "pid",
-      .name     = "PID",
+      .name     = N_("PID"),
+      .desc     = N_("Program identification (PID) number to compare. "
+                     "Zero means any. This comparison is processed "
+                     "only when service comparison is active and for "
+                     "the Conditional Access filter."),
       .off      = offsetof(esfilter_t, esf_pid),
     },
     {
       .type     = PT_INT,
       .id       = "action",
-      .name     = "Action",
+      .name     = N_("Action"),
+      .desc     = N_("The rule action defines the operation when all "
+                     "comparisons succeed. See Help for more "
+                     "information on what the various rules do."),
       .get      = esfilter_class_action_get,
       .set      = esfilter_class_action_set,
       .list     = esfilter_class_action_enum,
+      .opts     = PO_DOC_NLIST,
+      .doc      = prop_doc_action,
     },
     {
       .type     = PT_BOOL,
       .id       = "log",
-      .name     = "Log",
+      .name     = N_("Log"),
+      .desc     = N_("Write a short message to log identifying the "
+                     "matched parameters. It is useful for debugging "
+                     "your setup or structure of incoming streams."),
       .off      = offsetof(esfilter_t, esf_log),
     },
     {
       .type     = PT_STR,
       .id       = "comment",
-      .name     = "Comment",
+      .name     = N_("Comment"),
+      .desc     = N_("Free-format text field. Enter whatever you "
+                     "like here."),
       .off      = offsetof(esfilter_t, esf_comment),
     },
     {}
@@ -758,13 +813,15 @@ const idclass_t esfilter_class_audio = {
 const idclass_t esfilter_class_teletext = {
   .ic_super      = &esfilter_class,
   .ic_class      = "esfilter_teletext",
-  .ic_caption    = "Teletext Stream Filter",
+  .ic_caption    = N_("Teletext Stream Filter"),
   .ic_properties = (const property_t[]){
     {
       .type     = PT_STR,
       .islist   = 1,
       .id       = "type",
-      .name     = "Stream Type",
+      .name     = N_("Stream type"),
+      .desc     = N_("Teletext stream type is only available for "
+                     "this filter."),
       .get      = esfilter_class_type_get,
       .set      = esfilter_class_type_set_teletext,
       .list     = esfilter_class_type_enum_teletext,
@@ -773,15 +830,20 @@ const idclass_t esfilter_class_teletext = {
     {
       .type     = PT_STR,
       .id       = "language",
-      .name     = "Language",
+      .name     = N_("Language"),
+      .desc     = N_("The language to which the filter should apply."),
       .get      = esfilter_class_language_get,
       .set      = esfilter_class_language_set,
       .list     = esfilter_class_language_enum,
+      .opts     = PO_DOC_NLIST,
     },
     {
       .type     = PT_STR,
       .id       = "service",
-      .name     = "Service",
+      .name     = N_("Service"),
+      .desc     = N_("The service the filter should apply to. "
+                     "Leave blank to apply the filter to all "
+                     "services."),
       .get      = esfilter_class_service_get,
       .set      = esfilter_class_service_set,
       .list     = esfilter_class_service_enum,
@@ -789,33 +851,54 @@ const idclass_t esfilter_class_teletext = {
     {
       .type     = PT_INT,
       .id       = "sindex",
-      .name     = "Stream Index",
+      .name     = N_("Stream index"),
+      .desc     = N_("The logical stream index to compare. Note that "
+                     "this index is computed using all filters."
+                     "Example: If filter is set to AC3 audio type and "
+                     "the language to 'eng' and there are two AC3 "
+                     "'eng' streams in the service, the first stream "
+                     "could be identified using number 1 and the "
+                     "second using number 2."),
       .off      = offsetof(esfilter_t, esf_sindex),
     },
     {
       .type     = PT_INT,
       .id       = "pid",
-      .name     = "PID",
+      .name     = N_("PID"),
+      .desc     = N_("Program identification (PID) number to compare. "
+                     "Zero means any. This comparison is processed "
+                     "only when service comparison is active and for "
+                     "the Conditional Access filter."),
       .off      = offsetof(esfilter_t, esf_pid),
     },
     {
       .type     = PT_INT,
       .id       = "action",
-      .name     = "Action",
+      .name     = N_("Action"),
+      .desc     = N_("The rule action defines the operation when all "
+                     "comparisons succeed. See Help for more "
+                     "information on what the various rules do."),
       .get      = esfilter_class_action_get,
       .set      = esfilter_class_action_set,
       .list     = esfilter_class_action_enum,
+      .opts     = PO_DOC_NLIST,
+      .doc      = prop_doc_action,
     },
     {
       .type     = PT_BOOL,
       .id       = "log",
-      .name     = "Log",
+      .name     = N_("Log"),
+      .desc     = N_("Write a short message to log identifying the "
+                     "matched parameters. It is useful for debugging "
+                     "your setup or structure of incoming streams."),
       .off      = offsetof(esfilter_t, esf_log),
     },
     {
       .type     = PT_STR,
       .id       = "comment",
-      .name     = "Comment",
+      .name     = N_("Comment"),
+      .desc     = N_("Free-format text field. Enter whatever you "
+                     "like here."),
       .off      = offsetof(esfilter_t, esf_comment),
     },
     {}
@@ -825,13 +908,15 @@ const idclass_t esfilter_class_teletext = {
 const idclass_t esfilter_class_subtit = {
   .ic_super      = &esfilter_class,
   .ic_class      = "esfilter_subtit",
-  .ic_caption    = "Subtitle Stream Filter",
+  .ic_caption    = N_("Subtitle Stream Filter"),
   .ic_properties = (const property_t[]){
     {
       .type     = PT_STR,
       .islist   = 1,
       .id       = "type",
-      .name     = "Stream Type",
+      .name     = N_("Stream type"),
+      .desc     = N_("The subtitle stream types the filter should "
+                     "apply to."),
       .get      = esfilter_class_type_get,
       .set      = esfilter_class_type_set_subtit,
       .list     = esfilter_class_type_enum_subtit,
@@ -840,15 +925,20 @@ const idclass_t esfilter_class_subtit = {
     {
       .type     = PT_STR,
       .id       = "language",
-      .name     = "Language",
+      .name     = N_("Language"),
+      .desc     = N_("The language to which the filter should apply."),
       .get      = esfilter_class_language_get,
       .set      = esfilter_class_language_set,
       .list     = esfilter_class_language_enum,
+      .opts     = PO_DOC_NLIST,
     },
     {
       .type     = PT_STR,
       .id       = "service",
-      .name     = "Service",
+      .name     = N_("Service"),
+      .desc     = N_("The service the filter should apply to. "
+                     "Leave blank to apply the filter to all "
+                     "services."),
       .get      = esfilter_class_service_get,
       .set      = esfilter_class_service_set,
       .list     = esfilter_class_service_enum,
@@ -856,33 +946,54 @@ const idclass_t esfilter_class_subtit = {
     {
       .type     = PT_INT,
       .id       = "sindex",
-      .name     = "Stream Index",
+      .name     = N_("Stream index"),
+      .desc     = N_("The logical stream index to compare. Note that "
+                     "this index is computed using all filters."
+                     "Example: If filter is set to AC3 audio type and "
+                     "the language to 'eng' and there are two AC3 "
+                     "'eng' streams in the service, the first stream "
+                     "could be identified using number 1 and the "
+                     "second using number 2."),
       .off      = offsetof(esfilter_t, esf_sindex),
     },
     {
       .type     = PT_INT,
       .id       = "pid",
-      .name     = "PID",
+      .name     = N_("PID"),
+      .desc     = N_("Program identification (PID) number to compare. "
+                     "Zero means any. This comparison is processed "
+                     "only when service comparison is active and for "
+                     "the Conditional Access filter."),
       .off      = offsetof(esfilter_t, esf_pid),
     },
     {
       .type     = PT_INT,
       .id       = "action",
-      .name     = "Action",
+      .name     = N_("Action"),
+      .desc     = N_("The rule action defines the operation when all "
+                     "comparisons succeed. See Help for more "
+                     "information on what the various rules do."),
       .get      = esfilter_class_action_get,
       .set      = esfilter_class_action_set,
       .list     = esfilter_class_action_enum,
+      .opts     = PO_DOC_NLIST,
+      .doc      = prop_doc_action,
     },
     {
       .type     = PT_BOOL,
       .id       = "log",
-      .name     = "Log",
+      .name     = N_("Log"),
+      .desc     = N_("Write a short message to log identifying the "
+                     "matched parameters. It is useful for debugging "
+                     "your setup or structure of incoming streams."),
       .off      = offsetof(esfilter_t, esf_log),
     },
     {
       .type     = PT_STR,
       .id       = "comment",
-      .name     = "Comment",
+      .name     = N_("Comment"),
+      .desc     = N_("Free-format text field. Enter whatever you "
+                     "like here."),
       .off      = offsetof(esfilter_t, esf_comment),
     },
     {}
@@ -892,13 +1003,15 @@ const idclass_t esfilter_class_subtit = {
 const idclass_t esfilter_class_ca = {
   .ic_super      = &esfilter_class,
   .ic_class      = "esfilter_ca",
-  .ic_caption    = "CA Stream Filter",
+  .ic_caption    = N_("CA Stream Filter"),
   .ic_properties = (const property_t[]){
     {
       .type     = PT_STR,
       .islist   = 1,
       .id       = "type",
-      .name     = "Stream Type",
+      .name     = N_("Stream type"),
+      .desc     = N_("The CA stream type is only available for this "
+                     "filter."),
       .get      = esfilter_class_type_get,
       .set      = esfilter_class_type_set_ca,
       .list     = esfilter_class_type_enum_ca,
@@ -907,7 +1020,9 @@ const idclass_t esfilter_class_ca = {
     {
       .type     = PT_STR,
       .id       = "CAid",
-      .name     = "CA Identification",
+      .name     = N_("CA identification"),
+      .desc     = N_("The CAID to compare. Leave blank to apply to "
+                     "all IDs."),
       .get      = esfilter_class_caid_get,
       .set      = esfilter_class_caid_set,
       .list     = esfilter_class_caid_enum,
@@ -915,7 +1030,9 @@ const idclass_t esfilter_class_ca = {
     {
       .type     = PT_STR,
       .id       = "CAprovider",
-      .name     = "CA Provider",
+      .name     = N_("CA provider"),
+      .desc     = N_("The CA provider to compare. Leave blank to apply "
+                     "to all providers."),
       .get      = esfilter_class_caprovider_get,
       .set      = esfilter_class_caprovider_set,
       .list     = esfilter_class_caprovider_enum,
@@ -923,7 +1040,10 @@ const idclass_t esfilter_class_ca = {
     {
       .type     = PT_STR,
       .id       = "service",
-      .name     = "Service",
+      .name     = N_("Service"),
+      .desc     = N_("The service the filter should apply to. "
+                     "Leave blank to apply the filter to all "
+                     "services."),
       .get      = esfilter_class_service_get,
       .set      = esfilter_class_service_set,
       .list     = esfilter_class_service_enum,
@@ -931,33 +1051,54 @@ const idclass_t esfilter_class_ca = {
     {
       .type     = PT_INT,
       .id       = "sindex",
-      .name     = "Stream Index",
+      .name     = N_("Stream index"),
+      .desc     = N_("The logical stream index to compare. Note that "
+                     "this index is computed using all filters."
+                     "Example: If filter is set to AC3 audio type and "
+                     "the language to 'eng' and there are two AC3 "
+                     "'eng' streams in the service, the first stream "
+                     "could be identified using number 1 and the "
+                     "second using number 2."),
       .off      = offsetof(esfilter_t, esf_sindex),
     },
     {
       .type     = PT_INT,
       .id       = "pid",
-      .name     = "PID",
+      .name     = N_("PID"),
+      .desc     = N_("Program identification (PID) number to compare. "
+                     "Zero means any. This comparison is processed "
+                     "only when service comparison is active and for "
+                     "the Conditional Access filter."),
       .off      = offsetof(esfilter_t, esf_pid),
     },
     {
       .type     = PT_INT,
       .id       = "action",
-      .name     = "Action",
+      .name     = N_("Action"),
+      .desc     = N_("The rule action defines the operation when all "
+                     "comparisons succeed. See Help for more "
+                     "information on what the various rules do."),
       .get      = esfilter_class_action_get,
       .set      = esfilter_class_action_set,
       .list     = esfilter_class_action_enum,
+      .opts     = PO_DOC_NLIST,
+      .doc      = prop_doc_action,
     },
     {
       .type     = PT_BOOL,
       .id       = "log",
-      .name     = "Log",
+      .name     = N_("Log"),
+      .desc     = N_("Write a short message to log identifying the "
+                     "matched parameters. It is useful for debugging "
+                     "your setup or structure of incoming streams."),
       .off      = offsetof(esfilter_t, esf_log),
     },
     {
       .type     = PT_STR,
       .id       = "comment",
-      .name     = "Comment",
+      .name     = N_("Comment"),
+      .desc     = N_("Free-format text field. Enter whatever you "
+                     "like here."),
       .off      = offsetof(esfilter_t, esf_comment),
     },
     {}
@@ -967,13 +1108,15 @@ const idclass_t esfilter_class_ca = {
 const idclass_t esfilter_class_other = {
   .ic_super      = &esfilter_class,
   .ic_class      = "esfilter_other",
-  .ic_caption    = "Other Stream Filter",
+  .ic_caption    = N_("Other Stream Filter"),
   .ic_properties = (const property_t[]){
     {
       .type     = PT_STR,
       .islist   = 1,
       .id       = "type",
-      .name     = "Stream Type",
+      .name     = N_("Stream type"),
+      .desc     = N_("The MPEGTS stream type is only available for "
+                     "this filter."),
       .get      = esfilter_class_type_get,
       .set      = esfilter_class_type_set_other,
       .list     = esfilter_class_type_enum_other,
@@ -982,15 +1125,20 @@ const idclass_t esfilter_class_other = {
     {
       .type     = PT_STR,
       .id       = "language",
-      .name     = "Language",
+      .name     = N_("Language"),
+      .desc     = N_("The language to which the filter should apply."),
       .get      = esfilter_class_language_get,
       .set      = esfilter_class_language_set,
       .list     = esfilter_class_language_enum,
+      .opts     = PO_DOC_NLIST,
     },
     {
       .type     = PT_STR,
       .id       = "service",
-      .name     = "Service",
+      .name     = N_("Service"),
+      .desc     = N_("The service the filter should apply to. "
+                     "Leave blank to apply the filter to all "
+                     "services."),
       .get      = esfilter_class_service_get,
       .set      = esfilter_class_service_set,
       .list     = esfilter_class_service_enum,
@@ -998,27 +1146,41 @@ const idclass_t esfilter_class_other = {
     {
       .type     = PT_INT,
       .id       = "pid",
-      .name     = "PID",
+      .name     = N_("PID"),
+      .desc     = N_("Program identification (PID) number to compare. "
+                     "Zero means any. This comparison is processed "
+                     "only when service comparison is active and for "
+                     "the Conditional Access filter."),
       .off      = offsetof(esfilter_t, esf_pid),
     },
     {
       .type     = PT_INT,
       .id       = "action",
-      .name     = "Action",
+      .name     = N_("Action"),
+      .desc     = N_("The rule action defines the operation when all "
+                     "comparisons succeed. See Help for more "
+                     "information on what the various rules do."),
       .get      = esfilter_class_action_get,
       .set      = esfilter_class_action_set,
       .list     = esfilter_class_action_enum,
+      .opts     = PO_DOC_NLIST,
+      .doc      = prop_doc_action,
     },
     {
       .type     = PT_BOOL,
       .id       = "log",
-      .name     = "Log",
+      .name     = N_("Log"),
+      .desc     = N_("Write a short message to log identifying the "
+                     "matched parameters. It is useful for debugging "
+                     "your setup or structure of incoming streams."),
       .off      = offsetof(esfilter_t, esf_log),
     },
     {
       .type     = PT_STR,
       .id       = "comment",
-      .name     = "Comment",
+      .name     = N_("Comment"),
+      .desc     = N_("Free-format text field. Enter whatever you "
+                     "like here."),
       .off      = offsetof(esfilter_t, esf_comment),
     },
     {}
@@ -1035,8 +1197,11 @@ esfilter_init(void)
   htsmsg_field_t *f;
   int i;
 
-  for (i = 0; i <= ESF_CLASS_LAST; i++)
+  for (i = 0; i <= ESF_CLASS_LAST; i++) {
     TAILQ_INIT(&esfilters[i]);
+    if (esfilter_classes[i])
+      idclass_register(esfilter_classes[i]);
+  }
 
   if (!(c = hts_settings_load("esfilter")))
     return;
