@@ -23,22 +23,26 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include "build.h"
 #include "htsmsg.h"
 #include "misc/dbl.h"
 #include "htsmsg_json.h"
+#include "memoryinfo.h"
+
+#if ENABLE_SLOW_MEMORYINFO
+memoryinfo_t htsmsg_memoryinfo = { .my_name = "htsmsg" };
+memoryinfo_t htsmsg_field_memoryinfo = { .my_name = "htsmsg field" };
+#endif
 
 static void htsmsg_clear(htsmsg_t *msg);
-static htsmsg_t *
-htsmsg_field_get_msg ( htsmsg_field_t *f, int islist );
+static htsmsg_t *htsmsg_field_get_msg ( htsmsg_field_t *f, int islist );
 
 /**
  *
  */
-void
-htsmsg_field_destroy(htsmsg_t *msg, htsmsg_field_t *f)
+static void
+htsmsg_field_data_destroy(htsmsg_field_t *f)
 {
-  TAILQ_REMOVE(&msg->hm_fields, f, hmf_link);
-
   switch(f->hmf_type) {
   case HMF_MAP:
   case HMF_LIST:
@@ -46,19 +50,52 @@ htsmsg_field_destroy(htsmsg_t *msg, htsmsg_field_t *f)
     break;
 
   case HMF_STR:
-    if(f->hmf_flags & HMF_ALLOCED)
+    if(f->hmf_flags & HMF_ALLOCED) {
+#if ENABLE_SLOW_MEMORYINFO
+      memoryinfo_remove(&htsmsg_field_memoryinfo, strlen(f->hmf_str) + 1);
+#endif
       free((void *)f->hmf_str);
+    }
     break;
 
   case HMF_BIN:
-    if(f->hmf_flags & HMF_ALLOCED)
+    if(f->hmf_flags & HMF_ALLOCED) {
+#if ENABLE_SLOW_MEMORYINFO
+      memoryinfo_remove(&htsmsg_field_memoryinfo, f->hmf_binsize);
+#endif
       free((void *)f->hmf_bin);
+    }
     break;
   default:
     break;
   }
-  if(f->hmf_flags & HMF_NAME_ALLOCED)
+  f->hmf_flags &= ~HMF_ALLOCED;
+}
+
+/**
+ *
+ */
+void
+htsmsg_field_destroy(htsmsg_t *msg, htsmsg_field_t *f)
+{
+#if ENABLE_SLOW_MEMORYINFO
+  size_t asize = 0;
+#endif
+
+  TAILQ_REMOVE(&msg->hm_fields, f, hmf_link);
+
+  htsmsg_field_data_destroy(f);
+
+  if (f->hmf_flags & HMF_NAME_ALLOCED) {
+#if ENABLE_SLOW_MEMORYINFO
+    asize += strlen(f->hmf_name);
+#endif
     free((void *)f->hmf_name);
+  }
+#if ENABLE_SLOW_MEMORYINFO
+  memoryinfo_free(&htsmsg_field_memoryinfo,
+                  sizeof(*f) + f->hmf_edata_size + asize);
+#endif
   free(f);
 }
 
@@ -75,15 +112,23 @@ htsmsg_clear(htsmsg_t *msg)
 }
 
 
-
 /*
  *
  */
 htsmsg_field_t *
-htsmsg_field_add(htsmsg_t *msg, const char *name, int type, int flags)
+htsmsg_field_add(htsmsg_t *msg, const char *name, int type, int flags, size_t esize)
 {
-  htsmsg_field_t *f = malloc(sizeof(htsmsg_field_t));
+  size_t nsize = 0;
+  htsmsg_field_t *f;
+#if ENABLE_SLOW_MEMORYINFO
+  size_t asize = 0;
+#endif
   
+  if((flags & HMF_NAME_INALLOCED) && name)
+    nsize = strlen(name) + 1;
+  f = malloc(sizeof(htsmsg_field_t) + nsize + esize);
+  if(f == NULL)
+    return NULL;
   TAILQ_INSERT_TAIL(&msg->hm_fields, f, hmf_link);
 
   if(msg->hm_islist) {
@@ -92,13 +137,36 @@ htsmsg_field_add(htsmsg_t *msg, const char *name, int type, int flags)
     assert(name != NULL);
   }
 
-  if(flags & HMF_NAME_ALLOCED)
+  if(flags & HMF_NAME_INALLOCED) {
+    if (name) {
+      f->hmf_name = f->hmf_edata;
+      strcpy(f->hmf_edata, name);
+    } else {
+      f->hmf_name = NULL;
+    }
+  } else if(flags & HMF_NAME_ALLOCED) {
     f->hmf_name = name ? strdup(name) : NULL;
-  else
+#if ENABLE_SLOW_MEMORYINFO
+    asize = name ? strlen(name) + 1 : 0;
+#endif
+  } else {
     f->hmf_name = name;
+  }
+
+  if(esize) {
+    if(type == HMF_STR)
+      f->hmf_str = f->hmf_edata + nsize;
+    else if(type == HMF_BIN)
+      f->hmf_bin = f->hmf_edata + nsize;
+  }
 
   f->hmf_type = type;
   f->hmf_flags = flags;
+#if ENABLE_SLOW_MEMORYINFO
+  f->hmf_edata_size = nsize + esize;
+  memoryinfo_alloc(&htsmsg_field_memoryinfo,
+                   sizeof(htsmsg_field_t) + f->hmf_edata_size + asize);
+#endif
   return f;
 }
 
@@ -121,6 +189,17 @@ htsmsg_field_find(htsmsg_t *msg, const char *name)
 }
 
 
+/*
+ *
+ */
+htsmsg_field_t *
+htsmsg_field_last(htsmsg_t *msg)
+{
+  if (msg == NULL)
+    return NULL;
+  return TAILQ_LAST(&msg->hm_fields, htsmsg_field_queue);
+}
+
 
 /**
  *
@@ -137,6 +216,20 @@ htsmsg_delete_field(htsmsg_t *msg, const char *name)
 }
 
 
+/**
+ *
+ */
+int
+htsmsg_is_empty(htsmsg_t *msg)
+{
+  if (msg == NULL)
+    return 1;
+
+  assert(msg->hm_data == NULL);
+  return TAILQ_EMPTY(&msg->hm_fields);
+}
+
+
 /*
  *
  */
@@ -146,9 +239,15 @@ htsmsg_create_map(void)
   htsmsg_t *msg;
 
   msg = malloc(sizeof(htsmsg_t));
-  TAILQ_INIT(&msg->hm_fields);
-  msg->hm_data = NULL;
-  msg->hm_islist = 0;
+  if (msg) {
+    TAILQ_INIT(&msg->hm_fields);
+    msg->hm_data = NULL;
+    msg->hm_data_size = 0;
+    msg->hm_islist = 0;
+#if ENABLE_SLOW_MEMORYINFO
+    memoryinfo_alloc(&htsmsg_memoryinfo, sizeof(htsmsg_t));
+#endif
+  }
   return msg;
 }
 
@@ -161,9 +260,15 @@ htsmsg_create_list(void)
   htsmsg_t *msg;
 
   msg = malloc(sizeof(htsmsg_t));
-  TAILQ_INIT(&msg->hm_fields);
-  msg->hm_data = NULL;
-  msg->hm_islist = 1;
+  if (msg) {
+    TAILQ_INIT(&msg->hm_fields);
+    msg->hm_data = NULL;
+    msg->hm_data_size = 0;
+    msg->hm_islist = 1;
+#if ENABLE_SLOW_MEMORYINFO
+    memoryinfo_alloc(&htsmsg_memoryinfo, sizeof(htsmsg_t));
+#endif
+  }
   return msg;
 }
 
@@ -174,11 +279,23 @@ htsmsg_create_list(void)
 void
 htsmsg_destroy(htsmsg_t *msg)
 {
+#if ENABLE_SLOW_MEMORYINFO
+  size_t size = 0;
+#endif
+
   if(msg == NULL)
     return;
 
   htsmsg_clear(msg);
-  free((void *)msg->hm_data);
+  if (msg->hm_data) {
+    free((void *)msg->hm_data);
+#if ENABLE_SLOW_MEMORYINFO
+    size += msg->hm_data_size;
+#endif
+  }
+#if ENABLE_SLOW_MEMORYINFO
+  memoryinfo_free(&htsmsg_memoryinfo, sizeof(htsmsg_t) + size);
+#endif
   free(msg);
 }
 
@@ -188,7 +305,19 @@ htsmsg_destroy(htsmsg_t *msg)
 void
 htsmsg_add_bool(htsmsg_t *msg, const char *name, int b)
 {
-  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_BOOL, HMF_NAME_ALLOCED);
+  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_BOOL, HMF_NAME_INALLOCED, 0);
+  f->hmf_bool = !!b;
+}
+
+/*
+ *
+ */
+void
+htsmsg_set_bool(htsmsg_t *msg, const char *name, int b)
+{
+  htsmsg_field_t *f = htsmsg_field_find(msg, name);
+  if (!f)
+    f = htsmsg_field_add(msg, name, HMF_BOOL, HMF_NAME_INALLOCED, 0);
   f->hmf_bool = !!b;
 }
 
@@ -198,7 +327,7 @@ htsmsg_add_bool(htsmsg_t *msg, const char *name, int b)
 void
 htsmsg_add_s64(htsmsg_t *msg, const char *name, int64_t s64)
 {
-  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_S64, HMF_NAME_ALLOCED);
+  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_S64, HMF_NAME_INALLOCED, 0);
   f->hmf_s64 = s64;
 }
 
@@ -210,7 +339,7 @@ htsmsg_set_s64(htsmsg_t *msg, const char *name, int64_t s64)
 {
   htsmsg_field_t *f = htsmsg_field_find(msg, name);
   if (!f)
-    f = htsmsg_field_add(msg, name, HMF_S64, HMF_NAME_ALLOCED);
+    f = htsmsg_field_add(msg, name, HMF_S64, HMF_NAME_INALLOCED, 0);
   if (f->hmf_type != HMF_S64)
     return 1;
   f->hmf_s64 = s64;
@@ -224,7 +353,7 @@ htsmsg_set_s64(htsmsg_t *msg, const char *name, int64_t s64)
 void
 htsmsg_add_dbl(htsmsg_t *msg, const char *name, double dbl)
 {
-  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_DBL, HMF_NAME_ALLOCED);
+  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_DBL, HMF_NAME_INALLOCED, 0);
   f->hmf_dbl = dbl;
 }
 
@@ -236,9 +365,82 @@ htsmsg_add_dbl(htsmsg_t *msg, const char *name, double dbl)
 void
 htsmsg_add_str(htsmsg_t *msg, const char *name, const char *str)
 {
-  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_STR, 
-				        HMF_ALLOCED | HMF_NAME_ALLOCED);
+  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_STR, HMF_NAME_INALLOCED,
+                                       strlen(str) + 1);
+  strcpy((char *)f->hmf_str, str);
+  f->hmf_flags |= HMF_INALLOCED;
+}
+
+/*
+ *
+ */
+void
+htsmsg_add_str2(htsmsg_t *msg, const char *name, const char *str)
+{
+  if (msg && name && str)
+    htsmsg_add_str(msg, name, str);
+}
+
+/*
+ *
+ */
+void
+htsmsg_add_str_exclusive(htsmsg_t *msg, const char *str)
+{
+  htsmsg_field_t *f;
+
+  assert(msg->hm_islist);
+
+  TAILQ_FOREACH(f, &msg->hm_fields, hmf_link) {
+    assert(f->hmf_type == HMF_STR);
+    if (strcmp(f->hmf_str, str) == 0)
+      return;
+  }
+
+  htsmsg_add_str(msg, NULL, str);
+}
+
+/*
+ *
+ */
+int
+htsmsg_field_set_str(htsmsg_field_t *f, const char *str)
+{
+  if (f->hmf_type != HMF_STR)
+    return 1;
+  if (f->hmf_flags & HMF_ALLOCED) {
+#if ENABLE_SLOW_MEMORYINFO
+    memoryinfo_remove(&htsmsg_field_memoryinfo, strlen(f->hmf_str) + 1);
+#endif
+    free((void *)f->hmf_str);
+  }
+  else if (f->hmf_flags & HMF_INALLOCED) {
+    if (strlen(f->hmf_str) >= strlen(str)) {
+      strcpy((char *)f->hmf_str, str);
+      return 0;
+    }
+    f->hmf_flags &= ~HMF_INALLOCED;
+  }
+  f->hmf_flags |= HMF_ALLOCED;
   f->hmf_str = strdup(str);
+#if ENABLE_SLOW_MEMORYINFO
+  memoryinfo_alloc(&htsmsg_field_memoryinfo, strlen(str) + 1);
+#endif
+  return 0;
+}
+
+/*
+ *
+ */
+int
+htsmsg_field_set_str_force(htsmsg_field_t *f, const char *str)
+{
+  if (f->hmf_type != HMF_STR) {
+    htsmsg_field_data_destroy(f);
+    f->hmf_type = HMF_STR;
+    f->hmf_str = "";
+  }
+  return htsmsg_field_set_str(f, str);
 }
 
 /*
@@ -248,16 +450,11 @@ int
 htsmsg_set_str(htsmsg_t *msg, const char *name, const char *str)
 {
   htsmsg_field_t *f = htsmsg_field_find(msg, name);
-  if (!f)
-    f = htsmsg_field_add(msg, name, HMF_STR, HMF_ALLOCED | HMF_NAME_ALLOCED);
-  else {
-    if (f->hmf_type != HMF_STR)
-      return 1;
-    if(f->hmf_flags & HMF_ALLOCED)
-      free((void *)f->hmf_str);
+  if (!f) {
+    htsmsg_add_str(msg, name, str);
+    return 0;
   }
-  f->hmf_str = strdup(str);
-  return 0;
+  return htsmsg_field_set_str(f, str);
 }
 
 /*
@@ -266,12 +463,11 @@ htsmsg_set_str(htsmsg_t *msg, const char *name, const char *str)
 void
 htsmsg_add_bin(htsmsg_t *msg, const char *name, const void *bin, size_t len)
 {
-  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_BIN, 
-				       HMF_ALLOCED | HMF_NAME_ALLOCED);
-  void *v;
-  f->hmf_bin = v = malloc(len);
+  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_BIN, HMF_NAME_INALLOCED, len);
+  f->hmf_bin = f->hmf_str;
   f->hmf_binsize = len;
-  memcpy(v, bin, len);
+  f->hmf_flags |= HMF_INALLOCED;
+  memcpy((void *)f->hmf_bin, bin, len);
 }
 
 /*
@@ -280,12 +476,34 @@ htsmsg_add_bin(htsmsg_t *msg, const char *name, const void *bin, size_t len)
 void
 htsmsg_add_binptr(htsmsg_t *msg, const char *name, const void *bin, size_t len)
 {
-  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_BIN, HMF_NAME_ALLOCED);
+  htsmsg_field_t *f = htsmsg_field_add(msg, name, HMF_BIN, HMF_NAME_INALLOCED, 0);
   f->hmf_bin = bin;
   f->hmf_binsize = len;
 }
 
 
+/*
+ *
+ */
+static htsmsg_t *
+htsmsg_field_set_msg(htsmsg_field_t *f, htsmsg_t *sub)
+{
+  assert(sub->hm_data == NULL);
+  f->hmf_msg.hm_data = NULL;
+  f->hmf_msg.hm_data_size = 0;
+  f->hmf_msg.hm_islist = sub->hm_islist;
+  TAILQ_MOVE(&f->hmf_msg.hm_fields, &sub->hm_fields, hmf_link);
+#if ENABLE_SLOW_MEMORYINFO
+  memoryinfo_free(&htsmsg_memoryinfo, sizeof(htsmsg_t));
+#endif
+  free(sub);
+
+  if (f->hmf_type == (f->hmf_msg.hm_islist ? HMF_LIST : HMF_MAP))
+    return &f->hmf_msg;
+
+  return NULL;
+}
+
 /*
  *
  */
@@ -295,18 +513,21 @@ htsmsg_add_msg(htsmsg_t *msg, const char *name, htsmsg_t *sub)
   htsmsg_field_t *f;
 
   f = htsmsg_field_add(msg, name, sub->hm_islist ? HMF_LIST : HMF_MAP,
-		       HMF_NAME_ALLOCED);
+		       HMF_NAME_INALLOCED, 0);
+  return htsmsg_field_set_msg(f, sub);
+}
 
-  assert(sub->hm_data == NULL);
-  f->hmf_msg.hm_data = NULL;
-  f->hmf_msg.hm_islist = sub->hm_islist;
-  TAILQ_MOVE(&f->hmf_msg.hm_fields, &sub->hm_fields, hmf_link);
-  free(sub);
-
-  if (f->hmf_type == (f->hmf_msg.hm_islist ? HMF_LIST : HMF_MAP))
-    return &f->hmf_msg;
-
-  return NULL;
+/*
+ *
+ */
+htsmsg_t *
+htsmsg_set_msg(htsmsg_t *msg, const char *name, htsmsg_t *sub)
+{
+  htsmsg_field_t *f = htsmsg_field_find(msg, name);
+  if (!f)
+    return htsmsg_add_msg(msg, name, sub);
+  htsmsg_field_data_destroy(f);
+  return htsmsg_field_set_msg(f, sub);
 }
 
 
@@ -319,12 +540,16 @@ htsmsg_add_msg_extname(htsmsg_t *msg, const char *name, htsmsg_t *sub)
 {
   htsmsg_field_t *f;
 
-  f = htsmsg_field_add(msg, name, sub->hm_islist ? HMF_LIST : HMF_MAP, 0);
+  f = htsmsg_field_add(msg, name, sub->hm_islist ? HMF_LIST : HMF_MAP, 0, 0);
 
   assert(sub->hm_data == NULL);
   f->hmf_msg.hm_data = NULL;
+  f->hmf_msg.hm_data_size = 0;
   TAILQ_MOVE(&f->hmf_msg.hm_fields, &sub->hm_fields, hmf_link);
   f->hmf_msg.hm_islist = sub->hm_islist;
+#if ENABLE_SLOW_MEMORYINFO
+  memoryinfo_free(&htsmsg_memoryinfo, sizeof(htsmsg_t));
+#endif
   free(sub);
 }
 
@@ -582,21 +807,15 @@ htsmsg_field_get_string(htsmsg_field_t *f)
   case HMF_STR:
     break;
   case HMF_BOOL:
-    f->hmf_str = strdup(f->hmf_bool ? "true" : "false");
-    f->hmf_type = HMF_STR;
-    f->hmf_flags |= HMF_ALLOCED;
+    htsmsg_field_set_str_force(f, f->hmf_bool ? "true" : "false");
     break;
   case HMF_S64:
     snprintf(buf, sizeof(buf), "%"PRId64, f->hmf_s64);
-    f->hmf_str = strdup(buf);
-    f->hmf_type = HMF_STR;
-    f->hmf_flags |= HMF_ALLOCED;
+    htsmsg_field_set_str_force(f, buf);
     break;
   case HMF_DBL:
     snprintf(buf, sizeof(buf), "%lf", f->hmf_dbl);
-    f->hmf_str = strdup(buf);
-    f->hmf_type = HMF_STR;
-    f->hmf_flags |= HMF_ALLOCED;
+    htsmsg_field_set_str_force(f, buf);
     break;
   }
   return f->hmf_str;
@@ -710,11 +929,20 @@ htsmsg_field_get_msg ( htsmsg_field_t *f, int islist )
   /* Deserialize JSON (will keep either list or map) */
   if (f->hmf_type == HMF_STR) {
     if ((m = htsmsg_json_deserialize(f->hmf_str))) {
-      free((void*)f->hmf_str);
+      if (f->hmf_flags & HMF_ALLOCED) {
+#if ENABLE_SLOW_MEMORYINFO
+        memoryinfo_free(&htsmsg_field_memoryinfo, strlen(f->hmf_str) + 1);
+#endif
+        free((void*)f->hmf_str);
+      }
       f->hmf_type          = m->hm_islist ? HMF_LIST : HMF_MAP;
       f->hmf_msg.hm_islist = m->hm_islist;
       f->hmf_msg.hm_data   = NULL;
+      f->hmf_msg.hm_data_size = 0;
       TAILQ_MOVE(&f->hmf_msg.hm_fields, &m->hm_fields, hmf_link);
+#if ENABLE_SLOW_MEMORYINFO
+      memoryinfo_free(&htsmsg_memoryinfo, sizeof(htsmsg_t));
+#endif
       free(m);
     }
   }
@@ -854,11 +1082,81 @@ htsmsg_copy_i(htsmsg_t *src, htsmsg_t *dst)
 htsmsg_t *
 htsmsg_copy(htsmsg_t *src)
 {
-  htsmsg_t *dst = src->hm_islist ? htsmsg_create_list() : htsmsg_create_map();
+  htsmsg_t *dst;
+  if (src == NULL) return NULL;
+  dst = src->hm_islist ? htsmsg_create_list() : htsmsg_create_map();
   htsmsg_copy_i(src, dst);
   return dst;
 }
 
+/**
+ *
+ */
+int
+htsmsg_cmp(htsmsg_t *m1, htsmsg_t *m2)
+{
+  htsmsg_field_t *f1, *f2;
+
+  if (m1 == NULL && m2 == NULL)
+    return 0;
+  if (m1 == NULL || m2 == NULL)
+    return 1;
+
+  f2 = TAILQ_FIRST(&m2->hm_fields);
+  TAILQ_FOREACH(f1, &m1->hm_fields, hmf_link) {
+
+    if (f2 == NULL)
+      return 1;
+
+    if (f1->hmf_type != f2->hmf_type)
+      return 1;
+    if (strcmp(f1->hmf_name ?: "", f2->hmf_name ?: ""))
+      return 1;
+
+    switch(f1->hmf_type) {
+
+    case HMF_MAP:
+    case HMF_LIST:
+      if (htsmsg_cmp(&f1->hmf_msg, &f2->hmf_msg))
+        return 1;
+      break;
+      
+    case HMF_STR:
+      if (strcmp(f1->hmf_str, f2->hmf_str))
+        return 1;
+      break;
+
+    case HMF_S64:
+      if (f1->hmf_s64 != f2->hmf_s64)
+        return 1;
+      break;
+
+    case HMF_BOOL:
+      if (f1->hmf_bool != f2->hmf_bool)
+        return 1;
+      break;
+
+    case HMF_BIN:
+      if (f1->hmf_binsize != f2->hmf_binsize)
+        return 1;
+      if (memcmp(f1->hmf_bin, f2->hmf_bin, f1->hmf_binsize))
+        return 1;
+      break;
+
+    case HMF_DBL:
+      if (f1->hmf_dbl != f2->hmf_dbl)
+        return 1;
+      break;
+    }
+
+    f2 = TAILQ_NEXT(f2, hmf_link);
+  }
+
+  if (f2)
+    return 1;
+  return 0;
+}
+
 /**
  *
  */
@@ -904,15 +1202,15 @@ htsmsg_get_cdata(htsmsg_t *m, const char *field)
  * Note: this will NOT work for lists of complex types
  */
 char *
-htsmsg_list_2_csv(htsmsg_t *m)
+htsmsg_list_2_csv(htsmsg_t *m, char delim, int human)
 {
   int alloc, used, first = 1;
   char *ret;
   htsmsg_field_t *f;
-  const char *sep = ", ";
+  char sep[3];
+  const char *ssep;
   if (!m->hm_islist) return NULL;
 
-#define MAX(a,b) ((a) < (b)) ? (a) : (b)
 #define REALLOC(l)\
   if ((alloc - used) < l) {\
     alloc = MAX((l)*2, alloc*2);\
@@ -922,15 +1220,29 @@ htsmsg_list_2_csv(htsmsg_t *m)
   ret  = malloc(alloc = 512);
   *ret = 0;
   used = 0;
+  if (human) {
+    sep[0] = delim;
+    if (human & 2) {
+      sep[1] = '\0';
+    } else {
+      sep[1] = ' ';
+      sep[2] = '\0';
+    }
+    ssep = "";
+  } else {
+    sep[0] = delim;
+    sep[1] = '\0';
+    ssep = "\"";
+  }
   HTSMSG_FOREACH(f, m) {
     if (f->hmf_type == HMF_STR) {
-      REALLOC(2 + strlen(f->hmf_str));
-      used += sprintf(ret+used, "%s%s", !first ? sep : "", f->hmf_str);
+      REALLOC(4 + strlen(f->hmf_str));
+      used += sprintf(ret+used, "%s%s%s%s", !first ? sep : "", ssep, f->hmf_str, ssep);
     } else if (f->hmf_type == HMF_S64) {
       REALLOC(34); // max length is actually 20 chars + 2
       used += sprintf(ret+used, "%s%"PRId64, !first ? sep : "", f->hmf_s64);
     } else if (f->hmf_type == HMF_BOOL) {
-      REALLOC(2); // max length is actually 20 chars + 2
+      REALLOC(12); // max length is actually 10 chars + 2
       used += sprintf(ret+used, "%s%d", !first ? sep : "", f->hmf_bool);
     } else {
       // TODO: handle doubles
@@ -943,4 +1255,48 @@ htsmsg_list_2_csv(htsmsg_t *m)
   return ret;
 }
 
+htsmsg_t *
+htsmsg_csv_2_list(const char *str, char delim)
+{
+  char *tokbuf, *tok, *saveptr = NULL, *p;
+  const char d[2] = { delim, '\0' };
+  htsmsg_t *m = htsmsg_create_list();
 
+  if (str) {
+    tokbuf = strdup(str);
+    tok = strtok_r(tokbuf, d, &saveptr);
+    while (tok) {
+      if (tok[0] == '"') {
+        p = ++tok;
+        while (*p && *p != '"') {
+          if (*p == '\\') {
+            p++;
+            if (*p)
+              p++;
+            continue;
+          }
+          p++;
+        }
+        *p = '\0';
+      }
+      htsmsg_add_str(m, NULL, tok);
+      tok = strtok_r(NULL, ",", &saveptr);
+    }
+    free(tokbuf);
+  }
+  return m;
+}
+
+/*
+ *
+ */
+htsmsg_t *
+htsmsg_create_key_val(const char *key, const char *val)
+{
+  htsmsg_t *r = htsmsg_create_map();
+  if (r) {
+    htsmsg_add_str(r, "key", key);
+    htsmsg_add_str(r, "val", val);
+  }
+  return r;
+}
