@@ -300,7 +300,7 @@ htsmsg_xml_parse_attrib(xmlparser_t *xp, htsmsg_t *msg, char *src,
   attribname[attriblen] = 0;
   payload[payloadlen] = 0;
 
-  f = htsmsg_field_add(msg, attribname, HMF_STR, 0);
+  f = htsmsg_field_add(msg, attribname, HMF_STR, 0, 0);
   f->hmf_str = payload;
   return src;
 }
@@ -427,7 +427,7 @@ htsmsg_xml_parse_pi(xmlparser_t *xp, htsmsg_t *parent, char *src)
   while(1) {
     if(*src == 0) {
       xmlerr(xp, "Unexpected end of file during parsing of "
-	     "Processing instructions");
+	         "Processing instructions");
       return NULL;
     }
 
@@ -702,7 +702,7 @@ htsmsg_xml_parse_cd(xmlparser_t *xp, htsmsg_t *parent, char *src)
     assert(cc != NULL);
     assert(TAILQ_NEXT(cc, cc_link) == NULL);
     
-    f = htsmsg_field_add(parent, "cdata", HMF_STR, 0);
+    f = htsmsg_field_add(parent, "cdata", HMF_STR, 0, 0);
     f->hmf_str = cc->cc_start;
     *cc->cc_end = 0;
     free(cc);
@@ -731,7 +731,7 @@ htsmsg_xml_parse_cd(xmlparser_t *xp, htsmsg_t *parent, char *src)
     }
     body[c] = 0;
 
-    f = htsmsg_field_add(parent, "cdata", HMF_STR, HMF_ALLOCED);
+    f = htsmsg_field_add(parent, "cdata", HMF_STR, HMF_ALLOCED, 0);
     f->hmf_str = body;
 
   } else {
@@ -768,9 +768,7 @@ htsmsg_parse_prolog(xmlparser_t *xp, char *src)
   htsmsg_t *xmlpi;
   const char *encoding;
 
-  while(1) {
-    if(*src == 0)
-      break;
+  while(src != NULL && *src != 0) {
 
     while(is_xmlws(*src))
       src++;
@@ -849,6 +847,7 @@ htsmsg_xml_deserialize(char *src, char *errbuf, size_t errbufsize)
 
   if(xp.xp_srcdataused) {
     m->hm_data = src0;
+    m->hm_data_size = strlen(src0) + 1;
   } else {
     free(src0);
   }
