@@ -32,7 +32,7 @@
 #include "access.h"
 
 static const idnodes_rb_t * idnode_domain ( const idclass_t *idc );
-static void idclass_root_register ( idnode_t *in );
+static idnodes_rb_t * idclass_find_domain ( const idclass_t *idc );
 
 typedef struct idclass_link
 {
@@ -41,9 +41,15 @@ typedef struct idclass_link
   RB_ENTRY(idclass_link) link;
 } idclass_link_t;
 
-static idnodes_rb_t           idnodes;
-static RB_HEAD(,idclass_link) idclasses;
-static RB_HEAD(,idclass_link) idrootclasses;
+static idnodes_rb_t             idnodes;
+static RB_HEAD(,idclass_link)   idclasses;
+static RB_HEAD(,idclass_link)   idrootclasses;
+static TAILQ_HEAD(,idnode_save) idnodes_save;
+
+tvh_cond_t save_cond;
+pthread_t save_tid;
+static int save_running;
+static mtimer_t save_timer;
 
 SKEL_DECLARE(idclasses_skel, idclass_link_t);
 
@@ -57,7 +63,7 @@ SKEL_DECLARE(idclasses_skel, idclass_link_t);
 static int
 in_cmp(const idnode_t *a, const idnode_t *b)
 {
-  return memcmp(a->in_uuid, b->in_uuid, sizeof(a->in_uuid));
+  return uuid_cmp(&a->in_uuid, &b->in_uuid);
 }
 
 static int
@@ -72,35 +78,6 @@ ic_cmp ( const idclass_link_t *a, const idclass_link_t *b )
  * Registration
  * *************************************************************************/
 
-/**
- *
- */
-pthread_t idnode_tid;
-
-void
-idnode_init(void)
-{
-  RB_INIT(&idnodes);
-  RB_INIT(&idclasses);
-  RB_INIT(&idrootclasses);
-}
-
-void
-idnode_done(void)
-{
-  idclass_link_t *il;
-
-  while ((il = RB_FIRST(&idclasses)) != NULL) {
-    RB_REMOVE(&idclasses, il, link);
-    free(il);
-  }
-  while ((il = RB_FIRST(&idrootclasses)) != NULL) {
-    RB_REMOVE(&idrootclasses, il, link);
-    free(il);
-  }
-  SKEL_FREE(idclasses_skel);
-}
-
 static const idclass_t *
 idnode_root_class(const idclass_t *idc)
 {
@@ -119,7 +96,8 @@ idnode_insert(idnode_t *in, const char *uuid, const idclass_t *class, int flags)
   tvh_uuid_t u;
   int retries = 5;
   uint32_t u32;
-  const idclass_t *idc;;
+  const idclass_t *idc;
+  char ubuf[UUID_HEX_SIZE];
 
   lock_assert(&global_lock);
 
@@ -130,7 +108,7 @@ idnode_insert(idnode_t *in, const char *uuid, const idclass_t *class, int flags)
       in->in_class = NULL;
       return -1;
     }
-    memcpy(in->in_uuid, u.bin, sizeof(in->in_uuid));
+    uuid_duplicate(&in->in_uuid, &u);
 
     c = NULL;
     if (flags & IDNODE_SHORT_UUID) {
@@ -150,23 +128,25 @@ idnode_insert(idnode_t *in, const char *uuid, const idclass_t *class, int flags)
   } while (c != NULL && --retries > 0);
 
   if(c != NULL) {
+    tvherror(LS_IDNODE, "Id node collission (%s) %s",
+             uuid, (flags & IDNODE_SHORT_UUID) ? " (short)" : "");
     fprintf(stderr, "Id node collision (%s) %s\n",
             uuid, (flags & IDNODE_SHORT_UUID) ? " (short)" : "");
-    tvherror("idnode", "Id node collision (%s) %s\n",
-            uuid, (flags & IDNODE_SHORT_UUID) ? " (short)" : "");
     abort();
   }
-  tvhtrace("idnode", "insert node %s", idnode_uuid_as_str(in));
+  tvhtrace(LS_IDNODE, "insert node %s", idnode_uuid_as_str(in, ubuf));
 
   /* Register the class */
-  idclass_register(class); // Note: we never actually unregister
-  idclass_root_register(in);
-  assert(in->in_domain);
+  in->in_domain = idclass_find_domain(class);
+  if (in->in_domain == NULL) {
+    tvherror(LS_IDNODE, "class '%s' is not registered", class->ic_class);
+    abort();
+  }
   c = RB_INSERT_SORTED(in->in_domain, in, in_domain_link, in_cmp);
   assert(c == NULL);
 
   /* Fire event */
-  idnode_notify_changed(in);
+  idnode_notify(in, "create");
 
   return 0;
 }
@@ -177,11 +157,14 @@ idnode_insert(idnode_t *in, const char *uuid, const idclass_t *class, int flags)
 void
 idnode_unlink(idnode_t *in)
 {
+  char ubuf[UUID_HEX_SIZE];
+
   lock_assert(&global_lock);
   RB_REMOVE(&idnodes, in, in_link);
   RB_REMOVE(in->in_domain, in, in_domain_link);
-  tvhtrace("idnode", "unlink node %s", idnode_uuid_as_str(in));
+  tvhtrace(LS_IDNODE, "unlink node %s", idnode_uuid_as_str(in, ubuf));
   idnode_notify(in, "delete");
+  assert(in->in_save == NULL || in->in_save == SAVEPTR_OUTOFSERVICE);
 }
 
 /**
@@ -231,7 +214,7 @@ uint32_t
 idnode_get_short_uuid (const idnode_t *in)
 {
   uint32_t u32;
-  memcpy(&u32, in->in_uuid, sizeof(u32));
+  memcpy(&u32, in->in_uuid.bin, sizeof(u32));
   return u32 & 0x7FFFFFFF; // compat needs to be +ve signed
 }
 
@@ -239,28 +222,25 @@ idnode_get_short_uuid (const idnode_t *in)
  *
  */
 const char *
-idnode_uuid_as_str(const idnode_t *in)
+idnode_uuid_as_str(const idnode_t *in, char *uuid)
 {
-  static tvh_uuid_t __thread ret[16];
-  static uint8_t p = 0;
-  bin2hex(ret[p].hex, sizeof(ret[p].hex), in->in_uuid, sizeof(in->in_uuid));
-  const char *s = ret[p].hex;
-  p = (p + 1) % 16;
-  return s;
+  bin2hex(uuid, UUID_HEX_SIZE, in->in_uuid.bin, sizeof(in->in_uuid.bin));
+  return uuid;
 }
 
 /**
  *
  */
 const char *
-idnode_get_title(idnode_t *in)
+idnode_get_title(idnode_t *in, const char *lang)
 {
+  static char ubuf[UUID_HEX_SIZE];
   const idclass_t *ic = in->in_class;
   for(; ic != NULL; ic = ic->ic_super) {
     if(ic->ic_get_title != NULL)
-      return ic->ic_get_title(in);
+      return ic->ic_get_title(in, lang);
   }
-  return idnode_uuid_as_str(in);
+  return idnode_uuid_as_str(in, ubuf);
 }
 
 
@@ -327,20 +307,22 @@ idnode_find_prop
  */
 static char *
 idnode_get_display
-  ( idnode_t *self, const property_t *p )
+  ( idnode_t *self, const property_t *p, const char *lang )
 {
+  char *r = NULL;
   if (p) {
     if (p->rend)
-      return p->rend(self);
+      r = p->rend(self, lang);
     else if (p->islist) {
       htsmsg_t *l = (htsmsg_t*)p->get(self);
-      if (l)
-        return htsmsg_list_2_csv(l);
+      if (l) {
+        r = htsmsg_list_2_csv(l, ',', 1);
+        htsmsg_destroy(l);
+      }
     } else if (p->list) {
-      htsmsg_t *l = p->list(self), *m;
+      htsmsg_t *l = p->list(self, lang), *m;
       htsmsg_field_t *f;
       uint32_t k, v;
-      char *r = NULL;
       const char *s;
       if (l && !idnode_get_u32(self, p->id, &v))
         HTSMSG_FOREACH(f, l) {
@@ -353,10 +335,9 @@ idnode_get_display
           }
         }
       htsmsg_destroy(l);
-      return r;
     }
   }
-  return NULL;
+  return r;
 }
 
 /*
@@ -456,6 +437,33 @@ idnode_get_s64
   return 1;
 }
 
+/*
+ * Get field as signed 64-bit int
+ */
+int
+idnode_get_s64_atomic
+  ( idnode_t *self, const char *key, int64_t *s64 )
+{
+  const property_t *p = idnode_find_prop(self, key);
+  if (p) {
+    const void *ptr;
+    if (p->islist)
+      return 1;
+    else if (p->get)
+      ptr = p->get(self);
+    else
+      ptr = ((void*)self) + p->off;
+    switch (p->type) {
+      case PT_S64_ATOMIC:
+        *s64 = atomic_get_s64((int64_t*)ptr);
+        return 0;
+      default:
+        break;
+    }
+  }
+  return 1;
+}
+
 /*
  * Get field as double
  */
@@ -561,13 +569,22 @@ int
 idnode_perm(idnode_t *self, struct access *a, htsmsg_t *msg_to_write)
 {
   const idclass_t *ic = self->in_class;
+  int r;
 
   while (ic) {
     if (ic->ic_perm)
-      return self->in_class->ic_perm(self, a, msg_to_write);
-    if (ic->ic_perm_def)
-      return access_verify2(a, ic->ic_perm_def);
-    ic = ic->ic_super;
+      r = self->in_class->ic_perm(self, a, msg_to_write);
+    else if (ic->ic_perm_def)
+      r = access_verify2(a, ic->ic_perm_def);
+    else {
+      ic = ic->ic_super;
+      continue;
+    }
+    if (!r) {
+      self->in_access = a;
+      return 0;
+    }
+    return r;
   }
   return 0;
 }
@@ -596,10 +613,10 @@ idnode_find ( const char *uuid, const idclass_t *idc, const idnodes_rb_t *domain
 {
   idnode_t skel, *r;
 
-  tvhtrace("idnode", "find node %s class %s", uuid, idc ? idc->ic_class : NULL);
+  tvhtrace(LS_IDNODE, "find node %s class %s", uuid, idc ? idc->ic_class : NULL);
   if(uuid == NULL || strlen(uuid) != UUID_HEX_SIZE - 1)
     return NULL;
-  if(hex2bin(skel.in_uuid, sizeof(skel.in_uuid), uuid))
+  if(hex2bin(skel.in_uuid.bin, sizeof(skel.in_uuid.bin), uuid))
     return NULL;
   if (domain == NULL)
     domain = idnode_domain(idc);
@@ -623,7 +640,8 @@ idnode_find_all ( const idclass_t *idc, const idnodes_rb_t *domain )
 {
   idnode_t *in;
   const idclass_t *ic;
-  tvhtrace("idnode", "find class %s", idc->ic_class);
+  char ubuf[UUID_HEX_SIZE];
+  tvhtrace(LS_IDNODE, "find class %s", idc->ic_class);
   idnode_set_t *is = calloc(1, sizeof(idnode_set_t));
   if (domain == NULL)
     domain = idnode_domain(idc);
@@ -632,8 +650,8 @@ idnode_find_all ( const idclass_t *idc, const idnodes_rb_t *domain )
       ic = in->in_class;
       while (ic) {
         if (ic == idc) {
-          tvhtrace("idnode", "  add node %s", idnode_uuid_as_str(in));
-          idnode_set_add(is, in, NULL);
+          tvhtrace(LS_IDNODE, "  add node %s", idnode_uuid_as_str(in, ubuf));
+          idnode_set_add(is, in, NULL, NULL);
           break;
         }
         ic = ic->ic_super;
@@ -644,8 +662,8 @@ idnode_find_all ( const idclass_t *idc, const idnodes_rb_t *domain )
       ic = in->in_class;
       while (ic) {
         if (ic == idc) {
-          tvhtrace("idnode", "  add node %s", idnode_uuid_as_str(in));
-          idnode_set_add(is, in, NULL);
+          tvhtrace(LS_IDNODE, "  add node %s", idnode_uuid_as_str(in, ubuf));
+          idnode_set_add(is, in, NULL, NULL);
           break;
         }
         ic = ic->ic_super;
@@ -661,12 +679,12 @@ idnode_find_all ( const idclass_t *idc, const idnodes_rb_t *domain )
 
 static int
 idnode_cmp_title
-  ( const void *a, const void *b )
+  ( const void *a, const void *b, void *lang )
 {
   idnode_t      *ina  = *(idnode_t**)a;
   idnode_t      *inb  = *(idnode_t**)b;
-  const char *sa = idnode_get_title(ina);
-  const char *sb = idnode_get_title(inb);
+  const char *sa = idnode_get_title(ina, (const char *)lang);
+  const char *sb = idnode_get_title(inb, (const char *)lang);
   return strcmp(sa ?: "", sb ?: "");
 }
 
@@ -685,8 +703,8 @@ idnode_cmp_sort
   /* Get display string */
   if (p->islist || (p->list && !(p->opts & PO_SORTKEY))) {
     int r;
-    char *stra = idnode_get_display(ina, p);
-    char *strb = idnode_get_display(inb, p);
+    char *stra = idnode_get_display(ina, p, sort->lang);
+    char *strb = idnode_get_display(inb, p, sort->lang);
     if (sort->dir == IS_ASC)
       r = strcmp(stra ?: "", strb ?: "");
     else
@@ -745,6 +763,17 @@ idnode_cmp_sort
           return safecmp(s64b, s64a);
       }
       break;
+    case PT_S64_ATOMIC:
+      {
+        int64_t s64a = 0, s64b = 0;
+        idnode_get_s64_atomic(ina, sort->key, &s64a);
+        idnode_get_s64_atomic(inb, sort->key, &s64b);
+        if (sort->dir == IS_ASC)
+          return safecmp(s64a, s64b);
+        else
+          return safecmp(s64b, s64a);
+      }
+      break;
     case PT_DBL:
       {
         double dbla = 0, dblb = 0;
@@ -789,8 +818,8 @@ idnode_filter_init
         if (p->type == PT_U32 || p->type == PT_S64 ||
             p->type == PT_TIME) {
           int64_t v = f->u.n.n;
-          if (p->intsplit != f->u.n.intsplit) {
-            v = (v / MIN(1, f->u.n.intsplit)) * p->intsplit;
+          if (INTEXTRA_IS_SPLIT(p->intextra) && p->intextra != f->u.n.intsplit) {
+            v = (v / MIN(1, f->u.n.intsplit)) * p->intextra;
             f->u.n.n = v;
           }
         }
@@ -802,7 +831,7 @@ idnode_filter_init
 
 int
 idnode_filter
-  ( idnode_t *in, idnode_filter_t *filter )
+  ( idnode_t *in, idnode_filter_t *filter, const char *lang )
 {
   idnode_filter_ele_t *f;
   
@@ -813,7 +842,7 @@ idnode_filter
       const char *str;
       char *strdisp;
       int r = 1;
-      str = strdisp = idnode_get_display(in, idnode_find_prop(in, f->key));
+      str = strdisp = idnode_get_display(in, idnode_find_prop(in, f->key), lang);
       if (!str)
         if (!(str = idnode_get_str(in, f->key)))
           return 1;
@@ -952,17 +981,25 @@ idnode_filter_clear
 }
 
 void
-idnode_set_add
-  ( idnode_set_t *is, idnode_t *in, idnode_filter_t *filt )
+idnode_set_alloc
+  ( idnode_set_t *is, size_t alloc )
 {
-  if (filt && idnode_filter(in, filt))
+  if (is->is_alloc < alloc) {
+    is->is_alloc = alloc;
+    is->is_array = realloc(is->is_array, alloc * sizeof(idnode_t*));
+  }
+}
+
+void
+idnode_set_add
+  ( idnode_set_t *is, idnode_t *in, idnode_filter_t *filt, const char *lang )
+{
+  if (filt && idnode_filter(in, filt, lang))
     return;
   
   /* Allocate more space */
-  if (is->is_alloc == is->is_count) {
-    is->is_alloc = MAX(100, is->is_alloc * 2);
-    is->is_array = realloc(is->is_array, is->is_alloc * sizeof(idnode_t*));
-  }
+  if (is->is_alloc == is->is_count)
+    idnode_set_alloc(is, MAX(100, is->is_alloc * 2));
   if (is->is_sorted) {
     size_t i;
     idnode_t **a = is->is_array;
@@ -1007,8 +1044,9 @@ idnode_set_remove
 {
   ssize_t i = idnode_set_find_index(is, in);
   if (i >= 0) {
-    memmove(&is->is_array[i], &is->is_array[i+1],
-            (is->is_count - i - 1) * sizeof(idnode_t *));
+    if (is->is_count > 1)
+      memmove(&is->is_array[i], &is->is_array[i+1],
+              (is->is_count - i - 1) * sizeof(idnode_t *));
     is->is_count--;
     return 1;
   }
@@ -1024,9 +1062,9 @@ idnode_set_sort
 
 void
 idnode_set_sort_by_title
-  ( idnode_set_t *is )
+  ( idnode_set_t *is, const char *lang )
 {
-  qsort(is->is_array, is->is_count, sizeof(idnode_t*), idnode_cmp_title);
+  tvh_qsort_r(is->is_array, is->is_count, sizeof(idnode_t*), idnode_cmp_title, (void*)lang);
 }
 
 htsmsg_t *
@@ -1034,12 +1072,21 @@ idnode_set_as_htsmsg
   ( idnode_set_t *is )
 {
   htsmsg_t *l = htsmsg_create_list();
+  char ubuf[UUID_HEX_SIZE];
   int i;
   for (i = 0; i < is->is_count; i++)
-    htsmsg_add_str(l, NULL, idnode_uuid_as_str(is->is_array[i]));
+    htsmsg_add_str(l, NULL, idnode_uuid_as_str(is->is_array[i], ubuf));
   return l;
 }
 
+void
+idnode_set_clear ( idnode_set_t *is )
+{
+  free(is->is_array);
+  is->is_array = NULL;
+  is->is_count = is->is_alloc = 0;
+}
+
 void
 idnode_set_free ( idnode_set_t *is )
 {
@@ -1063,26 +1110,85 @@ idnode_class_write_values
 }
 
 static void
-idnode_savefn ( idnode_t *self )
+idnode_changedfn ( idnode_t *self )
 {
   const idclass_t *idc = self->in_class;
   while (idc) {
-    if (idc->ic_save) {
-      idc->ic_save(self);
+    if (idc->ic_changed) {
+      idc->ic_changed(self);
       break;
     }
     idc = idc->ic_super;
   }
 }
 
+static htsmsg_t *
+idnode_savefn ( idnode_t *self, char *filename, size_t fsize )
+{
+  const idclass_t *idc = self->in_class;
+  while (idc) {
+    if (idc->ic_save)
+      return idc->ic_save(self, filename, fsize);
+    idc = idc->ic_super;
+  }
+  return NULL;
+}
+
+static void
+idnode_save_trigger_thread_cb( void *aux )
+{
+  tvh_cond_signal(&save_cond, 0);
+}
+
+static void
+idnode_save_queue ( idnode_t *self )
+{
+  idnode_save_t *ise;
+
+  if (self->in_save)
+    return;
+  ise = malloc(sizeof(*ise));
+  ise->ise_node = self;
+  ise->ise_reqtime = mclk();
+  if (TAILQ_EMPTY(&idnodes_save) && atomic_get(&save_running))
+    mtimer_arm_rel(&save_timer, idnode_save_trigger_thread_cb, NULL, IDNODE_SAVE_DELAY);
+  TAILQ_INSERT_TAIL(&idnodes_save, ise, ise_link);
+  self->in_save = ise;
+}
+
+void
+idnode_save_check ( idnode_t *self, int weak )
+{
+  char filename[PATH_MAX];
+  htsmsg_t *m;
+
+  if (self->in_save == NULL || self->in_save == SAVEPTR_OUTOFSERVICE)
+    return;
+
+  TAILQ_REMOVE(&idnodes_save, self->in_save, ise_link);
+  free(self->in_save);
+  self->in_save = SAVEPTR_OUTOFSERVICE;
+
+  if (weak)
+    return;
+
+  m = idnode_savefn(self, filename, sizeof(filename));
+  if (m) {
+    hts_settings_save(m, "%s", filename);
+    htsmsg_destroy(m);
+  }
+}
+
 int
 idnode_write0 ( idnode_t *self, htsmsg_t *c, int optmask, int dosave )
 {
   int save = 0;
   const idclass_t *idc = self->in_class;
   save = idnode_class_write_values(self, idc, c, optmask);
-  if (save && dosave)
-    idnode_savefn(self);
+  if ((idc->ic_flags & IDCLASS_ALWAYS_SAVE) != 0 || (save && dosave)) {
+    idnode_changedfn(self);
+    idnode_save_queue(self);
+  }
   if (dosave)
     idnode_notify_changed(self);
   // Note: always output event if "dosave", reason is that UI updates on
@@ -1097,7 +1203,8 @@ void
 idnode_changed( idnode_t *self )
 {
   idnode_notify_changed(self);
-  idnode_savefn(self);
+  idnode_changedfn(self);
+  idnode_save_queue(self);
 }
 
 /* **************************************************************************
@@ -1105,11 +1212,12 @@ idnode_changed( idnode_t *self )
  * *************************************************************************/
 
 void
-idnode_read0 ( idnode_t *self, htsmsg_t *c, htsmsg_t *list, int optmask )
+idnode_read0 ( idnode_t *self, htsmsg_t *c, htsmsg_t *list,
+               int optmask, const char *lang )
 {
   const idclass_t *idc = self->in_class;
   for (; idc; idc = idc->ic_super)
-    prop_read_values(self, idc->ic_properties, c, list, optmask);
+    prop_read_values(self, idc->ic_properties, c, list, optmask, lang);
 }
 
 /**
@@ -1117,40 +1225,43 @@ idnode_read0 ( idnode_t *self, htsmsg_t *c, htsmsg_t *list, int optmask )
  */
 static void
 add_params
-  (struct idnode *self, const idclass_t *ic, htsmsg_t *p, htsmsg_t *list, int optmask)
+  (struct idnode *self, const idclass_t *ic, htsmsg_t *p, htsmsg_t *list,
+   int optmask, const char *lang)
 {
   /* Parent first */
   if(ic->ic_super != NULL)
-    add_params(self, ic->ic_super, p, list, optmask);
+    add_params(self, ic->ic_super, p, list, optmask, lang);
 
   /* Seperator (if not empty) */
 #if 0
   if(TAILQ_FIRST(&p->hm_fields) != NULL) {
     htsmsg_t *m = htsmsg_create_map();
-    htsmsg_add_str(m, "caption",  ic->ic_caption ?: ic->ic_class);
+    htsmsg_add_str(m, "caption",  tvh_gettext_lang(lang, ic->ic_caption) ?: ic->ic_class);
     htsmsg_add_str(m, "type",     "separator");
     htsmsg_add_msg(p, NULL, m);
   }
 #endif
 
   /* Properties */
-  prop_serialize(self, ic->ic_properties, p, list, optmask);
+  prop_serialize(self, ic->ic_properties, p, list, optmask, lang);
 }
 
 static htsmsg_t *
-idnode_params (const idclass_t *idc, idnode_t *self, htsmsg_t *list, int optmask)
+idnode_params
+  (const idclass_t *idc, idnode_t *self, htsmsg_t *list,
+   int optmask, const char *lang)
 {
   htsmsg_t *p  = htsmsg_create_list();
-  add_params(self, idc, p, list, optmask);
+  add_params(self, idc, p, list, optmask, lang);
   return p;
 }
 
-static const char *
-idclass_get_caption (const idclass_t *idc )
+const char *
+idclass_get_caption (const idclass_t *idc, const char *lang)
 {
   while (idc) {
     if (idc->ic_caption)
-      return idc->ic_caption;
+      return tvh_gettext_lang(lang, idc->ic_caption);
     idc = idc->ic_super;
   }
   return NULL;
@@ -1189,8 +1300,19 @@ idclass_get_order (const idclass_t *idc)
   return NULL;
 }
 
+const char **
+idclass_get_doc (const idclass_t *idc)
+{
+  while (idc) {
+    if (idc->ic_doc)
+      return idc->ic_doc;
+    idc = idc->ic_super;
+  }
+  return NULL;
+}
+
 static htsmsg_t *
-idclass_get_property_groups (const idclass_t *idc)
+idclass_get_property_groups (const idclass_t *idc, const char *lang)
 {
   const property_group_t *g;
   htsmsg_t *e, *m;
@@ -1202,7 +1324,7 @@ idclass_get_property_groups (const idclass_t *idc)
       for (g = idc->ic_groups; g->number && g->name; g++) {
         e = htsmsg_create_map();
         htsmsg_add_u32(e, "number", g->number);
-        htsmsg_add_str(e, "name",   g->name);
+        htsmsg_add_str(e, "name",   tvh_gettext_lang(lang, g->name));
         if (g->parent)
           htsmsg_add_u32(e, "parent", g->parent);
         if (g->column)
@@ -1220,39 +1342,51 @@ idclass_get_property_groups (const idclass_t *idc)
   return NULL;
 }
 
-void
-idclass_register(const idclass_t *idc)
+static idnodes_rb_t *
+idclass_find_domain(const idclass_t *idc)
 {
-  while (idc) {
-    SKEL_ALLOC(idclasses_skel);
-    idclasses_skel->idc = idc;
-    if (RB_INSERT_SORTED(&idclasses, idclasses_skel, link, ic_cmp))
-      break;
-    RB_INIT(&idclasses_skel->nodes); /* not used, but for sure */
-    SKEL_USED(idclasses_skel);
-    tvhtrace("idnode", "register class %s", idc->ic_class);
-    idc = idc->ic_super;
-  }
-}
-
-static void
-idclass_root_register(idnode_t *in)
-{
-  const idclass_t *idc = in->in_class;
   idclass_link_t *r;
   idc = idnode_root_class(idc);
   SKEL_ALLOC(idclasses_skel);
   idclasses_skel->idc = idc;
+  r = RB_FIND(&idrootclasses, idclasses_skel, link, ic_cmp);
+  if (r)
+    return &r->nodes;
+  return NULL;
+}
+
+static void
+idclass_root_register(const idclass_t *idc)
+{
+  idclass_link_t *r;
+  SKEL_ALLOC(idclasses_skel);
+  idclasses_skel->idc = idc;
   r = RB_INSERT_SORTED(&idrootclasses, idclasses_skel, link, ic_cmp);
-  if (r) {
-    in->in_domain = &r->nodes;
-    return;
-  }
+  if (r) return;
   RB_INIT(&idclasses_skel->nodes);
-  r = idclasses_skel;
   SKEL_USED(idclasses_skel);
-  tvhtrace("idnode", "register root class %s", idc->ic_class);
-  in->in_domain = &r->nodes;
+  tvhtrace(LS_IDNODE, "register root class %s", idc->ic_class);
+}
+
+void
+idclass_register(const idclass_t *idc)
+{
+  const idclass_t *prev = NULL;
+  while (idc) {
+    SKEL_ALLOC(idclasses_skel);
+    idclasses_skel->idc = idc;
+    if (RB_INSERT_SORTED(&idclasses, idclasses_skel, link, ic_cmp)) {
+      prev = NULL;
+      break;
+    }
+    RB_INIT(&idclasses_skel->nodes); /* not used, but for sure */
+    SKEL_USED(idclasses_skel);
+    tvhtrace(LS_IDNODE, "register class %s", idc->ic_class);
+    prev = idc;
+    idc = idc->ic_super;
+  }
+  if (prev)
+    idclass_root_register(prev);
 }
 
 const idclass_t *
@@ -1262,22 +1396,69 @@ idclass_find ( const char *class )
   idclass_t idc;
   skel.idc = &idc;
   idc.ic_class = class;
-  tvhtrace("idnode", "find class %s", class);
+  tvhtrace(LS_IDNODE, "find class %s", class);
   t = RB_FIND(&idclasses, &skel, link, ic_cmp);
   return t ? t->idc : NULL;
 }
 
+idclass_t const **
+idclass_find_all(void)
+{
+  idclass_link_t *l;
+  idclass_t const **ret;
+  int i = 0, count = 0;
+  RB_FOREACH(l, &idclasses, link)
+    count++;
+  if (count == 0)
+    return NULL;
+  ret = calloc(count + 1, sizeof(idclass_t *));
+  RB_FOREACH(l, &idclasses, link)
+    ret[i++] = l->idc;
+  ret[i] = NULL;
+  return ret;
+}
+
+idclass_t const **
+idclass_find_children(const char *clazz)
+{
+  const idclass_t *ic = idclass_find(clazz), *root;
+  idclass_link_t *l;
+  idclass_t const **ret;
+  int i, count;
+
+  if (ic == NULL)
+    return NULL;
+  root = idnode_root_class(ic);
+  if (root == NULL)
+    return NULL;
+  ret = NULL;
+  count = i = 0;
+  RB_FOREACH(l, &idclasses, link) {
+    if (root == idnode_root_class(l->idc)) {
+      if (i <= count) {
+        count += 50;
+        ret = realloc(ret, count * sizeof(const idclass_t *));
+      }
+      ret[i++] = ic;
+    }
+  }
+  if (i <= count)
+    ret = realloc(ret, (count + 1) * sizeof(const idclass_t *));
+  ret[i] = NULL;
+  return ret;
+}
+
 /*
  * Just get the class definition
  */
 htsmsg_t *
-idclass_serialize0(const idclass_t *idc, htsmsg_t *list, int optmask)
+idclass_serialize0(const idclass_t *idc, htsmsg_t *list, int optmask, const char *lang)
 {
   const char *s;
   htsmsg_t *p, *m = htsmsg_create_map();
 
   /* Caption and name */
-  if ((s = idclass_get_caption(idc)))
+  if ((s = idclass_get_caption(idc, lang)))
     htsmsg_add_str(m, "caption", s);
   if ((s = idclass_get_class(idc)))
     htsmsg_add_str(m, "class", s);
@@ -1285,11 +1466,11 @@ idclass_serialize0(const idclass_t *idc, htsmsg_t *list, int optmask)
     htsmsg_add_str(m, "event", s);
   if ((s = idclass_get_order(idc)))
     htsmsg_add_str(m, "order", s);
-  if ((p = idclass_get_property_groups(idc)))
+  if ((p = idclass_get_property_groups(idc, lang)))
     htsmsg_add_msg(m, "groups", p);
 
   /* Props */
-  if ((p = idnode_params(idc, NULL, list, optmask)))
+  if ((p = idnode_params(idc, NULL, list, optmask, lang)))
     htsmsg_add_msg(m, "props", p);
   
   return m;
@@ -1299,28 +1480,357 @@ idclass_serialize0(const idclass_t *idc, htsmsg_t *list, int optmask)
  *
  */
 htsmsg_t *
-idnode_serialize0(idnode_t *self, htsmsg_t *list, int optmask)
+idnode_serialize0(idnode_t *self, htsmsg_t *list, int optmask, const char *lang)
 {
   const idclass_t *idc = self->in_class;
   const char *uuid, *s;
+  char ubuf[UUID_HEX_SIZE];
 
   htsmsg_t *m = htsmsg_create_map();
-  uuid = idnode_uuid_as_str(self);
-  htsmsg_add_str(m, "uuid", uuid);
-  htsmsg_add_str(m, "id",   uuid);
-  htsmsg_add_str(m, "text", idnode_get_title(self) ?: "");
-  if ((s = idclass_get_caption(idc)))
+  if (!idc->ic_snode) {
+    uuid = idnode_uuid_as_str(self, ubuf);
+    htsmsg_add_str(m, "uuid", uuid);
+    htsmsg_add_str(m, "id",   uuid);
+  }
+  htsmsg_add_str(m, "text", idnode_get_title(self, lang) ?: "");
+  if ((s = idclass_get_caption(idc, lang)))
     htsmsg_add_str(m, "caption", s);
   if ((s = idclass_get_class(idc)))
     htsmsg_add_str(m, "class", s);
   if ((s = idclass_get_event(idc)))
     htsmsg_add_str(m, "event", s);
 
-  htsmsg_add_msg(m, "params", idnode_params(idc, self, list, optmask));
+  htsmsg_add_msg(m, "params", idnode_params(idc, self, list, optmask, lang));
 
   return m;
 }
 
+/* **************************************************************************
+ * Simple list helpers
+ * *************************************************************************/
+
+htsmsg_t *
+idnode_slist_enum ( idnode_t *in, idnode_slist_t *options, const char *lang )
+{
+  htsmsg_t *l = htsmsg_create_list(), *m;
+
+  for (; options->id; options++) {
+    m = htsmsg_create_key_val(options->id, tvh_gettext_lang(lang, options->name));
+    htsmsg_add_msg(l, NULL, m);
+  }
+  return l;
+}
+
+htsmsg_t *
+idnode_slist_get ( idnode_t *in, idnode_slist_t *options )
+{
+  htsmsg_t *l = htsmsg_create_list();
+  int *ip;
+
+  for (; options->id; options++) {
+    ip = (void *)in + options->off;
+    if (*ip)
+      htsmsg_add_str(l, NULL, options->id);
+  }
+  return l;
+}
+
+int
+idnode_slist_set ( idnode_t *in, idnode_slist_t *options, const htsmsg_t *vals )
+{
+  idnode_slist_t *o;
+  htsmsg_field_t *f;
+  int *ip, changed = 0;
+  const char *s;
+
+  for (o = options; o->id; o++) {
+    ip = (void *)in + o->off;
+    if (!changed) {
+      HTSMSG_FOREACH(f, vals) {
+        if ((s = htsmsg_field_get_str(f)) == NULL)
+          continue;
+        if (strcmp(s, o->id))
+          continue;
+        if (*ip == 0) changed = 1;
+        break;
+      }
+      if (f == NULL && *ip) changed = 1;
+    }
+    *ip = 0;
+  }
+  HTSMSG_FOREACH(f, vals) {
+    if ((s = htsmsg_field_get_str(f)) == NULL)
+      continue;
+    for (o = options; o->id; o++) {
+      if (strcmp(o->id, s)) continue;
+      ip = (void *)in + o->off;
+      *ip = 1;
+      break;
+    }
+  }
+  return changed;
+}
+
+char *
+idnode_slist_rend ( idnode_t *in, idnode_slist_t *options, const char *lang )
+{
+  int *ip;
+  size_t l = 0;
+
+  prop_sbuf[0] = '\0';
+  for (; options->id; options++) {
+    ip = (void *)in + options->off;
+    if (*ip)
+     tvh_strlcatf(prop_sbuf, PROP_SBUF_LEN, l, "%s%s", prop_sbuf[0] ? "," : "",
+                   tvh_gettext_lang(lang, options->name));
+  }
+  return strdup(prop_sbuf);
+}
+
+/* **************************************************************************
+ * List helpers
+ * *************************************************************************/
+
+static void
+idnode_list_notify ( idnode_list_mapping_t *ilm, void *origin )
+{
+  if (origin == NULL)
+    return;
+  if (origin == ilm->ilm_in1) {
+    idnode_notify_changed(ilm->ilm_in2);
+    if (ilm->ilm_in2_save)
+      idnode_save_queue(ilm->ilm_in2);
+  }
+  if (origin == ilm->ilm_in2) {
+    idnode_notify_changed(ilm->ilm_in1);
+    if (ilm->ilm_in1_save)
+      idnode_save_queue(ilm->ilm_in1);
+  }
+}
+
+/*
+ * Link class1 and class2
+ */
+idnode_list_mapping_t *
+idnode_list_link ( idnode_t *in1, idnode_list_head_t *in1_list,
+                   idnode_t *in2, idnode_list_head_t *in2_list,
+                   void *origin, uint32_t savemask )
+{
+  idnode_list_mapping_t *ilm;
+
+  /* Already linked */
+  LIST_FOREACH(ilm, in1_list, ilm_in1_link)
+    if (ilm->ilm_in2 == in2) {
+      ilm->ilm_mark = 0;
+      return NULL;
+    }
+  LIST_FOREACH(ilm, in2_list, ilm_in2_link)
+    if (ilm->ilm_in1 == in1) {
+      ilm->ilm_mark = 0;
+      return NULL;
+    }
+
+  /* Link */
+  ilm = calloc(1, sizeof(idnode_list_mapping_t));
+  ilm->ilm_in1 = in1;
+  ilm->ilm_in2 = in2;
+  LIST_INSERT_HEAD(in1_list, ilm, ilm_in1_link);
+  LIST_INSERT_HEAD(in2_list, ilm, ilm_in2_link);
+  ilm->ilm_in1_save = savemask & 1;
+  ilm->ilm_in2_save = (savemask >> 1) & 1;
+  idnode_list_notify(ilm, origin);
+  return ilm;
+}
+
+void
+idnode_list_unlink ( idnode_list_mapping_t *ilm, void *origin )
+{
+  LIST_REMOVE(ilm, ilm_in1_link);
+  LIST_REMOVE(ilm, ilm_in2_link);
+  idnode_list_notify(ilm, origin);
+  free(ilm);
+}
+
+void
+idnode_list_destroy(idnode_list_head_t *ilh, void *origin)
+{
+  idnode_list_mapping_t *ilm;
+
+  while ((ilm = LIST_FIRST(ilh)) != NULL)
+    idnode_list_unlink(ilm, origin);
+}
+
+static int
+idnode_list_clean1
+  ( idnode_t *in1, idnode_list_head_t *in1_list )
+{
+  int save = 0;
+  idnode_list_mapping_t *ilm, *n;
+
+  for (ilm = LIST_FIRST(in1_list); ilm != NULL; ilm = n) {
+    n = LIST_NEXT(ilm, ilm_in1_link);
+    if (ilm->ilm_mark) {
+      idnode_list_unlink(ilm, in1);
+      save = 1;
+    }
+  }
+  return save;
+}
+
+static int
+idnode_list_clean2
+  ( idnode_t *in2, idnode_list_head_t *in2_list )
+{
+  int save = 0;
+  idnode_list_mapping_t *ilm, *n;
+
+  for (ilm = LIST_FIRST(in2_list); ilm != NULL; ilm = n) {
+    n = LIST_NEXT(ilm, ilm_in2_link);
+    if (ilm->ilm_mark) {
+      idnode_list_unlink(ilm, in2);
+      save = 1;
+    }
+  }
+  return save;
+}
+
+htsmsg_t *
+idnode_list_get1
+  ( idnode_list_head_t *in1_list )
+{
+  idnode_list_mapping_t *ilm;
+  htsmsg_t *l = htsmsg_create_list();
+  char ubuf[UUID_HEX_SIZE];
+
+  LIST_FOREACH(ilm, in1_list, ilm_in1_link)
+    htsmsg_add_str(l, NULL, idnode_uuid_as_str(ilm->ilm_in2, ubuf));
+  return l;
+}
+
+htsmsg_t *
+idnode_list_get2
+  ( idnode_list_head_t *in2_list )
+{
+  idnode_list_mapping_t *ilm;
+  htsmsg_t *l = htsmsg_create_list();
+  char ubuf[UUID_HEX_SIZE];
+
+  LIST_FOREACH(ilm, in2_list, ilm_in2_link)
+    htsmsg_add_str(l, NULL, idnode_uuid_as_str(ilm->ilm_in1, ubuf));
+  return l;
+}
+
+char *
+idnode_list_get_csv1
+  ( idnode_list_head_t *in1_list, const char *lang )
+{
+  char *str;
+  idnode_list_mapping_t *ilm;
+  htsmsg_t *l = htsmsg_create_list();
+
+  LIST_FOREACH(ilm, in1_list, ilm_in1_link)
+    htsmsg_add_str(l, NULL, idnode_get_title(ilm->ilm_in2, lang));
+
+  str = htsmsg_list_2_csv(l, ',', 1);
+  htsmsg_destroy(l);
+  return str;
+}
+
+char *
+idnode_list_get_csv2
+  ( idnode_list_head_t *in2_list, const char *lang )
+{
+  char *str;
+  idnode_list_mapping_t *ilm;
+  htsmsg_t *l = htsmsg_create_list();
+
+  LIST_FOREACH(ilm, in2_list, ilm_in2_link)
+    htsmsg_add_str(l, NULL, idnode_get_title(ilm->ilm_in1, lang));
+
+  str = htsmsg_list_2_csv(l, ',', 1);
+  htsmsg_destroy(l);
+  return str;
+}
+
+int
+idnode_list_set1
+  ( idnode_t *in1, idnode_list_head_t *in1_list,
+    const idclass_t *in2_class, htsmsg_t *in2_list,
+    int (*in2_create)(idnode_t *in1, idnode_t *in2, void *origin) )
+{
+  const char *str;
+  htsmsg_field_t *f;
+  idnode_t *in2;
+  idnode_list_mapping_t *ilm;
+  int save = 0;
+
+  /* Mark all for deletion */
+  LIST_FOREACH(ilm, in1_list, ilm_in1_link)
+    ilm->ilm_mark = 1;
+
+  /* Make new links */
+  HTSMSG_FOREACH(f, in2_list)
+    if ((str = htsmsg_field_get_str(f)))
+      if ((in2 = idnode_find(str, in2_class, NULL)) != NULL)
+        if (in2_create(in1, in2, in1))
+          save = 1;
+
+  /* Delete unlinked */
+  if (idnode_list_clean1(in1, in1_list))
+    save = 1;
+
+  /* Change notification */
+  if (save)
+    idnode_notify_changed(in1);
+
+  /* Save only on demand */
+  ilm = LIST_FIRST(in1_list);
+  if (ilm && !ilm->ilm_in1_save)
+    save = 0;
+
+  return save;
+}
+
+int
+idnode_list_set2
+  ( idnode_t *in2, idnode_list_head_t *in2_list,
+    const idclass_t *in1_class, htsmsg_t *in1_list,
+    int (*in1_create)(idnode_t *in1, idnode_t *in2, void *origin) )
+{
+  const char *str;
+  htsmsg_field_t *f;
+  idnode_t *in1;
+  idnode_list_mapping_t *ilm;
+  int save = 0;
+
+  /* Mark all for deletion */
+  LIST_FOREACH(ilm, in2_list, ilm_in2_link)
+    ilm->ilm_mark = 1;
+
+  /* Make new links */
+  HTSMSG_FOREACH(f, in1_list)
+    if ((str = htsmsg_field_get_str(f)))
+      if ((in1 = idnode_find(str, in1_class, NULL)) != NULL)
+        if (in1_create(in1, in2, in2))
+          save = 1;
+
+  /* Delete unlinked */
+  if (idnode_list_clean2(in2, in2_list))
+    save = 1;
+
+  /* Change notification */
+  if (save)
+    idnode_notify_changed(in2);
+
+  /* Save only on demand */
+  ilm = LIST_FIRST(in2_list);
+  if (ilm && !ilm->ilm_in2_save)
+    save = 0;
+
+  return save;
+}
+
+
 /* **************************************************************************
  * Notification
  * *************************************************************************/
@@ -1332,14 +1842,19 @@ void
 idnode_notify ( idnode_t *in, const char *action )
 {
   const idclass_t *ic = in->in_class;
-  const char *uuid = idnode_uuid_as_str(in);
+  char ubuf[UUID_HEX_SIZE];
+  const char *uuid = idnode_uuid_as_str(in, ubuf);
 
-  if (!tvheadend_running)
+  if (!tvheadend_is_running())
     return;
 
   while (ic) {
-    if (ic->ic_event)
-      notify_delayed(uuid, ic->ic_event, action);
+    if (ic->ic_event) {
+      if (!ic->ic_snode)
+        notify_delayed(uuid, ic->ic_event, action);
+      else
+        notify_reload(ic->ic_event);
+    }
     ic = ic->ic_super;
   }
 }
@@ -1351,15 +1866,113 @@ idnode_notify_changed (void *in)
 }
 
 void
-idnode_notify_title_changed (void *in)
+idnode_notify_title_changed (void *in, const char *lang)
 {
+  char ubuf[UUID_HEX_SIZE];
   htsmsg_t *m = htsmsg_create_map();
-  htsmsg_add_str(m, "uuid", idnode_uuid_as_str(in));
-  htsmsg_add_str(m, "text", idnode_get_title(in));
-  notify_by_msg("title", m);
+  htsmsg_add_str(m, "uuid", idnode_uuid_as_str(in, ubuf));
+  htsmsg_add_str(m, "text", idnode_get_title(in, lang));
+  notify_by_msg("title", m, 0);
   idnode_notify_changed(in);
 }
 
+/* **************************************************************************
+ * Save thread
+ * *************************************************************************/
+
+static void *
+save_thread ( void *aux )
+{
+  idnode_save_t *ise;
+  htsmsg_t *m;
+  char filename[PATH_MAX];
+
+  tvhthread_renice(15);
+
+  pthread_mutex_lock(&global_lock);
+
+  while (atomic_get(&save_running)) {
+    if ((ise = TAILQ_FIRST(&idnodes_save)) == NULL ||
+        (ise->ise_reqtime + IDNODE_SAVE_DELAY > mclk())) {
+      if (ise)
+        mtimer_arm_abs(&save_timer, idnode_save_trigger_thread_cb, NULL,
+                       ise->ise_reqtime + IDNODE_SAVE_DELAY);
+      tvh_cond_wait(&save_cond, &global_lock);
+      continue;
+    }
+    m = idnode_savefn(ise->ise_node, filename, sizeof(filename));
+    ise->ise_node->in_save = NULL;
+    TAILQ_REMOVE(&idnodes_save, ise, ise_link);
+    pthread_mutex_unlock(&global_lock);
+    free(ise);
+    if (m) {
+      hts_settings_save(m, "%s", filename);
+      htsmsg_destroy(m);
+    }
+    pthread_mutex_lock(&global_lock);
+  }
+
+  mtimer_disarm(&save_timer);
+
+  while ((ise = TAILQ_FIRST(&idnodes_save)) != NULL) {
+    m = idnode_savefn(ise->ise_node, filename, sizeof(filename));
+    ise->ise_node->in_save = NULL;
+    TAILQ_REMOVE(&idnodes_save, ise, ise_link);
+    pthread_mutex_unlock(&global_lock);
+    free(ise);
+    if (m) {
+      hts_settings_save(m, "%s", filename);
+      htsmsg_destroy(m);
+    }
+    pthread_mutex_lock(&global_lock);
+  }
+
+  pthread_mutex_unlock(&global_lock);
+  return NULL;
+}
+
+/* **************************************************************************
+ * Initialization
+ * *************************************************************************/
+
+void
+idnode_boot(void)
+{
+  RB_INIT(&idnodes);
+  RB_INIT(&idclasses);
+  RB_INIT(&idrootclasses);
+  TAILQ_INIT(&idnodes_save);
+  tvh_cond_init(&save_cond);
+}
+
+void
+idnode_init(void)
+{
+  atomic_set(&save_running, 1);
+  tvhthread_create(&save_tid, NULL, save_thread, NULL, "save");
+}
+
+void
+idnode_done(void)
+{
+  idclass_link_t *il;
+
+  atomic_set(&save_running, 0);
+  tvh_cond_signal(&save_cond, 0);
+  pthread_join(save_tid, NULL);
+  mtimer_disarm(&save_timer);
+
+  while ((il = RB_FIRST(&idclasses)) != NULL) {
+    RB_REMOVE(&idclasses, il, link);
+    free(il);
+  }
+  while ((il = RB_FIRST(&idrootclasses)) != NULL) {
+    RB_REMOVE(&idrootclasses, il, link);
+    free(il);
+  }
+  SKEL_FREE(idclasses_skel);
+}
+
 /******************************************************************************
  * Editor Configuration
  *
