@@ -28,7 +28,9 @@
 
 struct mpegts_table;
 struct mpegts_table_state;
+struct mpegts_network;
 struct mpegts_mux;
+struct lang_str;
 
 /* PIDs */
 
@@ -48,6 +50,8 @@ struct mpegts_mux;
 #define DVB_DIT_PID                   0x1E
 #define DVB_SIT_PID                   0x1F
 #define DVB_VCT_PID                   0x1FFB
+#define DVB_ATSC_STT_PID              0x1FFB
+#define DVB_ATSC_MGT_PID              0x1FFB
 
 /* Tables */
 
@@ -75,6 +79,9 @@ struct mpegts_mux;
 #define DVB_TOT_BASE                  0x73
 #define DVB_TOT_MASK                  0xFF
 
+#define DVB_HBBTV_BASE                0x74
+#define DVB_HBBTV_MASK                0xFF
+
 #define DVB_FASTSCAN_NIT_BASE         0xBC
 #define DVB_FASTSCAN_SDT_BASE         0xBD
 #define DVB_FASTSCAN_MASK             0xFF
@@ -83,6 +90,18 @@ struct mpegts_mux;
 #define DVB_VCT_C_BASE                0xC9
 #define DVB_VCT_MASK                  0xFF
 
+#define DVB_ATSC_MGT_BASE             0xC7
+#define DVB_ATSC_MGT_MASK             0xFF
+
+#define DVB_ATSC_EIT_BASE             0xCB
+#define DVB_ATSC_EIT_MASK             0xFF
+
+#define DVB_ATSC_ETT_BASE             0xCC
+#define DVB_ATSC_ETT_MASK             0xFF
+
+#define DVB_ATSC_STT_BASE             0xCD
+#define DVB_ATSC_STT_MASK             0xFF
+
 #define DVB_TELETEXT_BASE             0x2000
 
 /* Descriptors */
@@ -153,6 +172,22 @@ struct mpegts_mux;
 #define DVB_DESC_FREESAT_LCN          0xD3
 #define DVB_DESC_FREESAT_REGIONS      0xD4
 
+/* Descriptors defined in A/65:2009 */
+
+#define ATSC_DESC_STUFFING            0x80
+#define ATSC_DESC_AC3                 0x81
+#define ATSC_DESC_CAPTION             0x86
+#define ATSC_DESC_CONTENT_ADVISORY    0x87
+#define ATSC_DESC_EXT_CHANNEL_NAME    0xA0
+#define ATSC_DESC_SERVICE_LOCATION    0xA1
+#define ATSC_DESC_TIMESHIFTED_SVC     0xA2
+#define ATSC_DESC_COMPONENT_NAME      0xA3
+#define ATSC_DESC_DCC_DEPARTING       0xA8
+#define ATSC_DESC_DCC_ARRIVING        0xA9
+#define ATSC_DESC_REDISTRIB_CTRL      0xAA
+#define ATSC_DESC_GENRE               0xAB
+#define ATSC_DESC_PRIVATE_INFO        0xAD
+
 /* Service type lookup */
 
 int dvb_servicetype_lookup ( int t );
@@ -174,55 +209,77 @@ int dvb_get_string_with_len
   (char *dst, size_t dstlen, const uint8_t *buf, size_t buflen,
    const char *dvb_charset, dvb_string_conv_t *conv);
 
+struct lang_str *atsc_get_string
+  (const uint8_t *src, size_t srclen);
+
 /* Conversion */
 
-#define bcdtoint(i) ((((i & 0xf0) >> 4) * 10) + (i & 0x0f))
+static inline uint32_t bcdtoint(const uint32_t i) {
+  return ((((i & 0xf0) >> 4) * 10) + (i & 0x0f));
+}
+static inline uint32_t bcdtoint4(const uint8_t *ptr) {
+  return (bcdtoint(ptr[0]) * 1000000) +
+         (bcdtoint(ptr[1]) * 10000) +
+         (bcdtoint(ptr[2]) * 100) +
+         bcdtoint(ptr[3]);
+}
+static inline uint32_t bcdtoint41(const uint8_t *ptr) {
+  return (bcdtoint(ptr[0]) * 100000) +
+         (bcdtoint(ptr[1]) * 1000) +
+         (bcdtoint(ptr[2]) * 10) +
+         ((ptr[3]) >> 4);
+}
 
-time_t dvb_convert_date(const uint8_t *dvb_buf, int local);
+htsmsg_t *dvb_timezone_enum(void *p, const char *lang);
 
+time_t dvb_convert_date(const uint8_t *dvb_buf, int tmzone);
+time_t atsc_convert_gpstime(uint32_t gpstime);
 void atsc_utf16_to_utf8(const uint8_t *src, int len, char *buf, int buflen);
 
 /*
  * PSI processing
  */
 
-#define DVB_LOOP_INIT(ptr, len, off, lptr, llen)\
+#define DVB_LOOP_INIT(mt, ptr, len, off, lptr, llen)\
 do {\
   llen = ((ptr[off] & 0xF) << 8) | ptr[off+1];\
   lptr = 2 + off + ptr;\
   ptr += 2 + off + llen;\
   len -= 2 + off + llen;\
-  if (len < 0) {tvhtrace("psi", "len < 0"); return -1; }\
+  if (len < 0) {tvhtrace(mt->mt_subsys, "%s: len < 0", mt->mt_name); return -1; }\
 } while(0)
 
 #define DVB_LOOP_EACH(ptr, len, min)\
   for ( ; len > min ; )\
 
-#define DVB_LOOP_FOREACH(ptr, len, off, lptr, llen, min)\
-  DVB_LOOP_INIT(ptr, len, off, lptr, llen);\
+#define DVB_LOOP_FOREACH(mt, ptr, len, off, lptr, llen, min)\
+  DVB_LOOP_INIT(mt, ptr, len, off, lptr, llen);\
   DVB_LOOP_EACH(lptr, llen, min)
 
-#define DVB_DESC_EACH(ptr, len, dtag, dlen, dptr)\
+#define DVB_DESC_EACH(mt, ptr, len, dtag, dlen, dptr)\
   DVB_LOOP_EACH(ptr, len, 2)\
-    if      (!(dtag  = ptr[0]))      {tvhtrace("psi", "1");return -1;}\
-    else if ((dlen  = ptr[1]) < 0)   {tvhtrace("psi", "2");return -1;}\
-    else if (!(dptr  = ptr+2))       {tvhtrace("psi", "3");return -1;}\
-    else if ( (len -= 2 + dlen) < 0) {tvhtrace("psi", "4");return -1;}\
-    else if (!(ptr += 2 + dlen))     {tvhtrace("psi", "5");return -1;}\
+    if      (!(dtag  = ptr[0]))      {tvhtrace(mt->mt_subsys, "%s: 1", mt->mt_name);return -1;}\
+    else if ((dlen  = ptr[1]) < 0)   {tvhtrace(mt->mt_subsys, "%s: 2", mt->mt_name);return -1;}\
+    else if (!(dptr  = ptr+2))       {tvhtrace(mt->mt_subsys, "%s: 3", mt->mt_name);return -1;}\
+    else if ( (len -= 2 + dlen) < 0) {tvhtrace(mt->mt_subsys, "%s: 4", mt->mt_name);return -1;}\
+    else if (!(ptr += 2 + dlen))     {tvhtrace(mt->mt_subsys, "%s: 5", mt->mt_name);return -1;}\
     else
 
-#define DVB_DESC_FOREACH(ptr, len, off, lptr, llen, dtag, dlen, dptr)\
-  DVB_LOOP_INIT(ptr, len, off, lptr, llen);\
-  DVB_DESC_EACH(lptr, llen, dtag, dlen, dptr)\
+#define DVB_DESC_FOREACH(mt, ptr, len, off, lptr, llen, dtag, dlen, dptr)\
+  DVB_LOOP_INIT(mt, ptr, len, off, lptr, llen);\
+  DVB_DESC_EACH(mt, lptr, llen, dtag, dlen, dptr)\
 
 /*
  * SI typedefs
  */
 
 #define MPEGTS_PSI_SECTION_SIZE 5000
+#define MPEGTS_PSI_VERSION_NONE 255
 
 typedef struct mpegts_psi_section
 {
+  uint8_t ps_table;  // SI table ID
+  uint8_t ps_mask;   // mask
   int8_t  ps_cc;
   int8_t  ps_cco;
   int     ps_offset;
@@ -238,6 +295,7 @@ typedef struct mpegts_psi_table_state
   int      tableid;
   uint64_t extraid;
   int      version;
+  int      last;
   int      complete;
   int      working;
   uint32_t sections[8];
@@ -249,6 +307,7 @@ typedef struct mpegts_psi_table
   LIST_ENTRY(mpegts_table) mt_link;
   RB_HEAD(,mpegts_psi_table_state) mt_state;
 
+  int     mt_subsys;
   char   *mt_name;
   void   *mt_opaque;
 
@@ -271,7 +330,7 @@ typedef struct mpegts_psi_table
  * Assemble SI section
  */
 void mpegts_psi_section_reassemble
- ( mpegts_psi_table_t *mt, const uint8_t *tsb, int crc,
+ ( mpegts_psi_table_t *mt, const char *logpref, const uint8_t *tsb, int crc,
    mpegts_psi_section_callback_t cb, void *opaque );
 
 /* PSI table parser helpers */
@@ -291,13 +350,15 @@ typedef void (*mpegts_psi_parse_callback_t)
   ( mpegts_psi_table_t *, const uint8_t *buf, int len );
 
 void dvb_table_parse_init
-  ( mpegts_psi_table_t *mt, const char *name, int pid, void *opaque );
+  ( mpegts_psi_table_t *mt, const char *name, int subsys, int pid,
+    uint8_t table, uint8_t mask, void *opaque );
 
 void dvb_table_parse_done ( mpegts_psi_table_t *mt);
 
 void dvb_table_parse
-  (mpegts_psi_table_t *mt, const uint8_t *tsb, int len,
-   int crc, int full, mpegts_psi_parse_callback_t cb);
+  (mpegts_psi_table_t *mt, const char *logprefix,
+   const uint8_t *tsb, int len, int crc, int full,
+   mpegts_psi_parse_callback_t cb);
 
 int dvb_table_append_crc32(uint8_t *dst, int off, int maxlen);
 
@@ -343,8 +404,13 @@ typedef enum dvb_fe_type {
   DVB_TYPE_T = 1,		/* terrestrial */
   DVB_TYPE_C,			/* cable */
   DVB_TYPE_S,			/* satellite */
-  DVB_TYPE_ATSC,		/* terrestrial - north america */
-  DVB_TYPE_LAST = DVB_TYPE_ATSC
+  DVB_TYPE_ATSC_T,		/* terrestrial - north america */
+  DVB_TYPE_ATSC_C,		/* cable - north america */
+  DVB_TYPE_ISDB_T,              /* terrestrial - japan, brazil */
+  DVB_TYPE_ISDB_C,              /* cable - japan, brazil */
+  DVB_TYPE_ISDB_S,              /* satellite - japan, brazil */
+  DVB_TYPE_DAB,                 /* digital radio (europe) */
+  DVB_TYPE_LAST = DVB_TYPE_DAB
 } dvb_fe_type_t;
 
 typedef enum dvb_fe_spectral_inversion {
@@ -522,6 +588,17 @@ typedef struct dvb_ofdm_config {
   dvb_fe_hierarchy_t      hierarchy_information;
 } dvb_ofdm_config_t;
 
+typedef struct dvb_isdbt_config {
+  dvb_fe_bandwidth_t      bandwidth;
+  dvb_fe_guard_interval_t guard_interval;
+  struct {
+    dvb_fe_code_rate_t    fec;
+    dvb_fe_modulation_t   modulation;
+    uint32_t              segment_count;
+    uint32_t              time_interleaving;
+  } layers[3];
+} dvb_isdbt_config_t;
+
 typedef struct dvb_mux_conf
 {
   dvb_fe_type_t               dmc_fe_type;
@@ -534,10 +611,12 @@ typedef struct dvb_mux_conf
   int32_t                     dmc_fe_stream_id;
   dvb_fe_pls_mode_t           dmc_fe_pls_mode;
   uint32_t                    dmc_fe_pls_code;
+  uint32_t                    dmc_fe_data_slice;
   union {
     dvb_qpsk_config_t         dmc_fe_qpsk;
     dvb_qam_config_t          dmc_fe_qam;
     dvb_ofdm_config_t         dmc_fe_ofdm;
+    dvb_isdbt_config_t        dmc_fe_isdbt;
   } u;
 
   // For scan file configurations
@@ -583,15 +662,25 @@ static inline int dvb_bandwidth( dvb_fe_bandwidth_t bw )
   return bw < 1000 ? 0 : bw * 1000;
 }
 
-int dvb_delsys2type ( enum dvb_fe_delivery_system ds );
+int dvb_delsys2type ( struct mpegts_network *ln, enum dvb_fe_delivery_system ds );
 
-void dvb_mux_conf_init ( dvb_mux_conf_t *dmc, dvb_fe_delivery_system_t delsys );
+void dvb_mux_conf_init ( struct mpegts_network *ln, dvb_mux_conf_t *dmc,
+                         dvb_fe_delivery_system_t delsys );
 
 int dvb_mux_conf_str ( dvb_mux_conf_t *conf, char *buf, size_t bufsize );
 
 const char *dvb_sat_position_to_str( int position, char *buf, size_t buflen );
 
-const int dvb_sat_position_from_str( const char *buf );
+int dvb_sat_position_from_str( const char *buf );
+
+static inline int dvb_modulation_is_none_or_auto ( int modulation )
+{
+  return modulation == DVB_MOD_NONE ||
+         modulation == DVB_MOD_AUTO ||
+         modulation == DVB_MOD_QAM_AUTO;
+}
+
+uint32_t dvb_sat_pls( dvb_mux_conf_t *dmc );
 
 #endif /* ENABLE_MPEGTS_DVB */
 
