@@ -1,4 +1,4 @@
-/*
+ /*
  *  Tvheadend - Linux DVB CA
  *
  *  Copyright (C) 2015 Damjan Marion
@@ -20,9 +20,6 @@
 #include "tvheadend.h"
 #include "linuxdvb_private.h"
 #include "notify.h"
-#include "atomic.h"
-#include "tvhpoll.h"
-#include "streaming.h"
 
 #include <sys/ioctl.h>
 #include <sys/types.h>
@@ -34,8 +31,11 @@
 #include <linux/dvb/frontend.h>
 #include <linux/dvb/ca.h>
 
+#include "descrambler/caid.h"
 #include "descrambler/dvbcam.h"
 
+static void linuxdvb_ca_monitor ( void *aux );
+
 typedef enum {
   CA_SLOT_STATE_EMPTY = 0,
   CA_SLOT_STATE_MODULE_PRESENT,
@@ -108,49 +108,57 @@ ciplus13_app_ai_data_rate_info(linuxdvb_ca_t *lca, ciplus13_data_rate_t rate)
   if (lca->lca_ai_version != 3)
     return 0;
 
-  tvhinfo("en50221", "setting CI+ CAM data rate to %s Mbps", rate ? "96":"72");
+  tvhinfo(LS_EN50221, "setting CI+ CAM data rate to %s Mbps", rate ? "96":"72");
 
   return en50221_sl_send_data(lca->lca_sl, lca->lca_ai_session_number, data, sizeof(data));
 }
 
 static void
-linuxdvb_ca_class_save ( idnode_t *in )
+linuxdvb_ca_class_changed ( idnode_t *in )
 {
   linuxdvb_adapter_t *la = ((linuxdvb_ca_t*)in)->lca_adapter;
-  linuxdvb_adapter_save(la);
+  linuxdvb_adapter_changed(la);
 }
 
 static void
-linuxdvb_ca_class_enabled_notify ( void *p )
+linuxdvb_ca_class_enabled_notify ( void *p, const char *lang )
 {
   linuxdvb_ca_t *lca = (linuxdvb_ca_t *) p;
 
   if (lca->lca_enabled) {
     if (lca->lca_ca_fd < 0) {
       lca->lca_ca_fd = tvh_open(lca->lca_ca_path, O_RDWR | O_NONBLOCK, 0);
-      tvhtrace("linuxdvb", "opening ca%u %s (fd %d)",
+      tvhtrace(LS_LINUXDVB, "opening ca%u %s (fd %d)",
                lca->lca_number, lca->lca_ca_path, lca->lca_ca_fd);
+      if (lca->lca_ca_fd >= 0)
+        mtimer_arm_rel(&lca->lca_monitor_timer, linuxdvb_ca_monitor, lca, ms2mono(250));
     }
   } else {
-    tvhtrace("linuxdvb", "closing ca%u %s (fd %d)",
+    tvhtrace(LS_LINUXDVB, "closing ca%u %s (fd %d)",
              lca->lca_number, lca->lca_ca_path, lca->lca_ca_fd);
 
+    mtimer_disarm(&lca->lca_monitor_timer);
+
     if (lca->lca_en50221_thread_running) {
       lca->lca_en50221_thread_running = 0;
       pthread_join(lca->lca_en50221_thread, NULL);
     }
 
-    ioctl(lca->lca_ca_fd, CA_RESET, NULL);
+    if (lca->lca_ca_fd >= 0) {
+      if (ioctl(lca->lca_ca_fd, CA_RESET, NULL))
+        tvherror(LS_LINUXDVB, "unable to reset ca%u %s",
+                 lca->lca_number, lca->lca_ca_path);
 
-    close(lca->lca_ca_fd);
-    lca->lca_ca_fd = -1;
+      close(lca->lca_ca_fd);
+      lca->lca_ca_fd = -1;
+    }
 
-    idnode_notify_title_changed(&lca->lca_id);
+    idnode_notify_title_changed(&lca->lca_id, lang);
   }
 }
 
 static void
-linuxdvb_ca_class_high_bitrate_notify ( void *p )
+linuxdvb_ca_class_high_bitrate_notify ( void *p, const char *lang )
 {
   linuxdvb_ca_t *lca = (linuxdvb_ca_t *) p;
   ciplus13_app_ai_data_rate_info(lca, lca->lca_high_bitrate_mode ?
@@ -159,7 +167,7 @@ linuxdvb_ca_class_high_bitrate_notify ( void *p )
 }
 
 static const char *
-linuxdvb_ca_class_get_title ( idnode_t *in )
+linuxdvb_ca_class_get_title ( idnode_t *in, const char *lang )
 {
   linuxdvb_ca_t *lca = (linuxdvb_ca_t *) in;
   static char buf[256];
@@ -174,38 +182,58 @@ linuxdvb_ca_class_get_title ( idnode_t *in )
   return buf;
 }
 
+static const void *
+linuxdvb_ca_class_active_get ( void *obj )
+{
+  static int active;
+  linuxdvb_ca_t *lca = (linuxdvb_ca_t*)obj;
+  active = !!lca->lca_enabled;
+  return &active;
+}
+
 const idclass_t linuxdvb_ca_class =
 {
   .ic_class      = "linuxdvb_ca",
-  .ic_caption    = "Linux DVB CA",
-  .ic_save       = linuxdvb_ca_class_save,
-  .ic_get_title   = linuxdvb_ca_class_get_title,
+  .ic_caption    = N_("Linux DVB CA"),
+  .ic_changed    = linuxdvb_ca_class_changed,
+  .ic_get_title  = linuxdvb_ca_class_get_title,
   .ic_properties = (const property_t[]) {
+    {
+      .type     = PT_BOOL,
+      .id       = "active",
+      .name     = N_("Active"),
+      .opts     = PO_RDONLY | PO_NOSAVE | PO_NOUI,
+      .get      = linuxdvb_ca_class_active_get,
+    },
     {
       .type     = PT_BOOL,
       .id       = "enabled",
-      .name     = "Enabled",
+      .name     = N_("Enabled"),
+      .desc     = N_("Enable/disable the device."),
       .off      = offsetof(linuxdvb_ca_t, lca_enabled),
       .notify   = linuxdvb_ca_class_enabled_notify,
     },
     {
       .type     = PT_BOOL,
       .id       = "high_bitrate_mode",
-      .name     = "High Bitrate Mode (CI+ CAMs Only)",
+      .name     = N_("High bitrate mode (CI+ CAMs only)"),
+      .desc     = N_("Allow high bitrate mode (CI+ CAMs only)."),
       .off      = offsetof(linuxdvb_ca_t, lca_high_bitrate_mode),
       .notify   = linuxdvb_ca_class_high_bitrate_notify,
     },
     {
       .type     = PT_BOOL,
       .id       = "pin_reply",
-      .name     = "Reply to CAM PIN Enquiries",
+      .name     = N_("Reply to CAM PIN inquiries"),
+      .desc     = N_("Reply to PIN inquiries."),
       .off      = offsetof(linuxdvb_ca_t, lca_pin_reply),
       .opts     = PO_ADVANCED,
     },
     {
       .type     = PT_STR,
       .id       = "pin",
-      .name     = "PIN",
+      .name     = N_("PIN"),
+      .desc     = N_("The PIN to use."),
       .off      = offsetof(linuxdvb_ca_t, lca_pin_str),
       .opts     = PO_ADVANCED | PO_PASSWORD,
       .def.s    = "1234",
@@ -213,22 +241,51 @@ const idclass_t linuxdvb_ca_class =
     {
       .type     = PT_STR,
       .id       = "pin_match",
-      .name     = "PIN Enquiry Match String",
+      .name     = N_("PIN inquiry match string"),
+      .desc     = N_("PIN inquiry match string."),
       .off      = offsetof(linuxdvb_ca_t, lca_pin_match_str),
       .opts     = PO_ADVANCED,
       .def.s    = "PIN",
     },
+    {
+      .type     = PT_INT,
+      .id       = "capmt_interval",
+      .name     = N_("CAPMT interval (ms)"),
+      .desc     = N_("CAPMT interval (in ms)."),
+      .off      = offsetof(linuxdvb_ca_t, lca_capmt_interval),
+      .opts     = PO_ADVANCED,
+      .def.i    = 100,
+    },
+    {
+      .type     = PT_INT,
+      .id       = "capmt_query_interval",
+      .name     = N_("CAPMT query interval (ms)"),
+      .desc     = N_("CAPMT query interval (ms)."),
+      .off      = offsetof(linuxdvb_ca_t, lca_capmt_query_interval),
+      .opts     = PO_ADVANCED,
+      .def.i    = 1200,
+    },
+    {
+      .type     = PT_BOOL,
+      .id       = "query_before_ok_descrambling",
+      .name     = N_("Send CAPMT query"),
+      .desc     = N_("Send CAPMT OK query before descrambling."),
+      .off      = offsetof(linuxdvb_ca_t, lca_capmt_query),
+      .opts     = PO_ADVANCED,
+    },
     {
       .type     = PT_STR,
       .id       = "ca_path",
-      .name     = "Device Path",
+      .name     = N_("Device path"),
+      .desc     = N_("Path used by the device."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(linuxdvb_ca_t, lca_ca_path),
     },
     {
       .type     = PT_STR,
       .id       = "slot_state",
-      .name     = "Slot State",
+      .name     = N_("Slot state"),
+      .desc     = N_("The CAM slot status."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(linuxdvb_ca_t, lca_state_str),
     },
@@ -251,9 +308,9 @@ en50221_app_unknown_message(void *arg, uint8_t slot_id,
                             uint16_t session_num, uint32_t resource_id,
                             uint8_t *data, uint32_t data_length)
 {
-  tvhtrace("en50221", "unknown message slot_id %u, session_num %u, resource_id %x",
+  tvhtrace(LS_EN50221, "unknown message slot_id %u, session_num %u, resource_id %x",
            slot_id, session_num, resource_id);
-  tvhlog_hexdump("en50221", data, data_length);
+  tvhlog_hexdump(LS_EN50221, data, data_length);
   return 0;
 }
 
@@ -293,7 +350,7 @@ linuxdvb_ca_lookup_cb (void * arg, uint8_t slot_id, uint32_t requested_rid,
         *connected_rid = EN50221_APP_MMI_RESOURCEID;
         break;
       default:
-        tvhtrace("en50221", "lookup cb for unknown resource id %x on slot %u",
+        tvhtrace(LS_EN50221, "lookup cb for unknown resource id %x on slot %u",
                  requested_rid, slot_id);
         *callback_out = (en50221_sl_resource_callback) en50221_app_unknown_message;
         *arg_out = lca;
@@ -309,12 +366,12 @@ linuxdvb_ca_session_cb (void *arg, int reason, uint8_t slot_id,
   linuxdvb_ca_t * lca = arg;
 
   if (reason == S_SCALLBACK_REASON_CAMCONNECTING) {
-    tvhtrace("en50221", "0x%08x session %u connecting", rid, session_num);
+    tvhtrace(LS_EN50221, "0x%08x session %u connecting", rid, session_num);
     return 0;
   }
 
   if (reason == S_SCALLBACK_REASON_CAMCONNECTED) {
-    tvhtrace("en50221", "0x%08x session %u connected", rid, session_num);
+    tvhtrace(LS_EN50221, "0x%08x session %u connected", rid, session_num);
     switch(rid) {
       case EN50221_APP_RM_RESOURCEID:
         en50221_app_rm_enq(lca->lca_rm_resource, session_num);
@@ -337,14 +394,14 @@ linuxdvb_ca_session_cb (void *arg, int reason, uint8_t slot_id,
       case EN50221_APP_DATETIME_RESOURCEID:
         break;
       default:
-        tvhtrace("en50221", "session %u with unknown rid 0x%08x is connected",
+        tvhtrace(LS_EN50221, "session %u with unknown rid 0x%08x is connected",
                  session_num, rid);
     }
     return 0;
   }
 
   if (reason == S_SCALLBACK_REASON_CLOSE) {
-    tvhtrace("en50221", "0x%08x session %u close", rid, session_num);
+    tvhtrace(LS_EN50221, "0x%08x session %u close", rid, session_num);
     switch(rid) {
       case EN50221_APP_CA_RESOURCEID:
         dvbcam_unregister_cam(lca, 0);
@@ -354,11 +411,11 @@ linuxdvb_ca_session_cb (void *arg, int reason, uint8_t slot_id,
   }
 
   if (reason == S_SCALLBACK_REASON_TC_CONNECT) {
-    tvhtrace("en50221", "0x%08x session %u tc connect", rid, session_num);
+    tvhtrace(LS_EN50221, "0x%08x session %u tc connect", rid, session_num);
     return 0;
   }
 
-  tvhtrace("en50221", "unhandled session callback - reason %d slot_id %u "
+  tvhtrace(LS_EN50221, "unhandled session callback - reason %d slot_id %u "
            "session_num %u resource_id %x", reason, slot_id, session_num, rid);
   return 0;
 }
@@ -368,12 +425,12 @@ linuxdvb_ca_rm_enq_cb(void *arg, uint8_t slot_id, uint16_t session_num)
 {
     linuxdvb_ca_t * lca = arg;
 
-    tvhtrace("en50221", "rm enq callback received for slot %d", slot_id);
+    tvhtrace(LS_EN50221, "rm enq callback received for slot %d", slot_id);
 
     if (en50221_app_rm_reply(lca->lca_rm_resource, session_num,
         sizeof(resource_ids)/4, resource_ids))
     {
-      tvherror("en50221", "failed to send rm reply to slot %u session %u",
+      tvherror(LS_EN50221, "failed to send rm reply to slot %u session %u",
                slot_id, session_num);
     }
 
@@ -386,16 +443,16 @@ linuxdvb_ca_rm_reply_cb(void *arg, uint8_t slot_id, uint16_t session_num,
 {
     linuxdvb_ca_t * lca = arg;
 
-    tvhtrace("en50221", "rm reply cb received for slot %d, count %u", slot_id,
+    tvhtrace(LS_EN50221, "rm reply cb received for slot %d, count %u", slot_id,
              resource_id_count);
 
     uint32_t i;
     for(i=0; i< resource_id_count; i++) {
-        tvhtrace("en50221", "CAM provided resource id: %08x", _resource_ids[i]);
+        tvhtrace(LS_EN50221, "CAM provided resource id: %08x", _resource_ids[i]);
     }
 
     if (en50221_app_rm_changed(lca->lca_rm_resource, session_num)) {
-        tvherror("en50221", "failed to send RM reply on slot %u", slot_id);
+        tvherror(LS_EN50221, "failed to send RM reply on slot %u", slot_id);
     }
 
     return 0;
@@ -406,10 +463,10 @@ linuxdvb_ca_rm_changed_cb(void *arg, uint8_t slot_id,
                           uint16_t session_num)
 {
     linuxdvb_ca_t * lca = arg;
-    tvhtrace("en50221", "rm changed cb received for slot %d", slot_id);
+    tvhtrace(LS_EN50221, "rm changed cb received for slot %d", slot_id);
 
     if (en50221_app_rm_enq(lca->lca_rm_resource, session_num)) {
-        tvherror("en50221", "failed to send ENQ to slot %d", slot_id);
+        tvherror(LS_EN50221, "failed to send ENQ to slot %d", slot_id);
     }
 
     return 0;
@@ -420,10 +477,10 @@ linuxdvb_ca_dt_enquiry_cb(void *arg, uint8_t slot_id, uint16_t session_num,
                           uint8_t response_int)
 {
     linuxdvb_ca_t * lca = arg;
-    tvhtrace("en50221", "datetime enquiry cb received for slot %d", slot_id);
+    tvhtrace(LS_EN50221, "datetime enquiry cb received for slot %d", slot_id);
 
     if (en50221_app_datetime_send(lca->lca_dt_resource, session_num, time(NULL), -1)) {
-        tvherror("en50221", "Failed to send datetime to slot %d", slot_id);
+        tvherror(LS_EN50221, "Failed to send datetime to slot %d", slot_id);
     }
 
     return 0;
@@ -437,17 +494,17 @@ linuxdvb_ca_ai_callback(void *arg, uint8_t slot_id, uint16_t session_num,
 {
     linuxdvb_ca_t * lca = arg;
 
-    tvhinfo("en50221", "CAM slot %u: Application type: %02x, manufacturer: %04x,"
+    tvhinfo(LS_EN50221, "CAM slot %u: Application type: %02x, manufacturer: %04x,"
             " Manufacturer code: %04x", slot_id, app_type, app_manufacturer,
             manufacturer_code);
 
-    tvhinfo("en50221", "CAM slot %u: Menu string: %.*s", slot_id,
+    tvhinfo(LS_EN50221, "CAM slot %u: Menu string: %.*s", slot_id,
             menu_string_len, menu_string);
 
     snprintf(lca->lca_cam_menu_string, sizeof(lca->lca_cam_menu_string),
              "%.*s", menu_string_len, menu_string);
 
-    idnode_notify_title_changed(&lca->lca_id);
+    idnode_notify_title_changed(&lca->lca_id, NULL);
 
     return 0;
 }
@@ -457,21 +514,20 @@ linuxdvb_ca_ca_info_callback(void *arg, uint8_t slot_id, uint16_t session_num,
                              uint32_t ca_id_count, uint16_t *ca_ids)
 {
     linuxdvb_ca_t * lca = arg;
-    uint32_t i;
+    uint32_t i, j;
     char buf[256];
     size_t c = 0;
 
     dvbcam_unregister_cam(lca, 0);
     dvbcam_register_cam(lca, 0, ca_ids, ca_id_count);
 
-
-    for(i=0; i< ca_id_count; i++) {
-        tvh_strlcatf(buf, sizeof(buf), c, " %04X", ca_ids[i]);
-        tvh_strlcatf(buf, sizeof(buf), c, " (%s)",
-                     descrambler_caid2name(ca_ids[i]));
+    for (i = 0; i < ca_id_count; ) {
+      for (j = 0, buf[0] = '\0'; j < 4 && i < ca_id_count; i++, j++)
+          tvh_strlcatf(buf, sizeof(buf), c, " %04X (%s)",
+                       ca_ids[i], caid2name(ca_ids[i]));
+      tvhinfo(LS_EN50221, "CAM slot %u supported CAIDs: %s", slot_id, buf);
     }
 
-    tvhinfo("en50221", "CAM slot %u supported CAIDs: %s", slot_id, buf);
     return 0;
 }
 
@@ -491,7 +547,7 @@ linuxdvb_ca_ca_pmt_reply_cb(void *arg, uint8_t slot_id, uint16_t session_num,
       default:   str = "state unknown (unknown value received)";
     }
 
-    tvhinfo("en50221", "CAM slot %u: descrambling %s", slot_id, str);
+    tvhinfo(LS_EN50221, "CAM slot %u: descrambling %s", slot_id, str);
 
     return 0;
 }
@@ -500,7 +556,7 @@ static int
 linuxdvb_ca_mmi_close_cb(void *arg, uint8_t slot_id, uint16_t session_num,
                          uint8_t cmd_id, uint8_t delay)
 {
-    tvhtrace("en50221", "mmi close cb received for slot %u session_num %u "
+    tvhtrace(LS_EN50221, "mmi close cb received for slot %u session_num %u "
              "cmd_id 0x%02x delay %u" , slot_id, session_num, cmd_id, delay);
 
     return 0;
@@ -512,7 +568,7 @@ linuxdvb_ca_mmi_display_ctl_cb(void *arg, uint8_t slot_id, uint16_t session_num,
 {
     linuxdvb_ca_t * lca = arg;
 
-    tvhtrace("en50221", "mmi display ctl cb received for slot %u session_num %u "
+    tvhtrace(LS_EN50221, "mmi display ctl cb received for slot %u session_num %u "
              "cmd_id 0x%02x mmi_mode %u" , slot_id, session_num, cmd_id, mmi_mode);
 
     if (cmd_id == MMI_DISPLAY_CONTROL_CMD_ID_SET_MMI_MODE) {
@@ -521,7 +577,7 @@ linuxdvb_ca_mmi_display_ctl_cb(void *arg, uint8_t slot_id, uint16_t session_num,
         d.u.mode_ack.mmi_mode = mmi_mode;
         if (en50221_app_mmi_display_reply(lca->lca_mmi_resource, session_num,
                                           MMI_DISPLAY_REPLY_ID_MMI_MODE_ACK, &d)) {
-            tvherror("en50221","Slot %u: Failed to send MMI mode ack reply", slot_id);
+            tvherror(LS_EN50221,"Slot %u: Failed to send MMI mode ack reply", slot_id);
         }
     }
 
@@ -538,14 +594,14 @@ linuxdvb_ca_mmi_enq_cb(void *arg, uint8_t slot_id, uint16_t session_num,
 
     snprintf(buffer, sizeof(buffer), "%.*s", text_size, text);
 
-    tvhlog(LOG_NOTICE, "en50221", "MMI enquiry from CAM in slot %u:  %s (%s%u digits)",
-           slot_id, buffer, blind_answ ? "blind " : "" , exp_answ_len);
+    tvhnotice(LS_EN50221, "MMI enquiry from CAM in slot %u:  %s (%s%u digits)",
+              slot_id, buffer, blind_answ ? "blind " : "" , exp_answ_len);
 
     if (lca->lca_pin_reply &&
         (strlen((char *) lca->lca_pin_str) == exp_answ_len) &&
         strstr((char *) buffer, lca->lca_pin_match_str))
     {
-      tvhtrace("en50221", "answering to PIN enquiry");
+      tvhtrace(LS_EN50221, "answering to PIN enquiry");
       en50221_app_mmi_answ(lca->lca_mmi_resource, session_num,
                            MMI_ANSW_ID_ANSWER, (uint8_t *) lca->lca_pin_str,
                            exp_answ_len);
@@ -567,15 +623,15 @@ linuxdvb_ca_mmi_menu_cb(void *arg, uint8_t slot_id, uint16_t session_num,
 {
     linuxdvb_ca_t * lca = arg;
 
-    tvhlog(LOG_NOTICE, "en50221", "MMI menu from CAM in the slot %u:", slot_id);
-    tvhlog(LOG_NOTICE, "en50221", "  title:    %.*s", title->text_length, title->text);
-    tvhlog(LOG_NOTICE, "en50221", "  subtitle: %.*s", sub_title->text_length, sub_title->text);
+    tvhnotice(LS_EN50221, "MMI menu from CAM in the slot %u:", slot_id);
+    tvhnotice(LS_EN50221, "  title:    %.*s", title->text_length, title->text);
+    tvhnotice(LS_EN50221, "  subtitle: %.*s", sub_title->text_length, sub_title->text);
 
     uint32_t i;
     for(i=0; i< item_count; i++) {
-        tvhlog(LOG_NOTICE, "en50221", "  item %i:   %.*s", i+1,  items[i].text_length, items[i].text);
+        tvhnotice(LS_EN50221, "  item %i:   %.*s", i+1,  items[i].text_length, items[i].text);
     }
-    tvhlog(LOG_NOTICE, "en50221", "  bottom:   %.*s", bottom->text_length, bottom->text);
+    tvhnotice(LS_EN50221, "  bottom:   %.*s", bottom->text_length, bottom->text);
 
     /* cancel menu */
     en50221_app_mmi_close(lca->lca_mmi_resource, session_num,
@@ -593,15 +649,15 @@ linuxdvb_ca_app_mmi_list_cb(void *arg, uint8_t slot_id, uint16_t session_num,
 {
     linuxdvb_ca_t * lca = arg;
 
-    tvhlog(LOG_NOTICE, "en50221", "MMI list from CAM in the slot %u:", slot_id);
-    tvhlog(LOG_NOTICE, "en50221", "  title:    %.*s", title->text_length, title->text);
-    tvhlog(LOG_NOTICE, "en50221", "  subtitle: %.*s", sub_title->text_length, sub_title->text);
+    tvhnotice(LS_EN50221, "MMI list from CAM in the slot %u:", slot_id);
+    tvhnotice(LS_EN50221, "  title:    %.*s", title->text_length, title->text);
+    tvhnotice(LS_EN50221, "  subtitle: %.*s", sub_title->text_length, sub_title->text);
 
     uint32_t i;
     for(i=0; i< item_count; i++) {
-        tvhlog(LOG_NOTICE, "en50221", "  item %i:   %.*s", i+1,  items[i].text_length, items[i].text);
+        tvhnotice(LS_EN50221, "  item %i:   %.*s", i+1,  items[i].text_length, items[i].text);
     }
-    tvhlog(LOG_NOTICE, "en50221", "  bottom:   %.*s", bottom->text_length, bottom->text);
+    tvhnotice(LS_EN50221, "  bottom:   %.*s", bottom->text_length, bottom->text);
 
     /* cancel menu */
     en50221_app_mmi_close(lca->lca_mmi_resource, session_num,
@@ -617,19 +673,19 @@ linuxdvb_ca_en50221_thread ( void *aux )
 
   lca->lca_tl = en50221_tl_create(5, 32);
   if (!lca->lca_tl) {
-    tvherror("en50221", "failed to create transport layer");
+    tvherror(LS_EN50221, "failed to create transport layer");
     return NULL;
   }
 
   slot_id = en50221_tl_register_slot(lca->lca_tl, lca->lca_ca_fd, 0, 1000, 100);
   if (slot_id < 0) {
-    tvherror("en50221", "slot registration failed");
+    tvherror(LS_EN50221, "slot registration failed");
     return NULL;
   }
 
   lca->lca_sl = en50221_sl_create(lca->lca_tl, 256);
   if (lca->lca_sl == NULL) {
-    tvherror("en50221", "failed to create session layer");
+    tvherror(LS_EN50221, "failed to create session layer");
     return NULL;
   }
 
@@ -666,11 +722,11 @@ linuxdvb_ca_en50221_thread ( void *aux )
 
   lca->lca_tc = en50221_tl_new_tc(lca->lca_tl, slot_id);
 
-  while (tvheadend_running && lca->lca_en50221_thread_running) {
+  while (tvheadend_is_running() && lca->lca_en50221_thread_running) {
         int error;
         if ((error = en50221_tl_poll(lca->lca_tl)) != 0) {
             if (error != lasterror) {
-                tvherror("en50221", "poll error on slot %d [error:%i]",
+                tvherror(LS_EN50221, "poll error on slot %d [error:%i]",
                         en50221_tl_get_error_slot(lca->lca_tl),
                         en50221_tl_get_error(lca->lca_tl));
             }
@@ -693,45 +749,43 @@ linuxdvb_ca_monitor ( void *aux )
   ca_slot_info_t csi;
   int state;
 
-  csi.num = 0;
+  if (lca->lca_ca_fd < 0)
+    return;
 
-  if (lca->lca_ca_fd >= 0) {
-    if ((ioctl(lca->lca_ca_fd, CA_GET_SLOT_INFO, &csi)) != 0) {
-      tvherror("linuxdvb", "failed to get CAM slot %u info [e=%s]",
-               csi.num, strerror(errno));
-    }
-    if (csi.flags & CA_CI_MODULE_READY)
-      state = CA_SLOT_STATE_MODULE_READY;
-    else if (csi.flags & CA_CI_MODULE_PRESENT)
-      state = CA_SLOT_STATE_MODULE_PRESENT;
-    else
-      state = CA_SLOT_STATE_EMPTY;
+  memset(&csi, 0, sizeof(csi));
 
-    lca->lca_state_str = ca_slot_state2str(state);
+  if ((ioctl(lca->lca_ca_fd, CA_GET_SLOT_INFO, &csi)) != 0) {
+    tvherror(LS_LINUXDVB, "failed to get CAM slot %u info [e=%s]",
+             csi.num, strerror(errno));
+  }
+  if (csi.flags & CA_CI_MODULE_READY)
+    state = CA_SLOT_STATE_MODULE_READY;
+  else if (csi.flags & CA_CI_MODULE_PRESENT)
+    state = CA_SLOT_STATE_MODULE_PRESENT;
+  else
+    state = CA_SLOT_STATE_EMPTY;
 
-    if (lca->lca_state != state) {
-      tvhlog(LOG_INFO, "linuxdvb", "CAM slot %u status changed to %s",
-             csi.num, lca->lca_state_str);
-      idnode_notify_title_changed(&lca->lca_id);
-      lca->lca_state = state;
-    }
-
-    if ((!lca->lca_en50221_thread_running) &&
-        (state == CA_SLOT_STATE_MODULE_READY)) 
-    {
-      lca->lca_en50221_thread_running = 1;
-      tvhthread_create(&lca->lca_en50221_thread, NULL,
-                       linuxdvb_ca_en50221_thread, lca);
-    } else if (lca->lca_en50221_thread_running &&
-               (state != CA_SLOT_STATE_MODULE_READY))
-    {
-      lca->lca_en50221_thread_running = 0;
-      pthread_join(lca->lca_en50221_thread, NULL);
-    }
+  lca->lca_state_str = ca_slot_state2str(state);
 
+  if (lca->lca_state != state) {
+    tvhnotice(LS_LINUXDVB, "CAM slot %u status changed to %s",
+              csi.num, lca->lca_state_str);
+    idnode_notify_title_changed(&lca->lca_id, NULL);
+    lca->lca_state = state;
   }
 
-  gtimer_arm_ms(&lca->lca_monitor_timer, linuxdvb_ca_monitor, lca, 250);
+  if ((!lca->lca_en50221_thread_running) &&
+      (state == CA_SLOT_STATE_MODULE_READY)) {
+    lca->lca_en50221_thread_running = 1;
+    tvhthread_create(&lca->lca_en50221_thread, NULL,
+                     linuxdvb_ca_en50221_thread, lca, "lnxdvb-ca");
+  } else if (lca->lca_en50221_thread_running &&
+             (state != CA_SLOT_STATE_MODULE_READY)) {
+    lca->lca_en50221_thread_running = 0;
+    pthread_join(lca->lca_en50221_thread, NULL);
+  }
+
+  mtimer_arm_rel(&lca->lca_monitor_timer, linuxdvb_ca_monitor, lca, ms2mono(250));
 }
 
 linuxdvb_ca_t *
@@ -747,6 +801,8 @@ linuxdvb_ca_create
   lca->lca_number = number;
   lca->lca_ca_path  = strdup(ca_path);
   lca->lca_ca_fd = -1;
+  lca->lca_capmt_interval = 100;
+  lca->lca_capmt_query_interval = 1200;
 
   /* Internal config ID */
   snprintf(id, sizeof(id), "ca%u", number);
@@ -770,8 +826,6 @@ linuxdvb_ca_create
 
   TAILQ_INIT(&lca->lca_capmt_queue);
 
-  gtimer_arm_ms(&lca->lca_monitor_timer, linuxdvb_ca_monitor, lca, 250);
-
   return lca;
 }
 
@@ -784,7 +838,7 @@ linuxdvb_ca_process_capmt_queue ( void *aux )
   struct section_ext *result;
   struct mpeg_pmt_section *pmt;
   uint8_t capmt[4096];
-  int size;
+  int size, i;
 
   lcc = TAILQ_FIRST(&lca->lca_capmt_queue);
 
@@ -792,17 +846,17 @@ linuxdvb_ca_process_capmt_queue ( void *aux )
     return;
 
   if (!(section = section_codec(lcc->data, lcc->len))){
-    tvherror("en50221", "failed to decode PMT section");
+    tvherror(LS_EN50221, "failed to decode PMT section");
     goto done;
   }
 
   if (!(result = section_ext_decode(section, 0))){
-    tvherror("en50221", "failed to decode PMT ext_section");
+    tvherror(LS_EN50221, "failed to decode PMT ext_section");
     goto done;
   }
 
   if (!(pmt = mpeg_pmt_section_codec(result))){
-    tvherror("en50221", "failed to decode PMT");
+    tvherror(LS_EN50221, "failed to decode PMT");
     goto done;
   }
 
@@ -810,19 +864,21 @@ linuxdvb_ca_process_capmt_queue ( void *aux )
                                lcc->list_mgmt, lcc->cmd_id);
 
   if (size < 0) {
-    tvherror("en50221", "Failed to format CAPMT");
+    tvherror(LS_EN50221, "Failed to format CAPMT");
   }
 
   if (en50221_app_ca_pmt(lca->lca_ca_resource, lca->lca_ca_session_number,
                          capmt, size)) {
-        tvherror("en50221", "Failed to send CAPMT");
+        tvherror(LS_EN50221, "Failed to send CAPMT");
   }
 
-  tvhtrace("en50221", "%s CAPMT sent (%s)", ca_pmt_cmd_id2str(lcc->cmd_id),
+  tvhtrace(LS_EN50221, "%s CAPMT sent (%s)", ca_pmt_cmd_id2str(lcc->cmd_id),
            ca_pmt_list_mgmt2str(lcc->list_mgmt));
-  tvhlog_hexdump("en50221", capmt, size);
+  tvhlog_hexdump(LS_EN50221, capmt, size);
 
 done:
+  i = (lcc->cmd_id == CA_PMT_CMD_ID_QUERY) ?
+    lca->lca_capmt_query_interval : lca->lca_capmt_interval;
 
   TAILQ_REMOVE(&lca->lca_capmt_queue, lcc, lcc_link);
 
@@ -830,8 +886,8 @@ done:
   free(lcc);
 
   if (!TAILQ_EMPTY(&lca->lca_capmt_queue)) {
-    gtimer_arm_ms(&lca->lca_capmt_queue_timer,
-                  linuxdvb_ca_process_capmt_queue, lca, 1000);
+    mtimer_arm_rel(&lca->lca_capmt_queue_timer,
+                   linuxdvb_ca_process_capmt_queue, lca, ms2mono(i));
   }
 }
 
@@ -840,37 +896,43 @@ linuxdvb_ca_enqueue_capmt(linuxdvb_ca_t *lca, uint8_t slot, const uint8_t *ptr,
                           int len, uint8_t list_mgmt, uint8_t cmd_id)
 {
   linuxdvb_ca_capmt_t *lcc;
+  int c = 1;
 
   if (!lca)
     return;
 
-  lcc = calloc(1, sizeof(*lcc));
+  if (lca->lca_capmt_query && cmd_id == CA_PMT_CMD_ID_OK_DESCRAMBLING)
+    c = 2;
 
-  if (!lcc)
-    return;
+  while (c--) {
+    lcc = calloc(1, sizeof(*lcc));
 
-  lcc->data = malloc(len);
-  lcc->len = len;
-  lcc->slot = slot;
-  lcc->list_mgmt = list_mgmt;
-  lcc->cmd_id = cmd_id;
-  memcpy(lcc->data, ptr, len);
+    if (!lcc)
+      return;
 
-  TAILQ_INSERT_TAIL(&lca->lca_capmt_queue, lcc, lcc_link);
+    lcc->data = malloc(len);
+    lcc->len = len;
+    lcc->slot = slot;
+    lcc->list_mgmt = list_mgmt;
+    lcc->cmd_id = (c ? CA_PMT_CMD_ID_QUERY : cmd_id);
+    memcpy(lcc->data, ptr, len);
 
-  tvhtrace("en50221", "%s CAPMT enqueued (%s)", ca_pmt_cmd_id2str(lcc->cmd_id),
-           ca_pmt_list_mgmt2str(lcc->list_mgmt));
+    TAILQ_INSERT_TAIL(&lca->lca_capmt_queue, lcc, lcc_link);
 
-  gtimer_arm_ms(&lca->lca_capmt_queue_timer,
-                linuxdvb_ca_process_capmt_queue, lca, 50);
+    tvhtrace(LS_EN50221, "%s CAPMT enqueued (%s)", ca_pmt_cmd_id2str(lcc->cmd_id),
+             ca_pmt_list_mgmt2str(lcc->list_mgmt));
+  }
+
+  mtimer_arm_rel(&lca->lca_capmt_queue_timer,
+                 linuxdvb_ca_process_capmt_queue, lca, ms2mono(50));
 }
 
 void linuxdvb_ca_save( linuxdvb_ca_t *lca, htsmsg_t *msg )
 {
-  char id[8];
+  char id[8], ubuf[UUID_HEX_SIZE];
   htsmsg_t *m = htsmsg_create_map();
 
-  htsmsg_add_str(m, "uuid", idnode_uuid_as_str(&lca->lca_id));
+  htsmsg_add_str(m, "uuid", idnode_uuid_as_str(&lca->lca_id, ubuf));
   idnode_save(&lca->lca_id, m);
 
   /* Add to list */
