@@ -48,13 +48,12 @@ mpegts_service_class_get_mux ( void *ptr )
 static const void *
 mpegts_service_class_get_mux_uuid ( void *ptr )
 {
-  static char buf[UUID_HEX_SIZE], *s = buf;
   mpegts_service_t *ms = ptr;
   if (ms && ms->s_dvb_mux)
-    strcpy(buf, idnode_uuid_as_str(&ms->s_dvb_mux->mm_id) ?: "");
+    idnode_uuid_as_str(&ms->s_dvb_mux->mm_id, prop_sbuf);
   else
-    *buf = 0;
-  return &s;
+    prop_sbuf[0] = '\0';
+  return &prop_sbuf_ptr;
 }
 
 static const void *
@@ -71,148 +70,191 @@ mpegts_service_class_get_network ( void *ptr )
 }
 
 static htsmsg_t *
-mpegts_service_pref_capid_lock_list ( void *o )
+mpegts_service_pref_capid_lock_list ( void *o, const char *lang )
 {
   static const struct strtab tab[] = {
-    { "Off",                0 },
-    { "On",                 1 },
-    { "Only Pref. CA PID",  2 },
+    { N_("Off"),                    0 },
+    { N_("On"),                     1 },
+    { N_("Only preferred CA PID"),  2 },
   };
-   return strtab2htsmsg(tab);
+   return strtab2htsmsg(tab, 1, lang);
 }
 
+CLASS_DOC(mpegts_service)
+
 const idclass_t mpegts_service_class =
 {
   .ic_super      = &service_class,
   .ic_class      = "mpegts_service",
-  .ic_caption    = "MPEGTS Service",
+  .ic_caption    = N_("MPEG-TS Service"),
+  .ic_doc        = tvh_doc_mpegts_service_class,
   .ic_order      = "enabled,channel,svcname",
   .ic_properties = (const property_t[]){
     {
       .type     = PT_STR,
       .id       = "network",
-      .name     = "Network",
+      .name     = N_("Network"),
+      .desc     = N_("The network the service is on."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .get      = mpegts_service_class_get_network,
     },
     {
       .type     = PT_STR,
       .id       = "multiplex",
-      .name     = "Mux",
+      .name     = N_("Mux"),
+      .desc     = N_("The mux the service is on."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .get      = mpegts_service_class_get_mux,
     },
     {
       .type     = PT_STR,
       .id       = "multiplex_uuid",
-      .name     = "Mux UUID",
-      .opts     = PO_RDONLY | PO_NOSAVE | PO_HIDDEN,
+      .name     = N_("Mux UUID"),
+      .desc     = N_("The mux's universally unique identifier."),
+      .opts     = PO_RDONLY | PO_NOSAVE | PO_HIDDEN | PO_EXPERT,
       .get      = mpegts_service_class_get_mux_uuid,
     },
     {
       .type     = PT_U16,
       .id       = "sid",
-      .name     = "Service ID",
-      .opts     = PO_RDONLY,
+      .name     = N_("Service ID"),
+      .desc     = N_("The service ID as set by the provider."),
+      .opts     = PO_RDONLY | PO_ADVANCED,
       .off      = offsetof(mpegts_service_t, s_dvb_service_id),
     },
     {
       .type     = PT_U16,
       .id       = "lcn",
-      .name     = "Local Channel Number",
-      .opts     = PO_RDONLY,
+      .name     = N_("Local channel number"),
+      .desc     = N_("The service's channel number as set by the provider."),
+      .opts     = PO_RDONLY | PO_ADVANCED,
       .off      = offsetof(mpegts_service_t, s_dvb_channel_num),
     },
     {
       .type     = PT_U16,
       .id       = "lcn_minor",
-      .name     = "Local Channel Minor",
-      .opts     = PO_RDONLY,
+      .name     = N_("Local channel minor"),
+      .desc     = N_("The service's channel minor as set by the provider."),
+      .opts     = PO_RDONLY | PO_EXPERT,
       .off      = offsetof(mpegts_service_t, s_dvb_channel_minor),
     },
     {
       .type     = PT_U16,
       .id       = "lcn2",
-      .name     = "OpenTV Channel Number",
-      .opts     = PO_RDONLY,
+      .name     = N_("OpenTV channel number"),
+      .desc     = N_("The OpenTV channel number as set by the provider."),
+      .opts     = PO_RDONLY | PO_EXPERT,
       .off      = offsetof(mpegts_service_t, s_dvb_opentv_chnum),
     },
+    {
+      .type     = PT_U16,
+      .id       = "srcid",
+      .name     = N_("ATSC source ID"),
+      .desc     = N_("The ATSC source ID as set by the provider."),
+      .opts     = PO_RDONLY | PO_EXPERT,
+      .off      = offsetof(mpegts_service_t, s_atsc_source_id),
+    },
     {
       .type     = PT_STR,
       .id       = "svcname",
-      .name     = "Service Name",
+      .name     = N_("Service name"),
+      .desc     = N_("The service name as set by the provider."),
       .opts     = PO_RDONLY,
       .off      = offsetof(mpegts_service_t, s_dvb_svcname),
     },
     {
       .type     = PT_STR,
       .id       = "provider",
-      .name     = "Provider",
+      .name     = N_("Provider"),
+      .desc     = N_("The provider's name."),
       .opts     = PO_RDONLY | PO_HIDDEN,
       .off      = offsetof(mpegts_service_t, s_dvb_provider),
     },
     {
       .type     = PT_STR,
       .id       = "cridauth",
-      .name     = "CRID Authority",
-      .opts     = PO_RDONLY | PO_HIDDEN,
+      .name     = N_("CRID authority"),
+      .desc     = N_("Content reference identifier authority."),
+      .opts     = PO_RDONLY | PO_HIDDEN | PO_EXPERT,
       .off      = offsetof(mpegts_service_t, s_dvb_cridauth),
     },
     {
       .type     = PT_U16,
       .id       = "dvb_servicetype",
-      .name     = "Service Type",
-      .opts     = PO_RDONLY | PO_HIDDEN,
+      .name     = N_("Service type"),
+      .desc     = N_("The service type flag as defined by the DVB "
+                     "specifications (e.g. 0x02 = radio, 0x11 = MPEG2 "
+                     "HD TV, 0x19 = H.264 HD TV)"),
+      .opts     = PO_RDONLY | PO_HIDDEN | PO_EXPERT,
       .off      = offsetof(mpegts_service_t, s_dvb_servicetype),
     },
     {
       .type     = PT_BOOL,
       .id       = "dvb_ignore_eit",
-      .name     = "Ignore EPG (EIT)",
+      .name     = N_("Ignore EPG (EIT)"),
+      .desc     = N_("Enable or disable ignoring of Event Information "
+                     "Table (EIT) data on this mux."),
       .off      = offsetof(mpegts_service_t, s_dvb_ignore_eit),
-      .opts     = PO_ADVANCED,
+      .opts     = PO_EXPERT,
     },
     {
       .type     = PT_STR,
       .id       = "charset",
-      .name     = "Character Set",
+      .name     = N_("Character set"),
+      .desc     = N_("The character encoding for this service (e.g. UTF-8)."),
       .off      = offsetof(mpegts_service_t, s_dvb_charset),
       .list     = dvb_charset_enum,
-      .opts     = PO_ADVANCED,
+      .opts     = PO_EXPERT | PO_DOC_NLIST,
     },
     {
       .type     = PT_U16,
       .id       = "prefcapid",
-      .name     = "Pref. CA PID",
+      .name     = N_("Preferred CA PID"),
+      .desc     = N_("The Preferred Conditional Access Packet "
+                     "Identifier. Used for decrypting scrambled streams."),
       .off      = offsetof(mpegts_service_t, s_dvb_prefcapid),
-      .opts     = PO_ADVANCED,
+      .opts     = PO_EXPERT,
     },
     {
       .type     = PT_INT,
       .id       = "prefcapid_lock",
-      .name     = "Lock Pref. CA PID",
+      .name     = N_("Lock preferred CA PID"),
+      .desc     = N_("The locking mechanism selection for The Preferred "
+                     "Conditional Access Packet Identifier. See Help "
+                     "for more information."),
       .off      = offsetof(mpegts_service_t, s_dvb_prefcapid_lock),
-      .opts     = PO_ADVANCED,
+      .opts     = PO_EXPERT | PO_DOC_NLIST,
       .list     = mpegts_service_pref_capid_lock_list,
     },
     {
       .type     = PT_U16,
       .id       = "force_caid",
-      .name     = "Force CA ID (e.g. 0x2600)",
+      .name     = N_("Force CA ID (e.g. 0x2600)"),
+      .desc     = N_("Force usage of entered CA ID on this service."),
       .off      = offsetof(mpegts_service_t, s_dvb_forcecaid),
-      .opts     = PO_ADVANCED | PO_HEXA,
+      .opts     = PO_EXPERT | PO_HEXA,
+    },
+    {
+      .type     = PT_INT,
+      .id       = "pts_shift",
+      .name     = N_("Shift PTS (ms)"),
+      .desc     = N_("Add this value to PTS for the teletext subtitles. The time value is in milliseconds and may be negative."),
+      .off      = offsetof(mpegts_service_t, s_pts_shift),
+      .opts     = PO_EXPERT,
     },
     {
       .type     = PT_TIME,
       .id       = "created",
-      .name     = "Created",
+      .name     = N_("Created"),
+      .desc     = N_("When the service was first identified and recorded."),
       .off      = offsetof(mpegts_service_t, s_dvb_created),
       .opts     = PO_ADVANCED | PO_RDONLY,
     },
     {
       .type     = PT_TIME,
       .id       = "last_seen",
-      .name     = "Last Seen",
+      .name     = N_("Last seen"),
+      .desc     = N_("When the service was last seen during a mux scan."),
       .off      = offsetof(mpegts_service_t, s_dvb_last_seen),
       .opts     = PO_ADVANCED | PO_RDONLY,
     },
@@ -238,27 +280,23 @@ mpegts_service_is_enabled(service_t *t, int flags)
 /*
  * Save
  */
-static void
-mpegts_service_config_save ( service_t *t )
+static htsmsg_t *
+mpegts_service_config_save ( service_t *t, char *filename, size_t fsize )
 {
-  htsmsg_t *c = htsmsg_create_map();
   mpegts_service_t *s = (mpegts_service_t*)t;
-  service_save(t, c);
-  hts_settings_save(c, "input/dvb/networks/%s/muxes/%s/services/%s",
-                    idnode_uuid_as_str(&s->s_dvb_mux->mm_network->mn_id),
-                    idnode_uuid_as_str(&s->s_dvb_mux->mm_id),
-                    idnode_uuid_as_str(&s->s_id));
-  htsmsg_destroy(c);
+  idnode_changed(&s->s_dvb_mux->mm_id);
+  return NULL;
 }
 
 /*
  * Service instance list
  */
-static void
-mpegts_service_enlist(service_t *t, tvh_input_t *ti,
-                      struct service_instance_list *sil, int flags)
+static int
+mpegts_service_enlist_raw
+  ( service_t *t, tvh_input_t *ti, struct service_instance_list *sil,
+    int flags, int weight )
 {
-  int p = 0, w;
+  int p, w, r, added = 0, errcnt = 0;
   mpegts_service_t      *s = (mpegts_service_t*)t;
   mpegts_input_t        *mi;
   mpegts_mux_t          *m = s->s_dvb_mux;
@@ -279,26 +317,60 @@ mpegts_service_enlist(service_t *t, tvh_input_t *ti,
     if (ti && (tvh_input_t *)mi != ti)
       continue;
 
-    if (!mi->mi_is_enabled(mi, mmi->mmi_mux, flags)) continue;
+    r = mi->mi_is_enabled(mi, mmi->mmi_mux, flags, weight);
+    if (r == MI_IS_ENABLED_NEVER) {
+      tvhtrace(LS_MPEGTS, "enlist: input %p not enabled for mux %p service %s weight %d flags %x",
+                          mi, mmi->mmi_mux, s->s_nicename, weight, flags);
+      continue;
+    }
+    if (r == MI_IS_ENABLED_RETRY) {
+      /* temporary error - retry later */
+      tvhtrace(LS_MPEGTS, "enlist: input %p postponed for mux %p service %s weight %d flags %x",
+                          mi, mmi->mmi_mux, s->s_nicename, weight, flags);
+      errcnt++;
+      continue;
+    }
 
     /* Set weight to -1 (forced) for already active mux */
     if (mmi->mmi_mux->mm_active == mmi) {
       w = -1;
       p = -1;
     } else {
-      w = mi->mi_get_weight(mi, mmi->mmi_mux, flags);
+      w = mi->mi_get_weight(mi, mmi->mmi_mux, flags, weight);
       p = mi->mi_get_priority(mi, mmi->mmi_mux, flags);
+      if (w > 0 && mi->mi_free_weight &&
+          weight >= mi->mi_free_weight && w < mi->mi_free_weight)
+        w = 0;
     }
 
     service_instance_add(sil, t, mi->mi_instance, mi->mi_name, p, w);
+    added++;
   }
+
+  return added ? 0 : (errcnt ? SM_CODE_NO_FREE_ADAPTER : 0);
+}
+
+/*
+ * Service instance list
+ */
+static int
+mpegts_service_enlist
+  ( service_t *t, tvh_input_t *ti, struct service_instance_list *sil,
+    int flags, int weight )
+{
+  /* invalid PMT */
+  if (t->s_pmt_pid != SERVICE_PMT_AUTO &&
+      (t->s_pmt_pid <= 0 || t->s_pmt_pid >= 8191))
+    return SM_CODE_INVALID_SERVICE;
+
+  return mpegts_service_enlist_raw(t, ti, sil, flags, weight);
 }
 
 /*
  * Start service
  */
 static int
-mpegts_service_start(service_t *t, int instance, int flags)
+mpegts_service_start(service_t *t, int instance, int weight, int flags)
 {
   int r;
   mpegts_service_t      *s = (mpegts_service_t*)t;
@@ -319,14 +391,16 @@ mpegts_service_start(service_t *t, int instance, int flags)
     return SM_CODE_UNDEFINED_ERROR;
 
   /* Start Mux */
-  r = mpegts_mux_instance_start(&mmi, t);
+  mmi->mmi_start_weight = weight;
+  r = mpegts_mux_instance_start(&mmi, t, weight);
 
   /* Start */
   if (!r) {
 
     /* Open service */
     s->s_dvb_subscription_flags = flags;
-    mmi->mmi_input->mi_open_service(mmi->mmi_input, s, flags, 1);
+    s->s_dvb_subscription_weight = weight;
+    mmi->mmi_input->mi_open_service(mmi->mmi_input, s, flags, 1, weight);
   }
 
   return r;
@@ -368,7 +442,7 @@ mpegts_service_refresh(service_t *t)
   lock_assert(&global_lock);
 
   /* Re-open */
-  i->mi_open_service(i, s, s->s_dvb_subscription_flags, 0);
+  i->mi_open_service(i, s, s->s_dvb_subscription_flags, 0, s->s_dvb_subscription_weight);
 }
 
 /*
@@ -389,14 +463,20 @@ mpegts_service_setsourceinfo(service_t *t, source_info_t *si)
   memset(si, 0, sizeof(struct source_info));
   si->si_type = S_MPEG_TS;
 
+  uuid_duplicate(&si->si_network_uuid, &m->mm_network->mn_id.in_uuid);
+  uuid_duplicate(&si->si_mux_uuid, &m->mm_id.in_uuid);
+
   if(m->mm_network->mn_network_name != NULL)
     si->si_network = strdup(m->mm_network->mn_network_name);
+  mpegts_network_get_type_str(m->mm_network, buf, sizeof(buf));
+  si->si_network_type = strdup(buf);
 
   m->mm_display_name(m, buf, sizeof(buf));
   si->si_mux = strdup(buf);
 
   if(s->s_dvb_active_input) {
     mpegts_input_t *mi = s->s_dvb_active_input;
+    uuid_duplicate(&si->si_adapter_uuid, &mi->ti_id.in_uuid);
     mi->mi_display_name(mi, buf, sizeof(buf));
     si->si_adapter = strdup(buf);
   }
@@ -436,7 +516,7 @@ mpegts_service_grace_period(service_t *t)
 /*
  * Channel number
  */
-static int64_t
+int64_t
 mpegts_service_channel_number ( service_t *s )
 {
   mpegts_service_t *ms = (mpegts_service_t*)s;
@@ -494,7 +574,6 @@ mpegts_service_channel_icon ( service_t *s )
   if (ms->s_dvb_mux &&
       idnode_is_instance(&ms->s_dvb_mux->mm_id, &dvb_mux_class)) {
     int32_t hash = 0;
-    static char buf[128];
     dvb_mux_t *mmd = (dvb_mux_t*)ms->s_dvb_mux;
     int pos;
 
@@ -510,32 +589,101 @@ mpegts_service_channel_icon ( service_t *s )
                   (mmd->lm_tuning.u.dmc_fe_qpsk.orbital_pos == DVB_POLARISATION_HORIZONTAL ? 0x8000 : 0);
         break;
       case DVB_TYPE_C:
+      case DVB_TYPE_ATSC_C:
         hash = 0xFFFF0000;
         break;
       case DVB_TYPE_T:
         hash = 0xEEEE0000;
         break;
-      case DVB_TYPE_ATSC:
+      case DVB_TYPE_ATSC_T:
         hash = 0xDDDD0000;
         break;
       default:
         return NULL;
     }
 
-    snprintf(buf, sizeof(buf),
+    snprintf(prop_sbuf, PROP_SBUF_LEN,
              "picon://1_0_%X_%X_%X_%X_%X_0_0_0.png",
-             ms->s_dvb_servicetype,
+             config.picon_scheme == PICON_ISVCTYPE ? 1 : ms->s_dvb_servicetype,
              ms->s_dvb_service_id,
              ms->s_dvb_mux->mm_tsid,
              ms->s_dvb_mux->mm_onid,
              hash);
-    return buf;
+    return prop_sbuf;
   }
 #endif
 
   return NULL;
 }
 
+#if ENABLE_MPEGTS_DVB
+static int
+mpegts_service_match_network(mpegts_network_t *mn, uint32_t hash, const idclass_t *idc)
+{
+  int pos, pos2;
+
+  if (idc != &dvb_mux_dvbs_class) return 1;
+  pos = hash >> 16;
+  if (pos > 3600 || pos < 0) return 0;
+  pos = pos <= 1800 ? pos : pos - 3600;
+  if ((pos2 = dvb_network_get_orbital_pos(mn)) == INT_MAX) return 0;
+  return deltaI32(pos, pos2) <= 20;
+}
+#endif
+
+#if ENABLE_MPEGTS_DVB
+static int
+mpegts_service_match_mux(dvb_mux_t *mm, uint32_t hash, const idclass_t *idc)
+{
+  extern const idclass_t dvb_mux_dvbs_class;
+  dvb_mux_t *mmd;
+  int freq, pol;
+
+  if ((hash & 0xffff) == 0) return 1;
+  if (idc != &dvb_mux_dvbs_class) return 0;
+  freq = (hash & 0x7fff) * 1000;
+  pol = (hash & 0x8000) ? DVB_POLARISATION_HORIZONTAL : DVB_POLARISATION_VERTICAL;
+  mmd = mm;
+  return mmd->lm_tuning.u.dmc_fe_qpsk.orbital_pos == pol &&
+         deltaU32(mmd->lm_tuning.dmc_fe_freq, freq) < 2000;
+}
+#endif
+
+service_t *
+mpegts_service_find_e2(uint32_t stype, uint32_t sid, uint32_t tsid,
+                       uint32_t onid, uint32_t hash)
+{
+#if ENABLE_MPEGTS_DVB
+  mpegts_network_t *mn;
+  mpegts_mux_t *mm;
+  mpegts_service_t *s;
+  const idclass_t *idc;
+
+  lock_assert(&global_lock);
+
+  switch (hash & 0xFFFF0000) {
+  case 0xFFFF0000: idc = &dvb_mux_dvbc_class; break;
+  case 0xEEEE0000: idc = &dvb_mux_dvbt_class; break;
+  case 0xDDDD0000: idc = &dvb_mux_dvbt_class; break;
+  default:         idc = &dvb_mux_dvbs_class; break;
+  }
+  LIST_FOREACH(mn, &mpegts_network_all, mn_global_link) {
+    if (!idnode_is_instance(&mn->mn_id, &dvb_network_class)) continue;
+    if (dvb_network_mux_class(mn) != idc) continue;
+    if (!mpegts_service_match_network(mn, hash, idc)) continue;
+    LIST_FOREACH(mm, &mn->mn_muxes, mm_network_link) {
+      if (!idnode_is_instance(&mm->mm_id, idc)) continue;
+      if (mm->mm_tsid != tsid || mm->mm_onid != onid) continue;
+      if (!mpegts_service_match_mux((dvb_mux_t *)mm, hash, idc)) continue;
+      LIST_FOREACH(s, &mm->mm_services, s_dvb_mux_link)
+        if (s->s_dvb_service_id == sid)
+          return (service_t *)s;
+    }
+  }
+#endif
+  return NULL;
+}
+
 static void
 mpegts_service_mapped ( service_t *t )
 {
@@ -543,30 +691,36 @@ mpegts_service_mapped ( service_t *t )
 }
 
 void
-mpegts_service_delete ( service_t *t, int delconf )
+mpegts_service_unref ( service_t *t )
 {
   mpegts_service_t *ms = (mpegts_service_t*)t;
-  mpegts_mux_t     *mm = ms->s_dvb_mux;
 
-  /* Remove config */
-  if (delconf && t->s_type == STYPE_STD)
-    hts_settings_remove("input/dvb/networks/%s/muxes/%s/services/%s",
-                      idnode_uuid_as_str(&mm->mm_network->mn_id),
-                      idnode_uuid_as_str(&mm->mm_id),
-                      idnode_uuid_as_str(&t->s_id));
-
-  /* Free memory */
   free(ms->s_dvb_svcname);
   free(ms->s_dvb_provider);
   free(ms->s_dvb_cridauth);
   free(ms->s_dvb_charset);
+}
+
+void
+mpegts_service_delete ( service_t *t, int delconf )
+{
+  mpegts_service_t *ms = (mpegts_service_t*)t, *mms;
+  mpegts_mux_t     *mm = t->s_type == STYPE_STD ? ms->s_dvb_mux : NULL;
+
+  if (mm)
+    idnode_changed(&mm->mm_id);
+
+  /* Free memory */
   if (t->s_type == STYPE_STD)
     LIST_REMOVE(ms, s_dvb_mux_link);
   sbuf_free(&ms->s_tsbuf);
 
   /* Remove master/slave linking */
-  LIST_SAFE_REMOVE(ms, s_masters_link);
-  LIST_SAFE_REMOVE(ms, s_slaves_link);
+  while (ms->s_masters.is_count > 0) {
+    mms = (mpegts_service_t *)ms->s_masters.is_array[0];
+    mms->s_unlink(mms, ms);
+  }
+  idnode_set_clear(&ms->s_masters);
 
   /* Remove PID lists */
   mpegts_pid_destroy(&ms->s_pids);
@@ -576,6 +730,35 @@ mpegts_service_delete ( service_t *t, int delconf )
   //       is done in service_destroy
 }
 
+static int
+mpegts_service_satip_source ( service_t *t )
+{
+  mpegts_service_t *ms = (mpegts_service_t*)t;
+  mpegts_network_t *mn = ms->s_dvb_mux ? ms->s_dvb_mux->mm_network : NULL;
+  return mn ? mn->mn_satip_source : -1;
+}
+
+static void
+mpegts_service_memoryinfo ( service_t *t, int64_t *size )
+{
+  mpegts_service_t *ms = (mpegts_service_t*)t;
+  *size += sizeof(*ms);
+  *size += tvh_strlen(ms->s_nicename);
+  *size += tvh_strlen(ms->s_dvb_svcname);
+  *size += tvh_strlen(ms->s_dvb_provider);
+  *size += tvh_strlen(ms->s_dvb_cridauth);
+  *size += tvh_strlen(ms->s_dvb_charset);
+}
+
+static int
+mpegts_service_unseen( service_t *t, const char *type, time_t before )
+{
+  mpegts_service_t *ms = (mpegts_service_t*)t;
+  int pat = type && strcasecmp(type, "pat") == 0;
+  if (pat && ms->s_auto != SERVICE_AUTO_PAT_MISSING) return 0;
+  return ms->s_dvb_last_seen < before;
+}
+
 /* **************************************************************************
  * Creation/Location
  * *************************************************************************/
@@ -590,6 +773,7 @@ mpegts_service_create0
 {
   int r;
   char buf[256];
+  time_t dispatch_clock = gclk();
 
   /* defaults for older version */
   s->s_dvb_created = dispatch_clock;
@@ -603,6 +787,9 @@ mpegts_service_create0
   if (!conf) {
     if (sid)     s->s_dvb_service_id = sid;
     if (pmt_pid) s->s_pmt_pid        = pmt_pid;
+  } else {
+    if (s->s_dvb_last_seen > gclk()) /* sanity check */
+      s->s_dvb_last_seen = gclk();
   }
   s->s_dvb_mux        = mm;
   if ((r = dvb_servicetype_lookup(s->s_dvb_servicetype)) != -1)
@@ -610,6 +797,7 @@ mpegts_service_create0
   LIST_INSERT_HEAD(&mm->mm_services, s, s_dvb_mux_link);
   
   s->s_delete         = mpegts_service_delete;
+  s->s_unref          = mpegts_service_unref;
   s->s_is_enabled     = mpegts_service_is_enabled;
   s->s_config_save    = mpegts_service_config_save;
   s->s_enlist         = mpegts_service_enlist;
@@ -623,13 +811,16 @@ mpegts_service_create0
   s->s_provider_name  = mpegts_service_provider_name;
   s->s_channel_icon   = mpegts_service_channel_icon;
   s->s_mapped         = mpegts_service_mapped;
+  s->s_satip_source   = mpegts_service_satip_source;
+  s->s_memoryinfo     = mpegts_service_memoryinfo;
+  s->s_unseen         = mpegts_service_unseen;
 
   pthread_mutex_lock(&s->s_stream_mutex);
   service_make_nicename((service_t*)s);
   pthread_mutex_unlock(&s->s_stream_mutex);
 
   mpegts_mux_nice_name(mm, buf, sizeof(buf));
-  tvhlog(LOG_DEBUG, "mpegts", "%s - add service %04X %s", buf, s->s_dvb_service_id, s->s_dvb_svcname);
+  tvhdebug(LS_MPEGTS, "%s - add service %04X %s", buf, s->s_dvb_service_id, s->s_dvb_svcname);
 
   /* Notification */
   idnode_notify_changed(&mm->mm_id);
@@ -655,6 +846,9 @@ mpegts_service_find
   /* Validate */
   lock_assert(&global_lock);
 
+  if (mm->mm_sid_filter > 0 && sid != mm->mm_sid_filter)
+    return NULL;
+
   /* Find existing service */
   LIST_FOREACH(s, &mm->mm_services, s_dvb_mux_link) {
     if (s->s_dvb_service_id == sid) {
@@ -663,8 +857,8 @@ mpegts_service_find
         if (save) *save = 1;
       }
       if (create) {
-        if ((save && *save) || s->s_dvb_last_seen + 3600 < dispatch_clock) {
-          s->s_dvb_last_seen = dispatch_clock;
+        if ((save && *save) || s->s_dvb_last_seen + 3600 < gclk()) {
+          s->s_dvb_last_seen = gclk();
           if (save) *save = 1;
         }
       }
@@ -675,7 +869,7 @@ mpegts_service_find
   /* Create */
   if (create) {
     s = mm->mm_network->mn_create_service(mm, sid, pmt_pid);
-    s->s_dvb_created = s->s_dvb_last_seen = dispatch_clock;
+    s->s_dvb_created = s->s_dvb_last_seen = gclk();
     if (save) *save = 1;
   }
 
@@ -715,7 +909,7 @@ const idclass_t mpegts_service_raw_class =
 {
   .ic_super      = &service_raw_class,
   .ic_class      = "mpegts_raw_service",
-  .ic_caption    = "MPEGTS Raw Service",
+  .ic_caption    = N_("MPEG-TS raw service"),
   .ic_properties = NULL
 };
 
@@ -755,12 +949,12 @@ mpegts_service_raw_update_pids(mpegts_service_t *t, mpegts_apids_t *pids)
       mpegts_input_close_pids(mi, mm, t, 1);
       for (i = 0; i < x->count; i++) {
         pi = &x->pids[i];
-        mpegts_input_open_pid(mi, mm, pi->pid, MPS_RAW, pi->weight, t);
+        mpegts_input_open_pid(mi, mm, pi->pid, MPS_RAW, pi->weight, t, 0);
       }
     } else {
       if (pids->all) {
         mpegts_input_close_pids(mi, mm, t, 1);
-        mpegts_input_open_pid(mi, mm, MPEGTS_FULLMUX_PID, MPS_RAW, MPS_WEIGHT_RAW, t);
+        mpegts_input_open_pid(mi, mm, MPEGTS_FULLMUX_PID, MPS_RAW, MPS_WEIGHT_RAW, t, 0);
       } else {
         mpegts_pid_compare(p, x, &add, &del);
         for (i = 0; i < del.count; i++) {
@@ -769,7 +963,7 @@ mpegts_service_raw_update_pids(mpegts_service_t *t, mpegts_apids_t *pids)
         }
         for (i = 0; i < add.count; i++) {
           pi = &add.pids[i];
-          mpegts_input_open_pid(mi, mm, pi->pid, MPS_RAW, pi->weight, t);
+          mpegts_input_open_pid(mi, mm, pi->pid, MPS_RAW, pi->weight, t, 0);
         }
         mpegts_pid_done(&add);
         mpegts_pid_done(&del);
@@ -791,23 +985,71 @@ mpegts_service_raw_update_pids(mpegts_service_t *t, mpegts_apids_t *pids)
   return 0;
 }
 
+void
+mpegts_service_update_slave_pids ( mpegts_service_t *s, int del )
+{
+  mpegts_service_t *s2;
+  mpegts_apids_t *pids;
+  elementary_stream_t *st;
+  int i;
+
+  lock_assert(&s->s_stream_mutex);
+
+  if (s->s_pmt_pid == SERVICE_PMT_AUTO)
+    return;
+
+  pids = mpegts_pid_alloc();
+
+  mpegts_pid_add(pids, s->s_pmt_pid, MPS_WEIGHT_PMT);
+  mpegts_pid_add(pids, s->s_pcr_pid, MPS_WEIGHT_PCR);
+
+  /* Ensure that filtered PIDs are not send in ts_recv_raw */
+  TAILQ_FOREACH(st, &s->s_filt_components, es_filt_link)
+    if ((s->s_scrambled_pass || st->es_type != SCT_CA) &&
+        st->es_pid >= 0 && st->es_pid < 8192)
+      mpegts_pid_add(pids, st->es_pid, mpegts_mps_weight(st));
+
+  for (i = 0; i < s->s_masters.is_count; i++) {
+    s2 = (mpegts_service_t *)s->s_masters.is_array[i];
+    pthread_mutex_lock(&s2->s_stream_mutex);
+    if (!del)
+      mpegts_pid_add_group(s2->s_slaves_pids, pids);
+    else
+      mpegts_pid_del_group(s2->s_slaves_pids, pids);
+    pthread_mutex_unlock(&s2->s_stream_mutex);
+  }
+
+  mpegts_pid_destroy(&pids);
+}
+
 static int
 mpegts_service_link ( mpegts_service_t *master, mpegts_service_t *slave )
 {
+  pthread_mutex_lock(&slave->s_stream_mutex);
   pthread_mutex_lock(&master->s_stream_mutex);
-  LIST_INSERT_HEAD(&slave->s_masters, master, s_masters_link);
+  assert(!idnode_set_exists(&slave->s_masters, &master->s_id));
+  assert(LIST_SAFE_ENTRY(slave, s_slaves_link));
+  idnode_set_alloc(&slave->s_masters, 16);
+  idnode_set_add(&slave->s_masters, &master->s_id, NULL, NULL);
   LIST_INSERT_HEAD(&master->s_slaves, slave, s_slaves_link);
   pthread_mutex_unlock(&master->s_stream_mutex);
+  mpegts_service_update_slave_pids(slave, 0);
+  pthread_mutex_unlock(&slave->s_stream_mutex);
   return 0;
 }
 
 static int
 mpegts_service_unlink ( mpegts_service_t *master, mpegts_service_t *slave )
 {
+  pthread_mutex_lock(&slave->s_stream_mutex);
+  mpegts_service_update_slave_pids(slave, 1);
   pthread_mutex_lock(&master->s_stream_mutex);
-  LIST_SAFE_REMOVE(master, s_masters_link);
+  idnode_set_remove(&slave->s_masters, &master->s_id);
+  if (slave->s_masters.is_count == 0)
+    idnode_set_clear(&slave->s_masters);
   LIST_SAFE_REMOVE(slave, s_slaves_link);
   pthread_mutex_unlock(&master->s_stream_mutex);
+  pthread_mutex_unlock(&slave->s_stream_mutex);
   return 0;
 }
 
@@ -833,7 +1075,7 @@ mpegts_service_create_raw ( mpegts_mux_t *mm )
   s->s_delete         = mpegts_service_delete;
   s->s_is_enabled     = mpegts_service_is_enabled;
   s->s_config_save    = mpegts_service_config_save;
-  s->s_enlist         = mpegts_service_enlist;
+  s->s_enlist         = mpegts_service_enlist_raw;
   s->s_start_feed     = mpegts_service_start;
   s->s_stop_feed      = mpegts_service_stop;
   s->s_refresh_feed   = mpegts_service_refresh;
@@ -847,13 +1089,15 @@ mpegts_service_create_raw ( mpegts_mux_t *mm )
   s->s_update_pids    = mpegts_service_raw_update_pids;
   s->s_link           = mpegts_service_link;
   s->s_unlink         = mpegts_service_unlink;
+  s->s_satip_source   = mpegts_service_satip_source;
+  s->s_memoryinfo     = mpegts_service_memoryinfo;
 
   pthread_mutex_lock(&s->s_stream_mutex);
   free(s->s_nicename);
   s->s_nicename = strdup(buf);
   pthread_mutex_unlock(&s->s_stream_mutex);
 
-  tvhlog(LOG_DEBUG, "mpegts", "%s - add raw service", buf);
+  tvhdebug(LS_MPEGTS, "%s - add raw service", buf);
 
   return s;
 }
