@@ -45,14 +45,14 @@ static void
 satip_device_dbus_notify( satip_device_t *sd, const char *sig_name )
 {
 #if ENABLE_DBUS_1
-  char buf[256];
+  char buf[256], ubuf[UUID_HEX_SIZE];
 
   htsmsg_t *msg = htsmsg_create_list();
   htsmsg_add_str(msg, NULL, sd->sd_info.addr);
   htsmsg_add_str(msg, NULL, sd->sd_info.location);
   htsmsg_add_str(msg, NULL, sd->sd_info.server);
   htsmsg_add_s64(msg, NULL, sd->sd_info.rtsp_port);
-  snprintf(buf, sizeof(buf), "/input/mpegts/satip/%s", idnode_uuid_as_str(&sd->th_id));
+  snprintf(buf, sizeof(buf), "/input/mpegts/satip/%s", idnode_uuid_as_str(&sd->th_id, ubuf));
   dbus_emit_signal(buf, sig_name, msg);
 #endif
 }
@@ -77,7 +77,7 @@ satip_device_block( const char *addr, int block )
         TAILQ_FOREACH(lfe, &sd->sd_frontends, sf_link)
           mpegts_input_stop_all((mpegts_input_t *)lfe);
       }
-      tvhinfo("satip", "address %s is %s", addr,
+      tvhinfo(LS_SATIP, "address %s is %s", addr,
               block < 0 ? "stopped" : (block > 0 ? "allowed" : "disabled"));
     }
   }
@@ -117,10 +117,43 @@ satip_device_nicename( satip_device_t *sd, char *buf, int len )
  * SAT-IP client
  */
 
-static void
-satip_device_class_save ( idnode_t *in )
+static htsmsg_t *
+satip_device_class_save ( idnode_t *in, char *filename, size_t fsize )
 {
-  satip_device_save((satip_device_t *)in);
+  satip_device_t *sd = (satip_device_t *)in;
+  satip_frontend_t *lfe;
+  htsmsg_t *m, *l;
+  char ubuf[UUID_HEX_SIZE];
+
+  if (sd->sd_nosave)
+    return NULL;
+
+  m = htsmsg_create_map();
+  idnode_save(&sd->th_id, m);
+
+  l = htsmsg_create_map();
+  TAILQ_FOREACH(lfe, &sd->sd_frontends, sf_link)
+    satip_frontend_save(lfe, l);
+  htsmsg_add_msg(m, "frontends", l);
+
+  snprintf(filename, fsize, "input/satip/adapters/%s",
+           idnode_uuid_as_str(&sd->th_id, ubuf));
+  return m;
+}
+
+static const void *
+satip_device_class_active_get ( void * obj )
+{
+  static int active;
+  satip_device_t *sd = (satip_device_t *)obj;
+  satip_frontend_t *lfe;
+  active = 0;
+  TAILQ_FOREACH(lfe, &sd->sd_frontends, sf_link)
+    if (*(int *)mpegts_input_class_active_get(lfe)) {
+      active = 1;
+      break;
+    }
+  return &active;
 }
 
 static idnode_set_t *
@@ -131,12 +164,12 @@ satip_device_class_get_childs ( idnode_t *in )
   satip_frontend_t *lfe;
 
   TAILQ_FOREACH(lfe, &sd->sd_frontends, sf_link)
-    idnode_set_add(is, &lfe->ti_id, NULL);
+    idnode_set_add(is, &lfe->ti_id, NULL, NULL);
   return is;
 }
 
 static const char *
-satip_device_class_get_title( idnode_t *in )
+satip_device_class_get_title( idnode_t *in, const char *lang )
 {
   static char buf[256];
   satip_device_t *sd = (satip_device_t *)in;
@@ -170,7 +203,7 @@ static const char *satip_tunercfg_tab[] = {
 };
 
 static htsmsg_t *
-satip_device_class_tunercfg_list ( void *o )
+satip_device_class_tunercfg_list ( void *o, const char *lang )
 {
   htsmsg_t *l = htsmsg_create_list();
   const char **p;
@@ -181,225 +214,302 @@ satip_device_class_tunercfg_list ( void *o )
 }
 
 static void
-satip_device_class_tunercfg_notify ( void *o )
+satip_device_class_tunercfg_notify ( void *o, const char *lang )
 {
   satip_device_t *sd = (satip_device_t *)o;
   if (!sd->sd_inload)
     satip_device_destroy_later(sd, 100);
 }
 
+CLASS_DOC(satip_client)
+
 const idclass_t satip_device_class =
 {
   .ic_class      = "satip_client",
   .ic_event      = "satip_client",
-  .ic_caption    = "SAT>IP Client",
+  .ic_caption    = N_("SAT>IP Client"),
+  .ic_doc        = tvh_doc_satip_client_class,
   .ic_save       = satip_device_class_save,
   .ic_get_childs = satip_device_class_get_childs,
   .ic_get_title  = satip_device_class_get_title,
   .ic_properties = (const property_t[]){
+    {
+      .type     = PT_BOOL,
+      .id       = "active",
+      .name     = N_("Active"),
+      .opts     = PO_RDONLY | PO_NOSAVE | PO_NOUI,
+      .get      = satip_device_class_active_get,
+    },
     {
       .type     = PT_STR,
       .id       = "tunercfgu",
-      .name     = "Tuner Configuration",
+      .name     = N_("Tuner configuration"),
+      .desc     = N_("Tuner configuration."),
       .opts     = PO_SORTKEY,
       .off      = offsetof(satip_device_t, sd_tunercfg),
       .list     = satip_device_class_tunercfg_list,
       .notify   = satip_device_class_tunercfg_notify,
       .def.s    = "Auto"
     },
+    {
+      .type     = PT_BOOL,
+      .id       = "tcp_mode",
+      .name     = N_("RTP/AVP/TCP (embedded data)"),
+      .desc     = N_("Enable or disable RTP/AVP/TCP transfer mode "
+                     "(embedded data in the RTSP session) support."),
+      .opts     = PO_ADVANCED,
+      .off      = offsetof(satip_device_t, sd_tcp_mode),
+    },
+    {
+      .type     = PT_BOOL,
+      .id       = "fast_switch",
+      .name     = N_("Fast input switch"),
+      .desc     = N_("Enable or disable fast input switching."),
+      .opts     = PO_ADVANCED,
+      .off      = offsetof(satip_device_t, sd_fast_switch),
+    },
     {
       .type     = PT_BOOL,
       .id       = "fullmux_ok",
-      .name     = "Full Mux Rx mode supported",
+      .name     = N_("Full mux RX mode supported"),
+      .desc     = N_("Enable or disable full mux mode."),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_fullmux_ok),
     },
     {
       .type     = PT_INT,
       .id       = "sigscale",
-      .name     = "Signal scale (240 or 100)",
+      .name     = N_("Signal scale (240 or 100)"),
+      .desc     = N_("Not all SAT>IP servers use the same signal scaling. "
+                     "Change this setting if the signal level displayed "
+                     "within Tvheadend looks too low."),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_sig_scale),
     },
     {
       .type     = PT_INT,
       .id       = "pids_max",
-      .name     = "Maximum PIDs",
+      .name     = N_("Maximum PIDs"),
+      .desc     = N_("Set the maxiumum packet identifiers your SAT>IP "
+                     "server supports."),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_pids_max),
     },
     {
       .type     = PT_INT,
       .id       = "pids_len",
-      .name     = "Maximum length of PIDs",
+      .name     = N_("Maximum length of PIDs"),
+      .desc     = N_("Maximum length in characters for the command "
+                     "setting PIDs to the SAT>IP box."),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_pids_len),
     },
     {
       .type     = PT_BOOL,
       .id       = "pids_deladd",
-      .name     = "addpids/delpids supported",
+      .name     = N_("addpids/delpids supported"),
+      .desc     = N_("Enable if the SAT>IP box supports the "
+                     "addpids/delpids commands."),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_pids_deladd),
     },
     {
       .type     = PT_BOOL,
-      .id       = "pids0",
-      .name     = "PIDs in setup",
+      .id       = "fe",
+      .name     = N_("FE supported"),
+      .desc     = N_("Enable if the SAT>IP box supports the frontend "
+                     "identifier. This allows the auto-tuner allocation, "
+                     "but it might cause trouble for boxes with different "
+                     "tuner reception connections like satellite inputs."),
       .opts     = PO_ADVANCED,
-      .off      = offsetof(satip_device_t, sd_pids0),
+      .off      = offsetof(satip_device_t, sd_fe),
     },
     {
       .type     = PT_BOOL,
       .id       = "piloton",
-      .name     = "Force pilot for DVB-S2",
+      .name     = N_("Force pilot for DVB-S2"),
+      .desc     = N_("Enable if the SAT>IP box requests plts=on "
+                     "parameter in the SETUP RTSP command for DVB-S2 "
+                     "muxes."),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_pilot_on),
     },
     {
       .type     = PT_BOOL,
       .id       = "pids21",
-      .name     = "PIDs 21 in setup",
+      .name     = N_("PIDs 21 in setup"),
+      .desc     = N_("Enable if the SAT>IP box requires pids=21 "
+                     "parameter in the SETUP RTSP command"),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_pids21),
     },
     {
       .type     = PT_STR,
       .id       = "bindaddr",
-      .name     = "Local bind IP address",
+      .name     = N_("Local bind IP address"),
+      .desc     = N_("Bind to specific local IP address."),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_bindaddr),
     },
+    {
+      .type     = PT_INT,
+      .id       = "skip_ts",
+      .name     = N_("Skip TS packets (0-200)"),
+      .desc     = N_("Skip x number of transport packets."),
+      .opts     = PO_EXPERT,
+      .off      = offsetof(satip_device_t, sd_skip_ts),
+    },
     {
       .type     = PT_BOOL,
       .id       = "disableworkarounds",
-      .name     = "Disable device-/firmware-specific workarounds",
+      .name     = N_("Disable device/firmware-specific workarounds"),
       .opts     = PO_ADVANCED,
       .off      = offsetof(satip_device_t, sd_disable_workarounds),
     },
     {
       .type     = PT_STR,
       .id       = "addr",
-      .name     = "IP Address",
+      .name     = N_("IP address"),
+      .desc     = N_("Force all network connections to this tuner to be "
+                     "made over the specified IP address, similar to "
+                     "the setting for the SAT>IP device itself. Setting "
+                     "this overrides the device-specific setting."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.addr),
     },
     {
       .type     = PT_INT,
       .id       = "rtsp",
-      .name     = "RTSP Port",
+      .name     = N_("RTSP port"),
+      .desc     = N_("Current RTSP port."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.rtsp_port),
     },
     {
       .type     = PT_STR,
       .id       = "device_uuid",
-      .name     = "UUID",
+      .name     = N_("UUID"),
+      .desc     = N_("The SAT>IP server's universally unique identifier."),
       .opts     = PO_RDONLY,
       .off      = offsetof(satip_device_t, sd_info.uuid),
     },
     {
       .type     = PT_STR,
       .id       = "friendly",
-      .name     = "Friendly Name",
+      .name     = N_("Friendly name"),
+      .desc     = N_("The SAT>IP server's name."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.friendlyname),
     },
     {
       .type     = PT_STR,
       .id       = "serialnum",
-      .name     = "Serial Number",
+      .name     = N_("Serial number"),
+      .desc     = N_("The device's serial number."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.serialnum),
     },
     {
       .type     = PT_STR,
       .id       = "tunercfg",
-      .name     = "Tuner Configuration",
+      .name     = N_("Tuner configuration"),
+      .desc     = N_("Current tuner configuration."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.tunercfg),
     },
     {
       .type     = PT_STR,
       .id       = "manufacturer",
-      .name     = "Manufacturer",
+      .name     = N_("Manufacturer"),
+      .desc     = N_("The manufacturer of the SAT>IP server."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.manufacturer),
     },
     {
       .type     = PT_STR,
       .id       = "manufurl",
-      .name     = "Manufacturer URL",
+      .name     = N_("Manufacturer URL"),
+      .desc     = N_("Manufacturer's product information page for the device."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.manufacturerURL),
     },
     {
       .type     = PT_STR,
       .id       = "modeldesc",
-      .name     = "Model Description",
+      .name     = N_("Model description"),
+      .desc     = N_("Manufacturer's product description."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.modeldesc),
     },
     {
       .type     = PT_STR,
       .id       = "modelname",
-      .name     = "Model Name",
+      .name     = N_("Model name"),
+      .desc     = N_("Manufacturer's product name."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.modelname),
     },
     {
       .type     = PT_STR,
       .id       = "modelnum",
-      .name     = "Model Number",
+      .name     = N_("Model number"),
+      .desc     = N_("Manufacturer's model number."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.modelnum),
     },
     {
       .type     = PT_STR,
       .id       = "bootid",
-      .name     = "Boot ID",
+      .name     = N_("Boot ID"),
+      .desc     = N_("The current boot ID."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.bootid),
     },
     {
       .type     = PT_STR,
       .id       = "configid",
-      .name     = "Config ID",
+      .name     = N_("Configuration ID"),
+      .desc     = N_("The current configuration ID."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.configid),
     },
     {
       .type     = PT_STR,
       .id       = "deviceid",
-      .name     = "Device ID",
+      .name     = N_("Device ID"),
+      .desc     = N_("The device ID."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.deviceid),
     },
     {
       .type     = PT_STR,
       .id       = "presentation",
-      .name     = "Presentation",
+      .name     = N_("Presentation"),
+      .desc     = N_("Presentation details."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.presentation),
     },
     {
       .type     = PT_STR,
       .id       = "location",
-      .name     = "Location",
+      .name     = N_("Location"),
+      .desc     = N_("Location details of the SAT>IP Server."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.location),
     },
     {
       .type     = PT_STR,
       .id       = "server",
-      .name     = "Server",
+      .name     = N_("Server"),
+      .desc     = N_("Server details."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.server),
     },
     {
       .type     = PT_STR,
       .id       = "myaddr",
-      .name     = "Local Discovery IP Address",
+      .name     = N_("Local discovery IP address"),
+      .desc     = N_("The SAT>IP's discovered IP address."),
       .opts     = PO_RDONLY | PO_NOSAVE,
       .off      = offsetof(satip_device_t, sd_info.myaddr),
     },
@@ -425,7 +535,7 @@ satip_device_calc_uuid( tvh_uuid_t *uuid, const char *satip_uuid )
 {
   uint8_t uuidbin[20];
 
-  satip_device_calc_bin_uuid(uuidbin, satip_uuid);
+  sha1_calc(uuidbin, (const uint8_t *)satip_uuid, strlen(satip_uuid), NULL, 0);
   bin2hex(uuid->hex, sizeof(uuid->hex), uuidbin, sizeof(uuidbin));
 }
 
@@ -445,26 +555,30 @@ satip_device_hack( satip_device_t *sd )
     sd->sd_fullmux_ok  = 0;
     sd->sd_pids_max    = 32;
     sd->sd_pids_deladd = 0;
-    tvhwarn("satip", "Detected old Inverto firmware V1.13.0.105 and less");
-    tvhwarn("satip", "Upgrade to V1.16.0.120 - http://http://www.inverto.tv/support/ - IDL400s");
+    tvhwarn(LS_SATIP, "Detected old Inverto firmware V1.13.0.105 and less");
+    tvhwarn(LS_SATIP, "Upgrade to V1.16.0.120 - http://http://www.inverto.tv/support/ - IDL400s");
   } else
 #endif
   if (strstr(sd->sd_info.location, ":8888/octonet.xml")) {
     /* OctopusNet requires pids in the SETUP RTSP command */
-    sd->sd_pids0       = 1;
   } else if (strstr(sd->sd_info.manufacturer, "Triax") &&
              strstr(sd->sd_info.modelname, "TSS400")) {
+    sd->sd_fullmux_ok  = 0;
+    sd->sd_pids_max    = 64;
     sd->sd_pilot_on    = 1;
   } else if (strcmp(sd->sd_info.modelname, "TVHeadend SAT>IP") == 0)  {
     sd->sd_pids_max    = 128;
     sd->sd_pids_len    = 2048;
     sd->sd_no_univ_lnb = 1;
+    if (strcmp(sd->sd_info.modelnum ?: "", "1.0"))
+      sd->sd_can_weight  = 1;
   } else if (strstr(sd->sd_info.manufacturer, "AVM Berlin") &&
              strstr(sd->sd_info.modelname, "FRITZ!")) {
     sd->sd_fullmux_ok  = 0;
     sd->sd_pids_deladd = 0;
-    sd->sd_pids0       = 1;
-    sd->sd_pids21       = 1;
+    sd->sd_pids21      = 1;
+  } else if (strstr(sd->sd_info.modelname, "EyeTV Netstream 4C")) {
+    sd->sd_fe  = 0;
   }
 }
 
@@ -486,14 +600,15 @@ satip_device_create( satip_device_info_t *info )
   conf = hts_settings_load("input/satip/adapters/%s", uuid.hex);
 
   /* some sane defaults */
+  sd->sd_fast_switch = 1;
   sd->sd_fullmux_ok  = 1;
   sd->sd_pids_len    = 127;
   sd->sd_pids_max    = 32;
   sd->sd_pids_deladd = 1;
+  sd->sd_fe          = 1;
   sd->sd_sig_scale   = 240;
   sd->sd_dbus_allow  = 1;
 
-
   if (!tvh_hardware_create0((tvh_hardware_t*)sd, &satip_device_class,
                             uuid.hex, conf)) {
     /* Note: sd is freed in above fcn */
@@ -503,6 +618,9 @@ satip_device_create( satip_device_info_t *info )
   pthread_mutex_init(&sd->sd_tune_mutex, NULL);
 
   TAILQ_INIT(&sd->sd_frontends);
+  TAILQ_INIT(&sd->sd_serialize_queue);
+
+  atomic_set(&sd->sd_wake_ref, 0);
 
   /* we may check if uuid matches, but the SHA hash should be enough */
   if (sd->sd_info.uuid)
@@ -570,19 +688,19 @@ satip_device_create( satip_device_info_t *info )
     } else if (strncmp(argv[i], "DVBC-", 5) == 0) {
       type = DVB_TYPE_C;
       m = atoi(argv[i] + 5);
-    } else if (strncmp(argv[i], "ATSC-", 5) == 0) {
-      type = DVB_TYPE_ATSC;
-      m = atoi(argv[i] + 5);
-    } else if (strncmp(argv[i], "DVBCB-", 6) == 0) {
+    } else if (strncmp(argv[i], "ATSCT-", 6) == 0) {
+      type = DVB_TYPE_ATSC_T;
+      m = atoi(argv[i] + 6);
+    } else if (strncmp(argv[i], "ATSCC-", 6) == 0) {
+      type = DVB_TYPE_ATSC_C;
       m = atoi(argv[i] + 6);
-      v2 = 2;
     }
     if (type == DVB_TYPE_NONE) {
-      tvhlog(LOG_ERR, "satip", "%s: bad tuner type [%s]",
-             satip_device_nicename(sd, buf2, sizeof(buf2)), argv[i]);
+      tvherror(LS_SATIP, "%s: bad tuner type [%s]",
+               satip_device_nicename(sd, buf2, sizeof(buf2)), argv[i]);
     } else if (m < 0 || m > 32) {
-      tvhlog(LOG_ERR, "satip", "%s: bad tuner count [%s]",
-             satip_device_nicename(sd, buf2, sizeof(buf2)), argv[i]);
+      tvherror(LS_SATIP, "%s: bad tuner count [%s]",
+               satip_device_nicename(sd, buf2, sizeof(buf2)), argv[i]);
     } else {
       sd->sd_nosave = 1;
       for (j = 0; j < m; j++)
@@ -593,7 +711,7 @@ satip_device_create( satip_device_info_t *info )
   }
 
   if (save)
-    satip_device_save(sd);
+    satip_device_changed(sd);
 
   sd->sd_inload = 0;
 
@@ -613,7 +731,7 @@ satip_device_find( const char *satip_uuid )
   satip_device_calc_bin_uuid(binuuid, satip_uuid);
   TVH_HARDWARE_FOREACH(th) {
     if (idnode_is_instance(&th->th_id, &satip_device_class) &&
-        memcmp(th->th_id.in_uuid, binuuid, UUID_BIN_SIZE) == 0)
+        memcmp(th->th_id.in_uuid.bin, binuuid, UUID_BIN_SIZE) == 0)
       return (satip_device_t *)th;
   }
   return NULL;
@@ -632,28 +750,6 @@ satip_device_find_by_descurl( const char *descurl )
   return NULL;
 }
 
-void
-satip_device_save( satip_device_t *sd )
-{
-  satip_frontend_t *lfe;
-  htsmsg_t *m, *l;
-
-  if (sd->sd_nosave)
-    return;
-
-  m = htsmsg_create_map();
-  idnode_save(&sd->th_id, m);
-
-  l = htsmsg_create_map();
-  TAILQ_FOREACH(lfe, &sd->sd_frontends, sf_link)
-    satip_frontend_save(lfe, l);
-  htsmsg_add_msg(m, "frontends", l);
-
-  hts_settings_save(m, "input/satip/adapters/%s",
-                    idnode_uuid_as_str(&sd->th_id));
-  htsmsg_destroy(m);
-}
-
 void
 satip_device_destroy( satip_device_t *sd )
 {
@@ -661,7 +757,9 @@ satip_device_destroy( satip_device_t *sd )
 
   lock_assert(&global_lock);
 
-  gtimer_disarm(&sd->sd_destroy_timer);
+  mtimer_disarm(&sd->sd_destroy_timer);
+
+  idnode_save_check(&sd->th_id, 0);
 
   while ((lfe = TAILQ_FIRST(&sd->sd_frontends)) != NULL)
     satip_frontend_delete(lfe);
@@ -704,7 +802,7 @@ satip_device_destroy_cb( void *aux )
 void
 satip_device_destroy_later( satip_device_t *sd, int after )
 {
-  gtimer_arm_ms(&sd->sd_destroy_timer, satip_device_destroy_cb, sd, after);
+  mtimer_arm_rel(&sd->sd_destroy_timer, satip_device_destroy_cb, sd, ms2mono(after));
 }
 
 /*
@@ -722,18 +820,19 @@ typedef struct satip_discovery {
   char *deviceid;
   url_t url;
   http_client_t *http_client;
-  time_t http_start;
+  int64_t http_start;
 } satip_discovery_t;
 
 TAILQ_HEAD(satip_discovery_queue, satip_discovery);
 
+static int satip_enabled;
 static int satip_discoveries_count;
 static struct satip_discovery_queue satip_discoveries;
 static upnp_service_t *satip_discovery_service;
-static gtimer_t satip_discovery_timer;
-static gtimer_t satip_discovery_static_timer;
-static gtimer_t satip_discovery_timerq;
-static gtimer_t satip_discovery_msearch_timer;
+static mtimer_t satip_discovery_timer;
+static mtimer_t satip_discovery_static_timer;
+static mtimer_t satip_discovery_timerq;
+static mtimer_t satip_discovery_msearch_timer;
 static str_list_t *satip_static_clients;
 
 static void
@@ -742,7 +841,7 @@ satip_discovery_destroy(satip_discovery_t *d, int unlink)
   if (d == NULL)
     return;
   if (unlink) {
-    satip_discoveries_count--;
+    atomic_dec(&satip_discoveries_count, 1);
     TAILQ_REMOVE(&satip_discoveries, d, disc_link);
   }
   if (d->http_client)
@@ -794,14 +893,14 @@ satip_discovery_http_closed(http_client_t *hc, int errn)
   }
   if (errn != 0 || s == NULL || hc->hc_code != 200 ||
       hc->hc_data_size == 0 || hc->hc_data == NULL) {
-    tvhlog(LOG_ERR, "satip", "Cannot get %s: %s", d->location, strerror(errn));
+    tvherror(LS_SATIP, "Cannot get %s: %s", d->location, strerror(errn));
     return;
   }
 
-#if ENABLE_TRACE
-  tvhtrace("satip", "received XML description from %s", hc->hc_host);
-  tvhlog_hexdump("satip", hc->hc_data, hc->hc_data_size);
-#endif
+  if (tvhtrace_enabled()) {
+    tvhtrace(LS_SATIP, "received XML description from %s", hc->hc_host);
+    tvhlog_hexdump(LS_SATIP, hc->hc_data, hc->hc_data_size);
+  }
 
   if (d->myaddr == NULL || d->myaddr[0] == '\0') {
     struct sockaddr_storage ip;
@@ -824,7 +923,7 @@ satip_discovery_http_closed(http_client_t *hc, int errn)
   xml = htsmsg_xml_deserialize(hc->hc_data, errbuf, sizeof(errbuf));
   hc->hc_data = NULL;
   if (!xml) {
-    tvhlog(LOG_ERR, "satip_discovery_desc", "htsmsg_xml_deserialize error %s", errbuf);
+    tvherror(LS_SATIP, "satip_discovery_desc htsmsg_xml_deserialize error %s", errbuf);
     goto finish;
   }
   if ((tags         = htsmsg_get_map(xml, "tags")) == NULL)
@@ -844,7 +943,7 @@ satip_discovery_http_closed(http_client_t *hc, int errn)
   if ((friendlyname = htsmsg_xml_get_cdata_str(device, "friendlyName")) == NULL)
     goto finish;
   if ((manufacturer = htsmsg_xml_get_cdata_str(device, "manufacturer")) == NULL)
-    goto finish;
+    manufacturer = "";
   if ((manufacturerURL = htsmsg_xml_get_cdata_str(device, "manufacturerURL")) == NULL)
     manufacturerURL = "";
   if ((modeldesc    = htsmsg_xml_get_cdata_str(device, "modelDescription")) == NULL)
@@ -945,7 +1044,7 @@ satip_discovery_timerq_cb(void *aux)
     d = next;
     next = TAILQ_NEXT(d, disc_link);
     if (d->http_client) {
-      if (dispatch_clock - d->http_start > 4)
+      if (mclk() - d->http_start > sec2mono(4))
         satip_discovery_destroy(d, 1);
       continue;
     }
@@ -955,7 +1054,7 @@ satip_discovery_timerq_cb(void *aux)
     if (d->http_client == NULL)
       satip_discovery_destroy(d, 1);
     else {
-      d->http_start = dispatch_clock;
+      d->http_start = mclk();
       d->http_client->hc_conn_closed = satip_discovery_http_closed;
       http_client_register(d->http_client);
       r = http_client_simple(d->http_client, &d->url);
@@ -964,7 +1063,7 @@ satip_discovery_timerq_cb(void *aux)
     }
   }
   if (TAILQ_FIRST(&satip_discoveries))
-    gtimer_arm(&satip_discovery_timerq, satip_discovery_timerq_cb, NULL, 5);
+    mtimer_arm_rel(&satip_discovery_timerq, satip_discovery_timerq_cb, NULL, sec2mono(5));
 }
 
 static void
@@ -985,7 +1084,7 @@ satip_discovery_service_received
   satip_discovery_t *d;
   int n, i;
 
-  if (len > 8191 || satip_discoveries_count > 100)
+  if (len > 8191 || atomic_get(&satip_discoveries_count) > 100)
     return;
   buf = alloca(len+1);
   memcpy(buf, data, len);
@@ -1045,7 +1144,7 @@ satip_discovery_service_received
     goto add_uuid;
   if (location == NULL || strncmp(location, "http://", 7))
     goto add_uuid;
-  if (bootid == NULL || configid == NULL || server == NULL)
+  if (server == NULL)
     goto add_uuid;
 
   /* Forward information to next layer */
@@ -1060,9 +1159,9 @@ satip_discovery_service_received
   d->location = strdup(location);
   d->server   = strdup(server);
   d->uuid     = strdup(uuid);
-  d->bootid   = strdup(bootid);
-  d->configid = strdup(configid);
-  d->deviceid = strdup(deviceid ? deviceid : "");
+  d->bootid   = strdup(bootid ?: "");
+  d->configid = strdup(configid ?: "");
+  d->deviceid = strdup(deviceid ?: "");
   if (urlparse(d->location, &d->url)) {
     satip_discovery_destroy(d, 0);
     return;
@@ -1072,8 +1171,8 @@ satip_discovery_service_received
   i = 1;
   if (!satip_discovery_find(d) && !satip_device_find(d->uuid)) {
     TAILQ_INSERT_TAIL(&satip_discoveries, d, disc_link);
-    satip_discoveries_count++;
-    gtimer_arm_ms(&satip_discovery_timerq, satip_discovery_timerq_cb, NULL, 250);
+    atomic_add(&satip_discoveries_count, 1);
+    mtimer_arm_rel(&satip_discovery_timerq, satip_discovery_timerq_cb, NULL, ms2mono(250));
     i = 0;
   }
   pthread_mutex_unlock(&global_lock);
@@ -1087,7 +1186,7 @@ add_uuid:
   /* if new uuid was discovered, retrigger MSEARCH */
   pthread_mutex_lock(&global_lock);
   if (!satip_device_find(uuid))
-    gtimer_arm(&satip_discovery_timer, satip_discovery_timer_cb, NULL, 5);
+    mtimer_arm_rel(&satip_discovery_timer, satip_discovery_timer_cb, NULL, sec2mono(5));
   pthread_mutex_unlock(&global_lock);
 }
 
@@ -1101,6 +1200,7 @@ satip_discovery_static(const char *descurl)
   if (satip_device_find_by_descurl(descurl))
     return;
   d = calloc(1, sizeof(satip_discovery_t));
+  urlinit(&d->url);
   if (urlparse(descurl, &d->url)) {
     satip_discovery_destroy(d, 0);
     return;
@@ -1113,7 +1213,7 @@ satip_discovery_static(const char *descurl)
   d->configid = strdup("");
   d->deviceid = strdup("");
   TAILQ_INSERT_TAIL(&satip_discoveries, d, disc_link);
-  satip_discoveries_count++;
+  atomic_add(&satip_discoveries_count, 1);
   satip_discovery_timerq_cb(NULL);
 }
 
@@ -1145,33 +1245,40 @@ ST: urn:ses-com:device:SatIPServer:1\r\n"
   htsbuf_append(&q, MSG, sizeof(MSG)-1);
   htsbuf_qprintf(&q, "USER-AGENT: unix/1.0 UPnP/1.1 TVHeadend/%s\r\n", tvheadend_version);
   htsbuf_append(&q, "\r\n", 2);
-  upnp_send(&q, NULL, 0);
+  upnp_send(&q, NULL, 0, 0);
   htsbuf_queue_flush(&q);
 
-  gtimer_arm_ms(&satip_discovery_msearch_timer, satip_discovery_send_msearch,
-                (void *)(intptr_t)(attempt + 1), attempt * 11);
+  mtimer_arm_rel(&satip_discovery_msearch_timer, satip_discovery_send_msearch,
+                 (void *)(intptr_t)(attempt + 1), ms2mono(attempt * 11));
 #undef MSG
 }
 
 static void
 satip_discovery_static_timer_cb(void *aux)
 {
+  static int next_timeout = 10;
   int i;
 
-  if (!tvheadend_running)
+  if (!tvheadend_is_running())
     return;
   for (i = 0; i < satip_static_clients->num; i++)
     satip_discovery_static(satip_static_clients->str[i]);
-  gtimer_arm(&satip_discovery_static_timer, satip_discovery_static_timer_cb, NULL, 3600);
+  mtimer_arm_rel(&satip_discovery_static_timer, satip_discovery_static_timer_cb,
+                 NULL, sec2mono(next_timeout));
+  if (next_timeout < 3600)
+    next_timeout = next_timeout >= 30 ? 3600 : 30;
 }
 
 static void
 satip_discovery_timer_cb(void *aux)
 {
-  if (!tvheadend_running)
+  static int next_timeout = 10;
+
+  if (!tvheadend_is_running())
     return;
-  if (!upnp_running) {
-    gtimer_arm(&satip_discovery_timer, satip_discovery_timer_cb, NULL, 1);
+  if (!atomic_get(&upnp_running)) {
+    mtimer_arm_rel(&satip_discovery_timer, satip_discovery_timer_cb,
+                   NULL, sec2mono(1));
     return;
   }
   if (satip_discovery_service == NULL) {
@@ -1183,26 +1290,47 @@ satip_discovery_timer_cb(void *aux)
   }
   if (satip_discovery_service)
     satip_discovery_send_msearch((void *)1);
-  gtimer_arm(&satip_discovery_timer, satip_discovery_timer_cb, NULL, 3600);
+  mtimer_arm_rel(&satip_discovery_timer, satip_discovery_timer_cb,
+                 NULL, sec2mono(next_timeout));
+  if (next_timeout < 3600)
+    next_timeout = next_timeout >= 30 ? 3600 : 30;
 }
 
 void
 satip_device_discovery_start( void )
 {
-  gtimer_arm(&satip_discovery_timer, satip_discovery_timer_cb, NULL, 1);
-  gtimer_arm(&satip_discovery_static_timer, satip_discovery_static_timer_cb, NULL, 1);
+  if (!satip_enabled)
+    return;
+  mtimer_arm_rel(&satip_discovery_timer, satip_discovery_timer_cb,
+                 NULL, sec2mono(1));
+  mtimer_arm_rel(&satip_discovery_static_timer, satip_discovery_static_timer_cb,
+                 NULL, sec2mono(1));
 }
 
 /*
  * Initialization
  */
 
-void satip_init ( str_list_t *clients )
+void satip_init ( int nosatip, str_list_t *clients )
 {
+  idclass_register(&satip_device_class);
+
+  idclass_register(&satip_frontend_class);
+  idclass_register(&satip_frontend_dvbt_class);
+  idclass_register(&satip_frontend_dvbs_class);
+  idclass_register(&satip_frontend_dvbs_slave_class);
+  idclass_register(&satip_frontend_atsc_t_class);
+  idclass_register(&satip_frontend_atsc_c_class);
+
+  idclass_register(&satip_satconf_class);
+
+  satip_enabled = !nosatip;
   TAILQ_INIT(&satip_discoveries);
   satip_static_clients = clients;
-  dbus_register_rpc_str("satip_addr", NULL, satip_device_addr);
-  satip_device_discovery_start();
+  if (satip_enabled) {
+    dbus_register_rpc_str("satip_addr", NULL, satip_device_addr);
+    satip_device_discovery_start();
+  }
 }
 
 void satip_done ( void )
