@@ -21,11 +21,16 @@
 
 int ts_resync ( const uint8_t *tsb, int *len, int *idx );
 
+void ts_recv_packet0
+  (struct mpegts_service *t, elementary_stream_t *st, const uint8_t *tsb, int len);
+
 int ts_recv_packet1
-  (struct mpegts_service *t, const uint8_t *tsb, int len, int64_t *pcrp, int table);
+  (struct mpegts_service *t, const uint8_t *tsb, int len, int table);
 
 void ts_recv_packet2(struct mpegts_service *t, const uint8_t *tsb, int len);
 
+void ts_skip_packet2(struct mpegts_service *t, const uint8_t *tsb, int len);
+
 void ts_recv_raw(struct mpegts_service *t, const uint8_t *tsb, int len);
 
 #endif /* TSDEMUX_H */
