@@ -20,12 +20,7 @@ static inline size_t
 tvh_iconv(iconv_t cd, char **inbuf, size_t *inbytesleft,
                       char **outbuf, size_t *outbytesleft)
 {
-#ifdef PLATFORM_FREEBSD
-  return iconv(cd, (const char **)inbuf, inbytesleft,
-                   (const char **)outbuf, outbytesleft);
-#else
   return iconv(cd, inbuf, inbytesleft, outbuf, outbytesleft);
-#endif
 }
 
 void
@@ -79,7 +74,7 @@ intlconv_charset_id( const char *charset,
                      int transil,
                      int ignore_bad_chars )
 {
-  static char __thread buf[128];
+  static __thread char buf[128];
   const char *delim;
 
   if (charset == NULL || charset[0] == '\0' ||
@@ -94,10 +89,8 @@ intlconv_charset_id( const char *charset,
     snprintf(buf, sizeof(buf), "%s%sTRANSLIT", charset, delim);
   else if (ignore_bad_chars)
     snprintf(buf, sizeof(buf), "%s%sIGNORE", charset, delim);
-  else {
-    strncpy(buf, charset, sizeof(buf));
-    buf[sizeof(buf)-1] = '\0';
-  }
+  else
+    strlcpy(buf, charset, sizeof(buf));
   return buf;
 }
 
@@ -112,8 +105,7 @@ intlconv_utf8( char *dst, size_t dst_size,
   ssize_t res;
 
   if (dst_charset_id == NULL) {
-    strncpy(dst, src_utf8, dst_size);
-    dst[dst_size - 1] = '\0';
+    strlcpy(dst, src_utf8, dst_size);
     return strlen(dst);
   }
   templ.ic_charset_id = (char *)dst_charset_id;
@@ -201,8 +193,7 @@ intlconv_to_utf8( char *dst, size_t dst_size,
   ssize_t res;
 
   if (src_charset_id == NULL) {
-    strncpy(dst, src, dst_size);
-    dst[dst_size - 1] = '\0';
+    strlcpy(dst, src, dst_size);
     return strlen(dst);
   }
   templ.ic_charset_id = (char *)src_charset_id;
@@ -250,6 +241,27 @@ found:
   return res;
 }
 
+char *
+intlconv_to_utf8safestr( const char *src_charset_id,
+                         const char *src_str,
+                         size_t max_size )
+{
+  char *str;
+  ssize_t r;
+
+  if (max_size == 0 || *src_str == '\0')
+    return strdup("");
+
+  str = alloca(max_size);
+  r = intlconv_to_utf8(str, max_size, src_charset_id, src_str, strlen(src_str));
+  if (r <= 0)
+    return NULL;
+  if (r >= max_size)
+    r--;
+  str[r++] = '\0';
+  return strdup(str);
+}
+
 /*
  *
  */
