@@ -21,7 +21,9 @@
 
 #include "htsmsg.h"
 
-void notify_by_msg(const char *class, htsmsg_t *m);
+#define NOTIFY_REWRITE_SUBSCRIPTIONS 1
+
+void notify_by_msg(const char *class, htsmsg_t *m, int rewrite);
 
 void notify_reload(const char *class);
 
