@@ -1,6 +1,6 @@
 /*
  *  Teletext parsing functions
- *  Copyright (C) 2007 Andreas Öman
+ *  Copyright (C) 2007 Andreas man
  *  Copyright (C) 2014 Jaroslav Kysela
  *
  *  This program is free software: you can redistribute it and/or modify
@@ -39,6 +39,8 @@
 #include "input.h"
 #include "parser_teletext.h"
 
+#define MAX_SUB_TEXT_SIZE 2000
+
 /**
  *
  */
@@ -48,6 +50,8 @@ typedef struct tt_mag {
   uint8_t ttm_charset[2];
   uint8_t ttm_national;
   uint8_t ttm_page[23*40 + 1];
+  int64_t ttm_last_sub_pts;
+  uint8_t ttm_last_sub_text[MAX_SUB_TEXT_SIZE];
 } tt_mag_t;
 
 
@@ -619,7 +623,7 @@ tt_construct_unix_time(uint8_t *buf)
   int i;
   struct tm tm;
 
-  t = dispatch_clock;
+  t = gclk();
   localtime_r(&t, &tm);
 
   tm.tm_hour = atoi((char *)buf);
@@ -694,13 +698,13 @@ get_cset(uint8_t off)
   return char_table[off];
 }
 
-static void
+static int
 extract_subtitle(mpegts_service_t *t, elementary_stream_t *st,
 		 tt_mag_t *ttm, int64_t pts)
 {
   int i, j, start, off = 0;
   int k, current_color, is_font_tag_open, use_color_subs = 1;
-  uint8_t sub[2000];
+  uint8_t sub[MAX_SUB_TEXT_SIZE];
   uint8_t *cset, ch;
   int is_box = 0;
 
@@ -722,7 +726,7 @@ extract_subtitle(mpegts_service_t *t, elementary_stream_t *st,
           if (ch != current_color) {
             if (is_font_tag_open) {
               for (k = 0; k < 7; k++) {
-                if (off < sizeof(sub))
+                if (off < sizeof(sub) - 1)
                   sub[off++] = CLOSE_FONT[k];
               }
               is_font_tag_open = 0;
@@ -730,7 +734,7 @@ extract_subtitle(mpegts_service_t *t, elementary_stream_t *st,
             /* no need for font-tag for default-color */
             if (ch != 7) {
               for (k = 0; k < 22; k++) {
-                if (off < sizeof(sub))
+                if (off < sizeof(sub) - 1)
                   sub[off++] = COLOR_MAP[ch][k];
               }
               is_font_tag_open = 1;
@@ -757,16 +761,16 @@ extract_subtitle(mpegts_service_t *t, elementary_stream_t *st,
           if (ucs2 == 0) {
             /* nothing */
           } else if (ucs2 < 0x80) {
-            if (off < sizeof(sub))
+            if (off < sizeof(sub) - 1)
               sub[off++] = ucs2;
           } else if (ucs2 < 0x800) {
-            if (off + 1 < sizeof(sub)) {
+            if (off + 1 < sizeof(sub) - 1) {
               sub[off++] = (ucs2 >> 6)   | 0xc0;
               sub[off++] = (ucs2 & 0x3f) | 0x80;
             }
           } else {
             assert(ucs2 < 0xd800 || ucs2 >= 0xe000);
-            if (off + 2 < sizeof(sub)) {
+            if (off + 2 < sizeof(sub) - 1) {
               sub[off++] =  (ucs2 >> 12)        | 0xe0;
               sub[off++] = ((ucs2 >> 6) & 0x3f) | 0x80;
               sub[off++] =  (ucs2       & 0x3f) | 0x80;
@@ -777,34 +781,39 @@ extract_subtitle(mpegts_service_t *t, elementary_stream_t *st,
     }
     if (use_color_subs && is_font_tag_open) {
       for (k = 0; k < 7; k++) {
-        if (off < sizeof(sub))
+        if (off < sizeof(sub) - 1)
           sub[off++] = CLOSE_FONT[k];
       }
       is_font_tag_open = 0;
     }
-    if(start != off && off < sizeof(sub))
+    if(start != off && off < sizeof(sub) - 1)
       sub[off++] = '\n';
   }
 
+  /* Avoid multiple blank subtitles */
   if(off == 0 && st->es_blank)
-    return; // Avoid multiple blank subtitles
+    return 0;
 
   st->es_blank = !off;
 
-  if(st->es_curpts == pts)
-    pts++; // Avoid non-monotonic PTS
-
-  st->es_curpts = pts;
-
   sub[off++] = 0;
-  
-  th_pkt_t *pkt = pkt_alloc(sub, off, pts, pts);
+
+  /* Check for a duplicate */
+  if (strcmp((char *)ttm->ttm_last_sub_text, (char *)sub) == 0)
+    return 0;
+
+  ttm->ttm_last_sub_pts = pts;
+  strcpy((char *)ttm->ttm_last_sub_text, (char *)sub);
+
+  th_pkt_t *pkt = pkt_alloc(st->es_type, sub, off, pts, pts);
   pkt->pkt_componentindex = st->es_index;
 
   streaming_pad_deliver(&t->s_streaming_pad, streaming_msg_create_pkt(pkt));
 
   /* Decrease our own reference to the packet */
   pkt_ref_dec(pkt);
+
+  return 1;
 }
 
 /**
@@ -849,7 +858,8 @@ tt_subtitle_deliver(mpegts_service_t *t, elementary_stream_t *parent, tt_mag_t *
   TAILQ_FOREACH(st, &t->s_components, es_link) {
      if(parent->es_pid == st->es_parent_pid &&
 	ttm->ttm_curpage == st->es_pid -  PID_TELETEXT_BASE) {
-       extract_subtitle(t, st, ttm, ttm->ttm_current_pts);
+       if (extract_subtitle(t, st, ttm, ttm->ttm_current_pts))
+         ttm->ttm_current_pts++; // Avoid duplicate (non-monotonic) PTS
      }
   }
 }
@@ -946,21 +956,21 @@ tt_decode_line(mpegts_service_t *t, elementary_stream_t *st, uint8_t *buf)
  *
  */
 void
-teletext_input(mpegts_service_t *t, elementary_stream_t *st, const uint8_t *tsb)
+teletext_input
+  (mpegts_service_t *t, elementary_stream_t *st, const uint8_t *data, int len)
 {
-  int i, j;
-  const uint8_t *x;
+  int j;
   uint8_t buf[42];
 
-  x = tsb + 4;
-  for(i = 0; i < 4; i++) {
-    if(*x == 2 || *x == 3) {
+  data++; len--; /* first byte: 0x10 ? */
+
+  for (; len >= 46; data += 46, len -= 46) {
+    if (*data == 2 || *data == 3) {
       for(j = 0; j < 42; j++)
-	buf[j] = bitreverse(x[4 + j]);
+	buf[j] = bitreverse(data[4 + j]);
       tt_decode_line(t, st, buf);
     }
-    x += 46;
-  }
+ }
 }
 
 
