@@ -22,6 +22,7 @@
 
 typedef struct transcoder_prop {
   char     tp_vcodec[32];
+  char     tp_vcodec_preset[32];
   char     tp_acodec[32];
   char     tp_scodec[32];
 
@@ -32,6 +33,7 @@ typedef struct transcoder_prop {
   int32_t  tp_resolution;
 
   long     tp_nrprocessors;
+  char     tp_src_vcodec[128];
 } transcoder_props_t;
 
 extern uint32_t transcoding_enabled;
@@ -40,7 +42,7 @@ streaming_target_t *transcoder_create (streaming_target_t *output);
 void                transcoder_destroy(streaming_target_t *tr);
 
 htsmsg_t *transcoder_get_capabilities(int experimental);
-void transcoder_set_properties  (streaming_target_t *tr, 
+void transcoder_set_properties  (streaming_target_t *tr,
 				 transcoder_props_t *prop);
 
 
