@@ -31,6 +31,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <netinet/in.h>
+#include <netinet/ip.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 
@@ -39,11 +40,36 @@
 #include "tvhpoll.h"
 #include "notify.h"
 #include "access.h"
+#include "dvr/dvr.h"
+#define COMPAT_IPTOS
+#include "compat.h"
+
+#if ENABLE_LIBSYSTEMD_DAEMON
+#include <systemd/sd-daemon.h>
+#endif
 
 int tcp_preferred_address_family = AF_INET;
 int tcp_server_running;
 th_pipe_t tcp_server_pipe;
 
+/**
+ *
+ */
+int
+socket_set_dscp(int sockfd, uint32_t dscp, char *errbuf, size_t errbufsize)
+{
+  int r, v;
+
+  v = dscp & IPTOS_DSCP_MASK;
+  r = setsockopt(sockfd, IPPROTO_IP, IP_TOS, &v, sizeof(v));
+  if (r < 0) {
+    if (errbuf && errbufsize)
+      snprintf(errbuf, errbufsize, "IP_TOS failed: %s", strerror(errno));
+    return -1;
+  }
+  return 0;
+}
+
 /**
  *
  */
@@ -51,25 +77,56 @@ int
 tcp_connect(const char *hostname, int port, const char *bindaddr,
             char *errbuf, size_t errbufsize, int timeout)
 {
-  int fd, r, res, err;
-  struct addrinfo *ai;
+  int fd = -1, r, res, err;
+  struct addrinfo *ai, *rai = NULL, hints;
+  struct sockaddr_storage bindip;
   char portstr[6];
   socklen_t errlen = sizeof(err);
 
+  errbuf[0] = '\0';
+
+  memset(&bindip, 0, sizeof(bindip));
+  bindip.ss_family = AF_UNSPEC;
+  if (bindaddr && bindaddr[0] != '\0') {
+    if (tcp_get_ip_from_str(bindaddr, &bindip) == NULL) {
+      snprintf(errbuf, errbufsize, "Cannot bind to addr '%s'", bindaddr);
+      return -1;
+    }
+  }
+
   snprintf(portstr, 6, "%u", port);
-  res = getaddrinfo(hostname, portstr, NULL, &ai);
-  
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = AF_UNSPEC;
+  res = getaddrinfo(hostname, portstr, &hints, &ai);
   if (res != 0) {
     snprintf(errbuf, errbufsize, "%s", gai_strerror(res));
     return -1;
   }
 
-  fd = tvh_socket(ai->ai_family, SOCK_STREAM, 0);
+again:
+  if (fd >= 0) {
+    close(fd);
+    fd = -1;
+  }
+  rai = rai == NULL ? ai : rai->ai_next;
+  if (rai == NULL) {
+    if (errbuf[0] == '\0')
+      snprintf(errbuf, errbufsize, "Invalid or unresolved hostname '%s'", hostname);
+    goto error;
+  }
+
+  if (bindip.ss_family == AF_UNSPEC) {
+    if (rai->ai_family != AF_INET && rai->ai_family != AF_INET6)
+      goto again;
+  } else if (rai->ai_family != bindip.ss_family) {
+    goto again;
+  }
+
+  fd = tvh_socket(rai->ai_family, SOCK_STREAM, 0);
   if(fd < 0) {
     snprintf(errbuf, errbufsize, "Unable to create socket: %s",
 	     strerror(errno));
-    freeaddrinfo(ai);
-    return -1;
+    goto again;
   }
 
   /**
@@ -77,29 +134,17 @@ tcp_connect(const char *hostname, int port, const char *bindaddr,
    */
   fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) | O_NONBLOCK);
 
-  if (ai->ai_family == AF_INET || ai->ai_family == AF_INET6) {
-    if (bindaddr && bindaddr[0] != '\0') {
-      struct sockaddr_storage ip;
-      memset(&ip, 0, sizeof(ip));
-      ip.ss_family = ai->ai_family;
-      if (inet_pton(AF_INET, bindaddr, IP_IN_ADDR(ip)) <= 0 ||
-          bind(fd, (struct sockaddr *)&ip, IP_IN_ADDRLEN(ip)) < 0) {
-        snprintf(errbuf, errbufsize, "Cannot bind to IPv%s addr '%s'",
-                                     ai->ai_family == AF_INET6 ? "6" : "4",
-                                     bindaddr);
-        freeaddrinfo(ai);
-        close(fd);
-        return -1;
-      }
+  if (bindip.ss_family != AF_UNSPEC) {
+    if (bind(fd, (struct sockaddr *)&bindip, IP_IN_ADDRLEN(bindip)) < 0) {
+      snprintf(errbuf, errbufsize, "Cannot bind to IPv%s addr '%s:%i': %s",
+                                   bindip.ss_family == AF_INET6 ? "6" : "4",
+                                   bindaddr, htons(IP_PORT(bindip)),
+                                   strerror(errno));
+      goto error;
     }
-  } else {
-    snprintf(errbuf, errbufsize, "Invalid protocol family");
-    freeaddrinfo(ai);
-    return -1;
   }
 
-  r = connect(fd, ai->ai_addr, ai->ai_addrlen);
-  freeaddrinfo(ai);
+  r = connect(fd, rai->ai_addr, rai->ai_addrlen);
 
   if(r < 0) {
     /* timeout < 0 - do not wait at all */
@@ -118,14 +163,13 @@ tcp_connect(const char *hostname, int port, const char *bindaddr,
 
       /* minimal timeout is one second */
       if (timeout < 1)
-        timeout = 0;
+        timeout = 1;
 
       while (1) {
-        if (!tvheadend_running) {
+        if (!tvheadend_is_running()) {
           errbuf[0] = '\0';
           tvhpoll_destroy(efd);
-          close(fd);
-          return -1;
+          goto error;
         }
 
         r = tvhpoll_wait(efd, &ev, 1, timeout * 1000);
@@ -133,17 +177,15 @@ tcp_connect(const char *hostname, int port, const char *bindaddr,
           break;
         
         if (r == 0) { /* Timeout */
-          snprintf(errbuf, errbufsize, "Connection attempt timed out");
+          snprintf(errbuf, errbufsize, "Connection attempt to '%s' timed out", hostname);
           tvhpoll_destroy(efd);
-          close(fd);
-          return -1;
+          goto again;
         }
       
         if (!ERRNO_AGAIN(errno)) {
           snprintf(errbuf, errbufsize, "poll() error: %s", strerror(errno));
           tvhpoll_destroy(efd);
-          close(fd);
-          return -1;
+          goto error;
         }
       }
       
@@ -158,18 +200,23 @@ tcp_connect(const char *hostname, int port, const char *bindaddr,
 
   if(err != 0) {
     snprintf(errbuf, errbufsize, "%s", strerror(err));
-    close(fd);
-    return -1;
+    goto again;
   }
   
   fcntl(fd, F_SETFL, fcntl(fd, F_GETFL) & ~O_NONBLOCK);
 
-
   /* Set the keep-alive active */
   err = 1;
   setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, (void *)&err, errlen);
 
+  freeaddrinfo(ai);
   return fd;
+
+error:
+  if (fd >= 0)
+    close(fd);
+  freeaddrinfo(ai);
+  return -1;
 }
 
 
@@ -228,16 +275,18 @@ tcp_fill_htsbuf_from_fd(int fd, htsbuf_queue_t *hq)
   hd->hd_data_size = 1000;
   hd->hd_data = malloc(hd->hd_data_size);
 
-  c = read(fd, hd->hd_data, hd->hd_data_size);
-  if(c < 1) {
+  do {
+    r = read(fd, hd->hd_data, hd->hd_data_size);
+  } while (r < 0 && ERRNO_AGAIN(errno));
+  if(r < 1) {
     free(hd->hd_data);
     free(hd);
     return -1;
   }
-  hd->hd_data_len = c;
+  hd->hd_data_len = r;
   hd->hd_data_off = 0;
   TAILQ_INSERT_TAIL(&hq->hq_q, hd, hd_link);
-  hq->hq_size += c;
+  hq->hq_size += r;
   return 0;
 }
 
@@ -327,7 +376,7 @@ tcp_read_timeout(int fd, void *buf, size_t len, int timeout)
     if(x == 0)
       return ETIMEDOUT;
     if(x == -1) {
-      if (!tvheadend_running)
+      if (!tvheadend_is_running())
         return ECONNRESET;
       if (ERRNO_AGAIN(errno))
         continue;
@@ -354,26 +403,73 @@ tcp_read_timeout(int fd, void *buf, size_t len, int timeout)
 /**
  *
  */
-char *
-tcp_get_ip_str(const struct sockaddr_storage *sa, char *s, size_t maxlen)
+int
+tcp_socket_dead(int fd)
 {
-  if(sa == NULL || s == NULL)
+  int err = 0;
+  socklen_t errlen = sizeof(err);
+
+  if (getsockopt(fd, SOL_SOCKET, SO_ERROR, &err, &errlen))
+    return -errno;
+  if (err)
+    return -err;
+#ifdef PLATFORM_FREEBSD
+  if (recv(fd, NULL, 0, MSG_PEEK | MSG_DONTWAIT) < 0)
+    return -errno;
+#else
+  if (recv(fd, NULL, 0, MSG_PEEK | MSG_DONTWAIT) == 0)
+    return -EIO;
+#endif
+  return 0;
+}
+
+/**
+ *
+ */
+char *
+tcp_get_str_from_ip(const struct sockaddr_storage *sa, char *dst, size_t maxlen)
+{
+  if (sa == NULL || dst == NULL)
     return NULL;
 
   switch(sa->ss_family)
   {
     case AF_INET:
-      inet_ntop(AF_INET, &(((struct sockaddr_in*)sa)->sin_addr), s, maxlen);
+      inet_ntop(AF_INET, &(((struct sockaddr_in*)sa)->sin_addr), dst, maxlen);
       break;
     case AF_INET6:
-      inet_ntop(AF_INET6, &(((struct sockaddr_in6*)sa)->sin6_addr), s, maxlen);
+      inet_ntop(AF_INET6, &(((struct sockaddr_in6*)sa)->sin6_addr), dst, maxlen);
       break;
     default:
-      strncpy(s, "Unknown AF", maxlen);
+      strlcpy(dst, "Unknown AF", maxlen);
       return NULL;
   }
 
-  return s;
+  return dst;
+}
+
+/**
+ *
+ */
+struct sockaddr_storage *
+tcp_get_ip_from_str(const char *src, struct sockaddr_storage *sa)
+{
+  if (sa == NULL || src == NULL)
+    return NULL;
+
+  if (strstr(src, ":")) {
+    sa->ss_family = AF_INET6;
+    if (inet_pton(AF_INET6, src, &(((struct sockaddr_in6*)sa)->sin6_addr)) != 1)
+      return NULL;
+  } else if (strstr(src, ".")) {
+    sa->ss_family = AF_INET;
+    if (inet_pton(AF_INET, src, &(((struct sockaddr_in*)sa)->sin_addr)) != 1)
+      return NULL;
+  } else {
+    return NULL;
+  }
+
+  return sa;
 }
 
 /**
@@ -387,6 +483,7 @@ typedef struct tcp_server {
   struct sockaddr_storage bound;
   tcp_server_ops_t ops;
   void *opaque;
+  LIST_ENTRY(tcp_server) link;
 } tcp_server_t;
 
 typedef struct tcp_server_launch {
@@ -405,10 +502,31 @@ typedef struct tcp_server_launch {
   LIST_ENTRY(tcp_server_launch) jlink;
 } tcp_server_launch_t;
 
+static LIST_HEAD(, tcp_server) tcp_server_delete_list = { 0 };
 static LIST_HEAD(, tcp_server_launch) tcp_server_launches = { 0 };
 static LIST_HEAD(, tcp_server_launch) tcp_server_active = { 0 };
 static LIST_HEAD(, tcp_server_launch) tcp_server_join = { 0 };
 
+/**
+ *
+ */
+uint32_t
+tcp_connection_count(access_t *aa)
+{
+  tcp_server_launch_t *tsl;
+  uint32_t used = 0;
+
+  lock_assert(&global_lock);
+
+  if (aa == NULL)
+    return 0;
+
+  LIST_FOREACH(tsl, &tcp_server_active, alink)
+    if (!strcmp(aa->aa_representative ?: "", tsl->representative ?: ""))
+      used++;
+  return used;
+}
+
 /**
  *
  */
@@ -417,8 +535,9 @@ tcp_connection_launch
   (int fd, void (*status) (void *opaque, htsmsg_t *m), access_t *aa)
 {
   tcp_server_launch_t *tsl, *res;
-  uint32_t used = 0;
-  time_t started = dispatch_clock;
+  uint32_t used, used2;
+  int64_t started = mclk();
+  int c1, c2;
 
   lock_assert(&global_lock);
 
@@ -429,10 +548,11 @@ tcp_connection_launch
 
 try_again:
   res = NULL;
+  used = 0;
   LIST_FOREACH(tsl, &tcp_server_active, alink) {
     if (tsl->fd == fd) {
       res = tsl;
-      if (!aa->aa_conn_limit)
+      if (!aa->aa_conn_limit && !aa->aa_conn_limit_streaming)
         break;
       continue;
     }
@@ -442,18 +562,28 @@ try_again:
   if (res == NULL)
     return NULL;
 
-  if (aa->aa_conn_limit && used >= aa->aa_conn_limit) {
-    if (started + 3 < dispatch_clock) {
-      tvherror("tcp", "multiple connections are not allowed for user '%s' from '%s' (limit %u)",
-               aa->aa_username ?: "", aa->aa_representative ?: "", aa->aa_conn_limit);
+  if (aa->aa_conn_limit || aa->aa_conn_limit_streaming) {
+    used2 = aa->aa_conn_limit ? dvr_usage_count(aa) : 0;
+    /* the rule is: allow if one condition is OK */
+    c1 = aa->aa_conn_limit ? used + used2 >= aa->aa_conn_limit : -1;
+    c2 = aa->aa_conn_limit_streaming ? used >= aa->aa_conn_limit_streaming : -1;
+
+    if (c1 && c2) {
+      if (started + sec2mono(3) < mclk()) {
+        tvherror(LS_TCP, "multiple connections are not allowed for user '%s' from '%s' "
+                        "(limit %u, streaming limit %u, active streaming %u, DVR %u)",
+                 aa->aa_username ?: "", aa->aa_representative ?: "",
+                 aa->aa_conn_limit, aa->aa_conn_limit_streaming,
+                 used, used2);
+        return NULL;
+      }
+      pthread_mutex_unlock(&global_lock);
+      tvh_safe_usleep(250000);
+      pthread_mutex_lock(&global_lock);
+      if (tvheadend_is_running())
+        goto try_again;
       return NULL;
     }
-    pthread_mutex_unlock(&global_lock);
-    usleep(250000);
-    pthread_mutex_lock(&global_lock);
-    if (tvheadend_running)
-      goto try_again;
-    return NULL;
   }
 
   res->representative = aa->aa_representative ? strdup(aa->aa_representative) : NULL;
@@ -549,7 +679,8 @@ tcp_server_start(void *aux)
   LIST_REMOVE(tsl, alink);
   LIST_INSERT_HEAD(&tcp_server_join, tsl, jlink);
   pthread_mutex_unlock(&global_lock);
-  tvh_write(tcp_server_pipe.wr, &c, 1);
+  if (atomic_get(&tcp_server_running))
+    tvh_write(tcp_server_pipe.wr, &c, 1);
   return NULL;
 }
 
@@ -567,10 +698,12 @@ tcp_server_loop(void *aux)
   socklen_t slen;
   char c;
 
-  while(tcp_server_running) {
+  while(atomic_get(&tcp_server_running)) {
     r = tvhpoll_wait(tcp_server_poll, &ev, 1, -1);
-    if(r == -1) {
-      perror("tcp_server: tvhpoll_wait");
+    if(r < 0) {
+      if (ERRNO_AGAIN(errno))
+        continue;
+      tvherror(LS_TCP, "tcp_server_loop: tvhpoll_wait: %s", strerror(errno));
       continue;
     }
 
@@ -588,6 +721,10 @@ next:
           free(tsl);
           goto next;
         }
+        while ((ts = LIST_FIRST(&tcp_server_delete_list)) != NULL) {
+          LIST_REMOVE(ts, link);
+          free(ts);
+        }
         pthread_mutex_unlock(&global_lock);
       }
       continue;
@@ -628,25 +765,31 @@ next:
       pthread_mutex_lock(&global_lock);
       LIST_INSERT_HEAD(&tcp_server_active, tsl, alink);
       pthread_mutex_unlock(&global_lock);
-      tvhthread_create(&tsl->tid, NULL, tcp_server_start, tsl);
+      tvhthread_create(&tsl->tid, NULL, tcp_server_start, tsl, "tcp-start");
     }
   }
-  tvhtrace("tcp", "server thread finished");
+  tvhtrace(LS_TCP, "server thread finished");
   return NULL;
 }
 
+
+
 /**
  *
  */
-void *
-tcp_server_create
-  (const char *bindaddr, int port, tcp_server_ops_t *ops, void *opaque)
+#if ENABLE_LIBSYSTEMD_DAEMON
+static void *tcp_server_create_new
+#else
+void *tcp_server_create
+#endif
+  (int subsystem, const char *name, const char *bindaddr,
+   int port, tcp_server_ops_t *ops, void *opaque)
 {
   int fd, x;
   tcp_server_t *ts;
   struct addrinfo hints, *res, *ressave, *use = NULL;
   struct sockaddr_storage bound;
-  char port_buf[6];
+  char port_buf[6], buf[50];
   int one = 1;
   int zero = 0;
 
@@ -662,8 +805,8 @@ tcp_server_create
   x = getaddrinfo(bindaddr, port_buf, &hints, &res);
 
   if(x != 0) {
-    tvhlog(LOG_ERR, "tcp", "getaddrinfo: %s: %s", bindaddr != NULL ? bindaddr : "*",
-      x == EAI_SYSTEM ? strerror(errno) : gai_strerror(x));
+    tvherror(LS_TCP, "getaddrinfo: %s: %s", bindaddr != NULL ? bindaddr : "*",
+             x == EAI_SYSTEM ? strerror(errno) : gai_strerror(x));
     return NULL;
   }
 
@@ -698,7 +841,7 @@ tcp_server_create
 
   if(x != 0)
   {
-    tvhlog(LOG_ERR, "tcp", "bind: %s:%i: %s", bindaddr != NULL ? bindaddr : "*", port, strerror(errno));
+    tvherror(LS_TCP, "bind: %s:%i: %s", bindaddr != NULL ? bindaddr : "*", port, strerror(errno));
     close(fd);
     return NULL;
   }
@@ -710,14 +853,88 @@ tcp_server_create
   ts->bound  = bound;
   ts->ops    = *ops;
   ts->opaque = opaque;
+
+  tcp_get_str_from_ip(&bound, buf, sizeof(buf));
+  tvhinfo(subsystem, "Starting %s server %s:%d", name, buf, htons(IP_PORT(bound)));
+
   return ts;
 }
 
+#if ENABLE_LIBSYSTEMD_DAEMON
+/**
+ *
+ */
+void *
+tcp_server_create
+  (int subsystem, const char *name, const char *bindaddr,
+   int port, tcp_server_ops_t *ops, void *opaque)
+{
+  int sd_fds_num, i, fd;
+  struct sockaddr_storage bound;
+  tcp_server_t *ts;
+  struct in_addr addr4;
+  struct in6_addr addr6;
+  char buf[50];
+  int found = 0;
+
+  sd_fds_num = sd_listen_fds(0);
+  inet_pton(AF_INET, bindaddr ?: "0.0.0.0", &addr4);
+  inet_pton(AF_INET6, bindaddr ?: "::", &addr6);
+
+  for (i = 0; i < sd_fds_num && !found; i++) {
+    struct sockaddr_in *s_addr4;
+    struct sockaddr_in6 *s_addr6;
+    socklen_t s_len;
+
+    /* Check which of the systemd-managed descriptors
+     * corresponds to the requested server (if any) */
+    fd = SD_LISTEN_FDS_START + i;
+    memset(&bound, 0, sizeof(bound));
+    s_len = sizeof(bound);
+    if (getsockname(fd, (struct sockaddr *) &bound, &s_len) != 0) {
+      tvherror(LS_TCP, "getsockname failed: %s", strerror(errno));
+      continue;
+    }
+    switch (bound.ss_family) {
+      case AF_INET:
+        s_addr4 = (struct sockaddr_in *) &bound;
+        if (addr4.s_addr == s_addr4->sin_addr.s_addr
+            && htons(port) == s_addr4->sin_port)
+          found = 1;
+        break;
+      case AF_INET6:
+        s_addr6 = (struct sockaddr_in6 *) &bound;
+        if (memcmp(addr6.s6_addr, s_addr6->sin6_addr.s6_addr, 16) == 0
+            && htons(port) == s_addr6->sin6_port)
+          found = 1;
+        break;
+      default:
+        break;
+    }
+  }
+
+  if (found) {
+    /* use the systemd provided socket */
+    ts = malloc(sizeof(tcp_server_t));
+    ts->serverfd = fd;
+    ts->bound  = bound;
+    ts->ops    = *ops;
+    ts->opaque = opaque;
+    tcp_get_str_from_ip(&bound, buf, sizeof(buf));
+    tvhinfo(subsystem, "Starting %s server %s:%d (systemd)", name, buf, htons(IP_PORT(bound)));
+  } else {
+    /* no systemd-managed socket found, create a new one */
+    tvhinfo(LS_TCP, "No systemd socket: creating a new one");
+    ts = tcp_server_create_new(subsystem, name, bindaddr, port, ops, opaque);
+  }
+
+  return ts;
+}
+#endif
 
 /**
  *
  */
-
 void tcp_server_register(void *server)
 {
   tcp_server_t *ts = server;
@@ -742,6 +959,7 @@ tcp_server_delete(void *server)
 {
   tcp_server_t *ts = server;
   tvhpoll_event_t ev;
+  char c = 'D';
 
   if (server == NULL)
     return;
@@ -750,8 +968,11 @@ tcp_server_delete(void *server)
   ev.fd       = ts->serverfd;
   ev.events   = TVHPOLL_IN;
   ev.data.ptr = ts;
-  tvhpoll_rem(tcp_server_poll, &ev, 1);  
-  free(ts);
+  tvhpoll_rem(tcp_server_poll, &ev, 1);
+  close(ts->serverfd);
+  ts->serverfd = -1;
+  LIST_INSERT_HEAD(&tcp_server_delete_list, ts, link);
+  tvh_write(tcp_server_pipe.wr, &c, 1);
 }
 
 /**
@@ -839,6 +1060,26 @@ tcp_server_bound ( void *server, struct sockaddr_storage *bound, int family )
   return 0;
 }
 
+/**
+ *
+ */
+int
+tcp_server_onall ( void *server )
+{
+  tcp_server_t *ts = server;
+  int i, len;
+  uint8_t *ptr;
+
+  if (server == NULL) return 0;
+
+  len = IP_IN_ADDRLEN(ts->bound);
+  ptr = (uint8_t *)IP_IN_ADDR(ts->bound);
+  for (i = 0; i < len; i++)
+    if (ptr[0])
+      break;
+  return i >= len;
+}
+
 /*
  * Connections status
  */
@@ -857,9 +1098,13 @@ tcp_server_connections ( void )
     if (!tsl->status) continue;
     c++;
     e = htsmsg_create_map();
-    tcp_get_ip_str(&tsl->peer, buf, sizeof(buf));
     htsmsg_add_u32(e, "id", tsl->id);
+    tcp_get_str_from_ip(&tsl->self, buf, sizeof(buf));
+    htsmsg_add_str(e, "server", buf);
+    htsmsg_add_u32(e, "server_port", ntohs(IP_PORT(tsl->self)));
+    tcp_get_str_from_ip(&tsl->peer, buf, sizeof(buf));
     htsmsg_add_str(e, "peer", buf);
+    htsmsg_add_u32(e, "peer_port", ntohs(IP_PORT(tsl->peer)));
     htsmsg_add_s64(e, "started", tsl->started);
     tsl->status(tsl->opaque, e);
     htsmsg_add_msg(l, NULL, e);
@@ -897,18 +1142,19 @@ tcp_server_init(void)
   ev.data.ptr = &tcp_server_pipe;
   tvhpoll_add(tcp_server_poll, &ev, 1);
 
-  tcp_server_running = 1;
-  tvhthread_create(&tcp_server_tid, NULL, tcp_server_loop, NULL);
+  atomic_set(&tcp_server_running, 1);
+  tvhthread_create(&tcp_server_tid, NULL, tcp_server_loop, NULL, "tcp-loop");
 }
 
 void
 tcp_server_done(void)
 {
+  tcp_server_t *ts;
   tcp_server_launch_t *tsl;  
   char c = 'E';
   int64_t t;
 
-  tcp_server_running = 0;
+  atomic_set(&tcp_server_running, 0);
   tvh_write(tcp_server_pipe.wr, &c, 1);
 
   pthread_mutex_lock(&global_lock);
@@ -925,14 +1171,15 @@ tcp_server_done(void)
   tvh_pipe_close(&tcp_server_pipe);
   tvhpoll_destroy(tcp_server_poll);
   
-  t = getmonoclock();
-  while (LIST_FIRST(&tcp_server_active) != NULL) {
-    if (getmonoclock() - t > 5000000)
-      tvhtrace("tcp", "tcp server %p active too long", LIST_FIRST(&tcp_server_active));
-    usleep(20000);
-  }
-
   pthread_mutex_lock(&global_lock);
+  t = mclk();
+  while (LIST_FIRST(&tcp_server_active) != NULL) {
+    if (t + sec2mono(5) < mclk())
+      tvhtrace(LS_TCP, "tcp server %p active too long", LIST_FIRST(&tcp_server_active));
+    pthread_mutex_unlock(&global_lock);
+    tvh_safe_usleep(20000);
+    pthread_mutex_lock(&global_lock);
+  }
   while ((tsl = LIST_FIRST(&tcp_server_join)) != NULL) {
     LIST_REMOVE(tsl, jlink);
     pthread_mutex_unlock(&global_lock);
@@ -940,5 +1187,9 @@ tcp_server_done(void)
     free(tsl);
     pthread_mutex_lock(&global_lock);
   }
+  while ((ts = LIST_FIRST(&tcp_server_delete_list)) != NULL) {
+    LIST_REMOVE(ts, link);
+    free(ts);
+  }
   pthread_mutex_unlock(&global_lock);
 }
